package ns;

import java.io.File;
import java.util.logging.Logger;

public class MacApp {
    private static Logger log = Logger.getLogger(MacApp.class.getName());
    private final static boolean mac=(System.getProperty("os.name").toLowerCase().startsWith("mac os x"));

    public static boolean isMac(){
        return mac;
    }
    public static class CouldntSetupMacException extends Exception{
        public CouldntSetupMacException(Throwable cause) {
            super(cause);
        }
    }
    public MacApp(MacImplementation mi) throws CouldntSetupMacException {
        //macOSXRegistration(mi);
        //sanity check
        if(!isMac()) return;
        log.info("registering mac app");
        try {
            Class c = Class.forName("uk.co.hcsoftware.mac.MacApplication");
            Object o = c.newInstance();
            c.getMethod("register",new Class[]{MacImplementation.class}).invoke(o,mi);
        } catch (Exception e) {
            //log.log(Level.WARNING,"Exception",e);
            throw new CouldntSetupMacException(e);
        }
    }
    public interface MacImplementation{
        public boolean hasPrefsDialog();
        // General info dialog.  The OSXAdapter calls this method when "About OSXAdapter"
        // is selected from the application menu.
        public void about() ;

        // General preferences dialog.  The OSXAdapter calls this method when "Preferences..."
        // is selected from the application menu.
        public void preferences();
        public void openFile(File f);
        // General info dialog.  The OSXAdapter calls this method when "Quit OSXAdapter"
        // is selected from the application menu, Cmd-Q is pressed, or "Quit" is selected from the Dock.

        public boolean quit() ;
        public void printFile(File f);
        public void reOpenApplication();
        public void openApplication();
    }
}