package ns.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyAdapter;
import java.awt.event.MouseAdapter;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * Created by IntelliJ IDEA.
 * User: spurln
 * Date: 11-Mar-2004
 * Time: 15:27:23
 * To change this template use Options | File Templates.
 */

/**
 * This is the 3rd version of SwingWorker (also known as
 * SwingWorker 3), an abstract class that you subclass to
 * perform GUI-related work in a dedicated thread.  For
 * instructions on using this class, see:
 * <p/>
 * http://java.sun.com/docs/books/tutorial/uiswing/misc/threads.html
 * <p/>
 * Note that the API changed slightly in the 3rd version:
 * You must now invoke start() on the SwingWorker after
 * creating it.
 */
public abstract class SwingWorker3 {
    private static final Logger log = Logger.getLogger(SwingWorker3.class.getName());
    private Object value;

    /**
     * Class to maintain reference to current worker thread
     * under separate synchronization control.
     */
    private static class ThreadVar {
        private Thread thread;

        ThreadVar(Thread t) {
            thread = t;
        }

        synchronized Thread get() {
            return thread;
        }

        synchronized void clear() {
            thread = null;
        }
    }

    private ThreadVar threadVar;

    /**
     * Get the value produced by the worker thread, or null if it
     * hasn't been constructed yet.
     */
    protected synchronized Object getValue() {
        return value;
    }

    /**
     * Set the value produced by worker thread
     */
    private synchronized void setValue(Object x) {
        value = x;
    }

    /**
     * Compute the value to be returned by the <code>get</code> method. happens off edt,
     * override this to do lengthy processing
     */
    public abstract Object construct();


    /**
     * A new method that interrupts the worker thread.  Call this method
     * to force the worker to stop what it's doing.
     */
    public void interrupt() {
        Thread t = threadVar.get();
        if (t != null) {
            t.interrupt();
        }
        threadVar.clear();
    }

    /**
     * Return the value created by the <code>construct</code> method.
     * Returns null if either the constructing thread or the current
     * thread was interrupted before a value was produced.
     *
     * @return the value created by the <code>construct</code> method
     */
    public Object get() {
        while (true) {
            Thread t = threadVar.get();
            if (t == null) {
                return getValue();
            }
            try {
                t.join();
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt(); // propagate
                return null;
            }
        }
    }


    /**
     * Start a thread that will call the <code>construct</code> method
     * and then exit.
     */
    public SwingWorker3() {
        final Runnable doFinished = new Runnable() {
            public void run() {
                finished();
            }
        };
        final Runnable doRestoreCursor = new Runnable() {
            public void run() {
                restoreCursor();
            }
        };

        Runnable doConstruct = new Runnable() {
            public void run() {
                try {
                    setValue(construct());
                } finally {
                    threadVar.clear();
                }
                SwingUtilities.invokeLater(doRestoreCursor);
                SwingUtilities.invokeLater(doFinished);
            }
        };

        Thread t = new Thread(doConstruct);
        threadVar = new ThreadVar(t);
    }
    private Window progressDialog;
    private boolean disposeOnFinish;
    /**
     * only works well if the progressDialog is modal
     * @param progressDialog
     * @param disposeOnFinish
     */
    public SwingWorker3(Window progressDialog,boolean disposeOnFinish) {
        this(progressDialog);
        this.progressDialog=progressDialog;
        this.disposeOnFinish=disposeOnFinish;
    }
    /* public SwingWorker3(JFrame jf, Window progressDialog, boolean disposeOnFinish) {
        this(jf);
        this.progressDialog=progressDialog;
        this.disposeOnFinish=disposeOnFinish;
    }*/
    public SwingWorker3(Component c) {
        this();
        this.gp = c;
        this.previousComponentCursor = c.getCursor();
    }

    private boolean setglasspane;

    /**
     * create normal swingworker, but add wait cursor to component for duration of construct()
     *
     * @param c component for which cursor will be set to WAIT_CURSOR (will be reset afterwards)
     */
    public SwingWorker3(JFrame c, boolean setglasspane) {
        this(c);
        this.setglasspane = setglasspane;
    }
    private Cursor previousComponentCursor;

    /**
     * Start the worker thread.
     */
    public void start() {
        if (gp != null) {
            log.fine("sw3 setting wait cursor");
            if (setglasspane) {
                JPanel glassPane = new JPanel();
                glassPane.setOpaque(false);
                glassPane.addMouseListener(new MouseAdapter(){});
                glassPane.addKeyListener(new KeyAdapter(){});
                log.fine("glasspane set visible");
                ((JFrame)gp).setGlassPane(glassPane);
                gp=glassPane;
                gp.setVisible(true);
            }
            gp.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        }
        Thread t = threadVar.get();
        if (t != null) {
            t.start();
        }
        if(progressDialog!=null) progressDialog.setVisible(true);
    }

    /**
     * override these to perfrom actions on edt when construct has completed
     * NB: EDT is not blocked until this runs, this is scheduled using invokeLater after any EDT blocking (e.g. modal dialog)
     * has been removed. If you want to use a variable (or whatever) after swingworker3 has finished, initialise it in construct()
     * method not here.
     */
    public void finished() {
        ;
    }

    /**
     * Called on the event dispatching thread (not on the worker thread)
     * after the <code>construct</code> method has returned.
     */
    public void restoreCursor() {
        try {
            log.fine("finished called in sw gp=null=" + (gp == null));
            if (gp != null) {
                if (setglasspane) {
                    log.fine("glasspane set visible false");
                    gp.setVisible(false);
                }
                gp.setCursor(previousComponentCursor);
                log.fine("set cursor to original");
            }
            if(progressDialog!=null && disposeOnFinish){
                progressDialog.dispose();
            }
        } catch (Exception e) {
            log.log(Level.WARNING,"",e);
        }
    }

    Component gp;
}