package uk.co.hcsoftware.commander;

import uk.co.hcsoftware.commander.treeplugin.TreePlugin;
import uk.co.hcsoftware.commander.treeplugin.zip.ZipTreePlugin;
import uk.co.hcsoftware.commander.treeplugin.file.FileTreePlugin;
import uk.co.hcsoftware.commander.treeplugin.file.Vfile;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * todo: sort out url, it's not really a URL. Instead make it an object with stringify() unstringify() methods that always work or fail
 * 
 *
 * Title: Url
 * Copyright:    Copyright (c) 28-Oct-2005
 *
 * @author spurlinn
 */
public abstract class Url {
    public static final Logger log = Logger.getLogger(Url.class.getName());

    public abstract String getFriendlySpec();
    public abstract String stringify();

    public Vfile getVfile() throws CannotConvertUrlToFileException {
        return new Vfile(getFile());
    }
    public static boolean isRecommendedPlugin(TreePlugin tp, Url url){
        try {
            File f = url.getFile();
            if(f.isDirectory()){
                return tp instanceof FileTreePlugin;
            }
            else if (isZipFile(f)){
                return tp instanceof ZipTreePlugin;
            }
        } catch (CannotConvertUrlToFileException e) {
            log.log(Level.WARNING, "exception", e);
        }
        return false;

//        if(url instanceof StringUrl){
//            String s = ((StringUrl)url).getString();
//            if(s.startsWith("zip") || s.startsWith("jar")){
//                return tp instanceof ZipTreePlugin;
//            }
//        }
//        return tp instanceof FileTreePlugin;
    }

    private static boolean isZipFile(File f) {
        if(!f.isFile()) return false;
        final String name = f.getName().toLowerCase();
        return name.endsWith(".zip") || name.endsWith(".jar");
    }

    public static TreePlugin getPlugin(Url url) throws NoPluginAvailableException{
        try {
            File f = url.getFile();
            if(f.isDirectory()){
                return FileTreePlugin.createFileTreePlugin();
            }
            else if (isZipFile(f)){
                return new ZipTreePlugin();
            }
        } catch (CannotConvertUrlToFileException e) {
            log.log(Level.WARNING, "exception", e);
        }
        throw new NoPluginAvailableException("no plugin found",url);
//        if(url instanceof StringUrl){
//            String s = ((StringUrl)url).getString();
//            if(s.startsWith("zip") || s.startsWith("jar")){
//                return new ZipTreePlugin();
//                //todo: return zipplugin
//            }
//        }

    }


    public File getFile() throws CannotConvertUrlToFileException{
        if(this instanceof VfileUrl){
            return ((VfileUrl)this).file.getFile();
        }
        else if(this instanceof StringUrl){//must be a StringUrl
            String s = ((StringUrl)this).string;
            //remove everything after (and including) the ! character
            int i=s.indexOf('!');
            if(i>-1){
                s=s.substring(0,i);
            }
            if(s.startsWith("file:")){
                try {
                    URL u=new URL(s);
                    return new File(u.getFile());
                } catch (MalformedURLException e) {
                    throw new CannotConvertUrlToFileException(e,this);
                }
            }
            else{
                //log.fine("stringurl doesn't start with string: throwing exception s="+s);
                return new File(s);
            }
        }
        log.fine("unrecognized url type u="+this);
        //assume it's a filename of some sort

        throw new CannotConvertUrlToFileException(this);
    }

    public static Url getUrl(Vfile vf){
        return new VfileUrl(vf);
    }
    public static Url getUrl(File vf){
        return new VfileUrl(new Vfile(vf));
    }

    public static Url getUrl(String text) {
        return new StringUrl(text);
    }


    static class StringUrl extends Url{
        private String string;

        public StringUrl(String file) {
            this.string = file;
        }

        public String getString() {
            return string;
        }

        public String getFriendlySpec() {
            if(string.startsWith("file:")){
                return string.substring("file:".length()+1); //+1 for the leading /
            }
            return string;
            //return string.substring(string.lastIndexOf(':')+1);
        }

        public String stringify() {
            return string;
        }

        public String toString() {
            return "StringUrl string="+string;
        }
    }

    static class VfileUrl extends Url{
        private Vfile file;

        public VfileUrl(Vfile file) {
            this.file = file;
        }

        public String getFriendlySpec() {
            return file.getAbsolutePath();
        }

        public String stringify() {
            try {
                return file.toURL().toExternalForm();
            } catch (MalformedURLException e) {
                log.log(Level.WARNING, "exception", e);
                return file.getAbsolutePath();
            }
        }

        public String toString() {
            return "VfileUrl vf="+file;
        }
    }
}
