package uk.co.hcsoftware.commander.bookmarks;

import uk.co.hcsoftware.commander.Commandable;
import uk.co.hcsoftware.commander.Main;
import uk.co.hcsoftware.commander.NameComparator;
import uk.co.hcsoftware.commander.Url;
import uk.co.hcsoftware.commander.gui.CommanderFocus;
import uk.co.hcsoftware.commander.gui.ThemeListener;
import uk.co.hcsoftware.commander.gui.Themeable;
import uk.co.hcsoftware.commander.tree.*;
import uk.co.hcsoftware.commander.treeplugin.AutocompleteJumper;
import uk.co.hcsoftware.commander.treeplugin.file.Vfile;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


public class BookmarksBar extends JPanel implements Themeable, Commandable{
    private static final Logger log = Logger.getLogger(BookmarksBar.class.getName());

    private Opener opener;

    private static List<BookmarksBar> bars = new ArrayList<BookmarksBar>();
    private static DefaultTreeModel bookmarks;

    private static Url homeUrl;

    private static BookmarkReader br = BookmarkReader.getInstance();
    private CommanderTree ct;
    private final CtreeModel treeModel;// = ;
    private final CommanderFocus cf;
    private final JLabel dirName = new JLabel("/");

    private static Url tmpUrl;

    static{
        bookmarks=br.readBookmarks(true);
//        try {
            homeUrl=Url.getUrl(new File(System.getProperty("user.home")));
            tmpUrl=Url.getUrl(new File(System.getProperty("java.io.tmpdir")));
//        } catch (MalformedURLException e) {
//            log.log(Level.WARNING,"couldn't get home URL",e);
//        }
        log.fine("bookmarks="+getBookmarksPrintout(bookmarks));
    }
    private static String getBookmarksPrintout(TreeModel bm){
        StringBuffer sb = new StringBuffer();
        int level=0;
        appendNode(sb,(DefaultMutableTreeNode) bm.getRoot(),level);
        return sb.toString();
    }

    private static void appendNode(StringBuffer sb, DefaultMutableTreeNode root, int level) {
        for(int i=0;i<level;i++) sb.append(' ');
        sb.append((root).getUserObject());
        sb.append('\n');
        for(Enumeration e=root.children();e.hasMoreElements();){
            appendNode(sb,(DefaultMutableTreeNode) e.nextElement(),level+1);
        }
    }

    public BookmarksBar(Opener opener, CommanderFocus cf) {
        super(new BorderLayout(3,3));
        dirName.setHorizontalAlignment(JLabel.CENTER);
        this.cf = cf;
        final int width = 110;
        setPreferredSize(new Dimension(width,400));
        bars.add(this);

        this.opener = opener;
        JPanel homePanel = new JPanel(new GridLayout(2,1,2,2));//new FlowLayout(FlowLayout.LEFT,2,2));
        homePanel.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
        add(homePanel,BorderLayout.NORTH);
        homePanel.add(getHomeButton());
        homePanel.add(getTmpButton());
        treeModel =new TreeModelAdapter(bookmarks) {
            private NameComparator nc = new NameComparator();
            public int getAttributeCount() { return 1; }

            public boolean hasChildren(TreeNode treeNode) {
                Bookmark b = getBookmark(treeNode);
                return b.isDirectory();
            }
            public String getAttributeName(int index) {
                return "name";
            }
            public Object getAttributeValue(TreeNode treeNode, int attribute) {
                Bookmark b = getBookmark(treeNode);
                return b.getName();
            }
            public boolean isComparable(int attribute) {
                return true;
            }
            public int compare(TreeNode treeNode, TreeNode treeNode1, int attribute) {
                Bookmark b1 = getBookmark(treeNode),b2=getBookmark(treeNode1);
                return nc.compare(b1.getName(),b2.getName());
            }
            public SortDirection getDefaultSortDirection(int attribute) {
                return SortDirection.ascending;
            }
        };

        JPanel treePan = new JPanel(new BorderLayout(2,2));
        add(treePan);
        final CommanderTree ct = setupCtree(width);
        treePan.add(dirName,BorderLayout.NORTH);
        treePan.add(ct);
        setupDnD();
        new ThemeListener(this);

    }
    final static Font plainFont = new Font("lucida sans",Font.PLAIN,12);
    final static Font italicPlainFont = new Font("lucida sans",Font.ITALIC /*| Font.BOLD*/,12);
    final static Font boldFont = new Font("lucida sans",Font.BOLD,12);

    private CommanderTree setupCtree(final int width) {
        DefaultColumnSpec cs = new DefaultColumnSpec(width,treeModel);
        cs.setRenderer(0,new DefaultCommanderCellRenderer() {
            private void setColors(JLabel jl, boolean isSelected){
                if(backColor!=null&&foreColor!=null&&selColor!=null){
                    if(isSelected){
                        jl.setBackground(selColor);
                        jl.setForeground(backColor);
                    }
                    else{
                        jl.setBackground(backColor);
                        jl.setForeground(foreColor);
                    }
                }
            }
            public Component getSpecialRendererComponent(CommanderTree ct, SpecialValues value, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel) super.getSpecialRendererComponent(ct, value, node, isSelected, hasFocus);
                jl.setFont(boldFont);
                setColors(jl,isSelected);
                return jl;
            }

            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct, attributeValue, node, isSelected, hasFocus);
                DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode) node;
                if (dmtn!=null) {
                    Bookmark b = (Bookmark) dmtn.getUserObject();
                    if(b.isDirectory()){
                        jl.setFont(boldFont);
                    }
                    else if(b.isSuperBookmark()){
                        jl.setFont(italicPlainFont);
                    }
                    else{
                        jl.setFont(plainFont);
                    }
                }
                setColors(jl,isSelected);
                return jl;
            }
        });

        ct = new CommanderTree(cs,treeModel);
        ct.setSelectionMode(CommanderTree.SelectionMode.single);
        //initialise to root
        ct.setHideCurrentIfRoot(true);
        ct.setNodeAsynchronous(bookmarks.getRoot(), null, -1, false, null);

        //enter opens the selected bookmark (if any)
        ct.addKeyListener(new KeyAdapter() {
            public void keyReleased(KeyEvent e) {
                if(e.getKeyCode()==KeyEvent.VK_ENTER){
                    boolean ctrl=(e.getModifiers() & Main.ctrl_mask)!=0;
                    log.fine("entertyped ctrldown="+ctrl);
                    openSelectedBookmark(ctrl?OpenBookmark.pane2:OpenBookmark.mostRecent);
                }
            }
        });

        //double click opens the selected bookmark (if any)
        ct.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if(e.getClickCount()==2){
                    boolean ctrl=(e.getModifiers() & Main.ctrl_mask)!=0;
                    openSelectedBookmark(ctrl?OpenBookmark.pane2:OpenBookmark.mostRecent);
                }
            }
        });

        //delete deletes selected bookmakr
        final AbstractAction delAction = new AbstractAction("delete") {
            public void actionPerformed(ActionEvent e) {
                delete();
            }
        };
        ct.registerAction("deleteaction",delAction);
        ct.registerInput(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE,0),"deleteaction");
        ct.registerInput(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0),"deleteaction");

        new AutocompleteJumper(ct,new AutocompleteJumper.RowIdentifier() {
            public String getRowId(Object node) {
                Bookmark b = getBookmark((TreeNode) node);
                return b.getName();
            }
        });

        ct.setHeadersVisible(false);

        final Border emptyBorder = BorderFactory.createEmptyBorder(GAP,GAP,GAP,GAP);
        setBorder(emptyBorder);
        cf.addListener(new CommanderFocus.CommanderFocusListener() {
            public void focusChanged(Component lostFocus, Component gainedFocus) {
                boolean hasFocus = cf.hasFocus(BookmarksBar.this);
                if(!hasFocus){
                    log.finest("lost focus ");
                    setBorder(emptyBorder);
                }
                else{
                    setBorder(focusBorder);
                    log.finest("ct gained focus");
                    final boolean b = ct.requestFocusInWindow();
                    log.finest("requestFocusInWindow="+b);
                    if(!b){
                        log.finest("reqFoc=");
                        ct.requestFocus();
                    }
                }
            }
        });
        ct.addFocusListener(new FocusAdapter() {
            public void focusGained(FocusEvent e) {
                cf.giveFocus(BookmarksBar.this);
            }
        });

        ct.addNodeListener(new CommanderTree.NodeListener() {
            public void parentNodeChanged(Object newParent, Object oldParent) {
                log.fine("node changed");
                DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode) newParent;
                if (dmtn!=null) {
                    final String name;
                    if(dmtn.isRoot()){
                        name="/";
                    }
                    else{
                        Bookmark b = (Bookmark) dmtn.getUserObject();
                        log.fine("new parent b="+b);
                        name=b.getName();
                    }
                    dirName.setText(name);
                }
            }
        });
        return ct;
    }

    enum OpenBookmark{pane1,pane2,mostRecent}
    private void openSelectedBookmark(OpenBookmark pane) {
        Bookmark b = getSelectedBookmark();
        if(b!=null && !b.isDirectory()){
            if (b.isSuperBookmark()) {
                opener.open(b.getF1(), b.getF2());
            } else {
                if(pane==OpenBookmark.pane1){
                    opener.open(b.getF1(),null);
                }
                else if(pane==OpenBookmark.pane2){
                    opener.open(null,b.getF1());
                }
                else{
                    opener.openInMostRecentView(b.getF1());
                }
            }
        }
    }

    public boolean hasCommanderFocus(){
        return cf.hasFocus(this);
    }

    private void openBookmark(Bookmark b) {
        if (b.isSuperBookmark()) {
            opener.open(b.getF1(), b.getF2());
        } else {
            opener.openInMostRecentView(b.getF1());
        }
    }

    private static final int GAP = 6;
    private final int borderWidth = GAP/2;
    private Border focusBorder;
    private Border createFocusBorder(Color c) {
        return BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(borderWidth,borderWidth,borderWidth,borderWidth),
                BorderFactory.createLineBorder(c,borderWidth)
        );
    }

    private Bookmark getBookmark(TreeNode treeNode) {
        log.finest("getBookmark treeNode hasparent="+(treeNode.getParent()!=null));
        Bookmark b = (Bookmark) ((DefaultMutableTreeNode)treeNode).getUserObject();
        return b;
    }

    private JButton getHomeButton() {
        final Action homeAction = new AbstractAction("home") {
            public void actionPerformed(ActionEvent e) {
                opener.openInMostRecentView(homeUrl);
            }
        };
        homeAction.putValue(Action.SHORT_DESCRIPTION,"Open your home directory");
        homeAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_H);
        //NS I like the home icon, but on Mac OS X the buttons without icons have rounded edges and the ones with
        //have squared edges, so either tmp and home buttons need icons or neither (and I can't find an icon for tmp)
     //   homeAction.putValue(Action.SMALL_ICON, ImageProvider.getInstance().getIcon("icons/16/gohome.png"));
        final JButton jb = new JButton(homeAction);
        jb.setFocusable(false);
        return jb;
    }
    private JButton getTmpButton() {
        final Action homeAction = new AbstractAction("tmp") {
            public void actionPerformed(ActionEvent e) {
                opener.openInMostRecentView(tmpUrl);
            }
        };
        homeAction.putValue(Action.SHORT_DESCRIPTION,"Open temp directory");
        homeAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_T);
        //homeAction.putValue(Action.SMALL_ICON, ImageProvider.getInstance().getIcon("icons/16/gohome.png"));
        final JButton jb = new JButton(homeAction);
        jb.setFocusable(false);
        return jb;
    }


    //Themeable interface
    private Color backColor,foreColor,selColor;
    public void setColors(Color backColor, Color foreColor, Color selColor) {
        ct.setBackground(backColor);
        ct.setForeground(foreColor);
        focusBorder =createFocusBorder(selColor);
        this.backColor=backColor;
        this.foreColor=foreColor;
        this.selColor=selColor;
        repaint();
    }



    public interface Opener{
        public void open(Url f1, Url f2);
        public void openInMostRecentView(Url f1);
    }

    private void setupDnD() {
        final DropTargetListener dtl = new DropTargetListener(){
            public void dragEnter(DropTargetDragEvent evt) {
                if(!evt.isDataFlavorSupported(DataFlavor.javaFileListFlavor)){
                    log.finer("reject drag supported flavors="+evt.getCurrentDataFlavorsAsList());
                    evt.rejectDrag();
                }
                else evt.acceptDrag(DnDConstants.ACTION_COPY);
            }
            public void dragOver(DropTargetDragEvent evt) {; }
            public void dragExit(DropTargetEvent evt) {;}
            public void dropActionChanged(DropTargetDragEvent evt) {;}
            public void drop(DropTargetDropEvent evt) {
                log.fine("drop isEdt="+SwingUtilities.isEventDispatchThread());
                onDrop(evt);
            }
        };
        ct.setDropTarget(new DropTarget(this,dtl));
    }

    private void onDrop(DropTargetDropEvent evt) {
        final boolean dataFlavorSupported = evt.isDataFlavorSupported(DataFlavor.javaFileListFlavor);
        log.finer("drop dataFlavorSupported="+dataFlavorSupported);
        if(dataFlavorSupported){
            evt.acceptDrop(evt.getDropAction());
            log.fine("accept drop javafilelist");
            try {
                Object data=evt.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);
                List<File> files = (List<File>)data;
                //just drop the 1st Vfile
                if(files.size()>0){
                    final File f = files.get(0);
                    log.finer("drop file="+f);
                    final Vfile vfile = new Vfile(f);
                    //think there might be a swing bug circa 1.4 (seen on Mac OS 10.3)
                    //createBookmark springs a JDialog, which hangs the prog..
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            createBookmark(vfile);
                        }
                    }) ;
                }
            } catch (UnsupportedFlavorException e) {
                log.log(Level.WARNING,"UnsupportedFlavorException",e);
            } catch (IOException e) {
                log.log(Level.WARNING,"IOException",e);
            }
        }
    }

    private void createBookmark(Vfile Vfile) {
        if (Vfile!=null) {
            if (!Vfile.exists() || Vfile.isDirectory()) {
                log.finer("prompt user for create bookmark vfile="+Vfile);
                String name = (String) JOptionPane.showInputDialog(this
                        ,"enter a name for the bookmark\n"
                        +Vfile.getName()
                        ,"Create Bookmark"
                        ,JOptionPane.OK_CANCEL_OPTION,null,null
                        ,Vfile.getName()
                ) ;
                if(name!=null){
                 //   try {
                        addBookmark(Url.getUrl(Vfile), null, name);
//                    } catch (MalformedURLException e) {
//                        log.log(Level.WARNING,"couldn't convert vile="+Vfile+" into url",e);
//                        JOptionPane.showMessageDialog(this
//                                ,"can't convert file reference into URL ","Edit Bookmark",JOptionPane.ERROR_MESSAGE);
//                    }
                }
            }
            else{
                JOptionPane.showMessageDialog(this,"can't bookmark files, only directories","Create Bookmark",JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void addBookmark(Url url1, Url url2, String name) {
        log.fine("addBookmark");
        final Bookmark b = Bookmark.createBookmark(url1,url2,name);
        DefaultMutableTreeNode parent = (DefaultMutableTreeNode) ct.getNode();
        DefaultMutableTreeNode child = new DefaultMutableTreeNode(b);
        DefaultTreeModel dtm = bookmarks;
        parent.add(child);
        dtm.nodesWereInserted(parent,new int[]{parent.getChildCount()-1});
        //now select the new b
        ct.selectNode(child);
    }


    // commandable interface impl
    public void delete() {
        DefaultMutableTreeNode dmtn = getSelectedNode();
        if(dmtn!=null){
            Bookmark b = (Bookmark) dmtn.getUserObject();
            String message = b==null?"Delete bookmark node?":"Delete bookmark "+b.getName()+" ?";
            if(JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(this, message
                    ,"Delete",JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE)){
                DefaultMutableTreeNode parent=(DefaultMutableTreeNode) dmtn.getParent();
                if(parent!=null){
                    Object[] nodes = {dmtn};
                    int[] indices={bookmarks.getIndexOfChild(parent,dmtn)};
                    parent.remove(dmtn);
                    bookmarks.nodesWereRemoved(parent,indices,nodes);
                }
                else log.warning("couddn't remove bookmark as its parent was null! bm="+dmtn);
            }
        }
    }

    public void launch() {
        Bookmark b = (Bookmark) ct.getSelectedNode();
        if (b!=null) {
            openBookmark(b);
        }
    }

    public void mkdir() {
        log.fine("mkdir pressed");
        String dest=(String)JOptionPane.showInputDialog(this,
                "Create Directory:"
                ,"Create Directory",JOptionPane.QUESTION_MESSAGE,null
                ,null,"");
        if(dest!=null && dest.trim().length()>0){
            DefaultMutableTreeNode parent = (DefaultMutableTreeNode) ct.getNode();
            Bookmark b = new Bookmark(dest);
            DefaultMutableTreeNode child = new DefaultMutableTreeNode(b);
            parent.add(child);
            bookmarks.nodesWereInserted(parent,new int[]{parent.getChildCount()-1});
            //now select the new dir
            ct.selectNode(child);
        }
    }

    public void move() {
        //todo:?
    }

    private Bookmark getSelectedBookmark(){
        DefaultMutableTreeNode dmtn = getSelectedNode();
        if(dmtn!=null){
            return (Bookmark) dmtn.getUserObject();
        }
        return null;
    }

    private DefaultMutableTreeNode getSelectedNode() {
        DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode) ct.getSelectedNode();
        return dmtn;
    }

    public void rename() {
        Bookmark b = getSelectedBookmark();
        if(b!=null){
            String dest=(String)JOptionPane.showInputDialog(this,
                    "rename "+b.getName()+" to:"
                    ,"Rename",JOptionPane.QUESTION_MESSAGE,null
                    ,null,b.getName());
            if(dest!=null){
                b.setName(dest);
            }
        }
    }

    public void copy() {
        //todo:?
    }

    public void duplicate() {
        Bookmark b = getSelectedBookmark();
        if(b!=null){
            String dest=(String)JOptionPane.showInputDialog(this,
                    "duplicate "+b.getName()+" new name:"
                    ,"Duplicate",JOptionPane.QUESTION_MESSAGE,null
                    ,null,b.getName());
            if(dest!=null){
                addBookmark(b.getF1(),b.getF2(),dest);
            }
        }
    }

}
