package uk.co.hcsoftware.commander.gui;

import ns.MacApp;
import ns.gui.GlassDialog;
import ns.gui.TextDisplay;
import uk.co.hcsoftware.commander.CannotOpenUrlException;
import uk.co.hcsoftware.commander.Commandable;
import uk.co.hcsoftware.commander.Main;
import uk.co.hcsoftware.commander.Url;
import uk.co.hcsoftware.commander.treeplugin.TreePlugin;
import uk.co.hcsoftware.commander.bookmarks.BookmarksBar;
import uk.co.hcsoftware.commander.prefs.PrefsDialog;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

/**  todo: F3 views file (open in viewer) F4 edits file (open in notepad?) ENTER 'launches' file
 *
 */
public class CommanderFrame extends  JFrame{
    private static final Logger log=Logger.getLogger(CommanderFrame.class.getName());
    private static final Preferences prefs = Preferences.userNodeForPackage(CommanderFrame.class);

    private static final int GAP = 6;

    private final DirectoryView pane1;
    private final DirectoryView pane2;
    private JSplitPane mainPan;//=new JPanel();
    private JToolBar buttonPanel= new JToolBar();
    private static AboutDialog aboutDialog;
    private final JMenu fileMenu = new JMenu("File");
    private static boolean registeredMac;
    //private static volatile int frames=0;
    private List<Action> actions;
    private GlassDialog glassDialog;
    private final static Font messageFont = new Font("Dialog",Font.BOLD,20);
    private final JTextArea messageArea;
    private final BookmarksBar bookmarsBar;
    private final CommanderFocus cf = new CommanderFocus();

    public CommanderFrame() throws HeadlessException {
        super("CommanderBond");
        super.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        super.setIconImage(ImageProvider.getInstance().getImage("icons/16/file-manager.png"));

        final Container cp = getContentPane();
        cp.setLayout(new BorderLayout());
        mainPan=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        cp.add(mainPan);
        setupActions();
        //  mainPan.setLayout(new GridLayout(1,2));
        pane1 = new DirectoryView("pane1", this, cf);
        pane2 = new DirectoryView("pane2", this, cf);
        bookmarsBar = new BookmarksBar(new BookmarksBar.Opener() {
                public void open(Url pane1Url, Url pane2url) {
                    if(pane1Url!=null){
                        //pane1.openFile(new Vfile(pane1Url), null);
                        try {
                            pane1.openUrl(pane1Url);
                        } catch (CannotOpenUrlException e) {
                            log.log(Level.WARNING, "exception", e);
                        }
                    }
                    if(pane2url!=null){
                        //pane2.openFile(new Vfile(pane2url), null);
                        try {
                            pane2.openUrl(pane2url);
                        } catch (CannotOpenUrlException e) {
                            log.log(Level.WARNING, "exception", e);
                        }
                    }
                }
                public void openInMostRecentView(Url f1) {
                    if (f1!=null) {
                        DirectoryView view = getView();
                        if(view!=null){
                            //view.openFile(new Vfile(f1), null);
                            try {
                                view.openUrl(f1);
                            } catch (CannotOpenUrlException e) {
                                log.log(Level.WARNING, "exception", e);
                            }
                        }
                    }
                }
            }, cf);

        mainPan.add(pane1);
        mainPan.add(pane2);
        mainPan.setOneTouchExpandable(true);
        mainPan.setResizeWeight(0.5);  //share extra space equally


        createMenuBar();
        boolean mac=false;
        try {
            if(MacApp.isMac()){
                if (!registeredMac) {
                    log.info("is mac! registering application");
                    new MacApp(new MacApp.MacImplementation() {
                        public boolean hasPrefsDialog() {
                            return true;
                        }
                        public void about() {
                            showHelpDialog(CommanderFrame.this);
                        }
                        public void preferences() { showPrefs(CommanderFrame.this);}
                        public void openFile(File f) {
                            log.fine("open file="+f);
                            //pane1.openFile(new Vfile(f), null);
                            try {
                                // try {
                                pane1.openUrl(Url.getUrl(f));//f.toURL());
//                            } catch (MalformedURLException e) {
//                                log.log(Level.WARNING, "exception", e);
//                            }
                            } catch (CannotOpenUrlException e) {
                                log.log(Level.WARNING, "exception", e);
                            }
                        }
                        public boolean quit() {
                            return true;
                        }
                        public void printFile(File f) {
                        }
                        public void reOpenApplication() {
                        }
                        public void openApplication() {
                        }
                    });
                    registeredMac=true;
                }
                mac=true;
            }
        } catch (MacApp.CouldntSetupMacException e) {
            log.log(Level.WARNING,"error",e);
        }
        final JMenu helpMenu = new JMenu("Help");
        getJMenuBar().add(helpMenu);
        helpMenu.add(new AbstractAction("Keyboard shortcuts"){
            ShortcutsDialog sd;
            public void actionPerformed(ActionEvent e) {
                if (sd==null) {
                    List<ShortcutsDialog.ShortcutDescriptor> l=new ArrayList<ShortcutsDialog.ShortcutDescriptor>();
                    for(Action a:actions){
                        l.add(new ShortcutsDialog.ShortcutDescriptor(a));
                    }
                    sd = new ShortcutsDialog(CommanderFrame.this,l);
                    sd.pack();
                }
                sd.setLocationRelativeTo(CommanderFrame.this);
                sd.setVisible(true);
                sd.requestFocus();
            }
        });

        if(!mac){//not a mac so add a help menu item
            helpMenu.addSeparator();
            helpMenu.add(new AbstractAction("about..."){
                public void actionPerformed(ActionEvent e) {
                    showHelpDialog(CommanderFrame.this);
                }
            });
            fileMenu.addSeparator();
            final AbstractAction prefsAction = new AbstractAction("Preferences.."){
                public void actionPerformed(ActionEvent e) {
                    showPrefs(CommanderFrame.this);
                }
            };
            setupAction("configure",prefsAction,KeyStroke.getKeyStroke(KeyEvent.VK_BACK_QUOTE,Main.ctrl_mask),"preferences");
            fileMenu.add(prefsAction);//add it before teh Exit item
            fileMenu.addSeparator();
            fileMenu.add(new AbstractAction("Exit") {
                public void actionPerformed(ActionEvent e) {
                    System.exit(0);
                }
            }) ;
        }


        buttonPanel.setBorder(BorderFactory.createEmptyBorder(GAP,GAP,GAP,GAP));
        buttonPanel.setFloatable(false);
        cp.add(buttonPanel,BorderLayout.NORTH);

        cp.add(bookmarsBar,BorderLayout.WEST);
        bookmarsBar.setBorder(BorderFactory.createEmptyBorder(GAP,GAP,GAP,GAP));


        pane1.getFocus();
        pack();
        log.fine("getfocus="+pane1.getFocus());

//        addWindowListener(new WindowAdapter() {
//            public void windowActivated(WindowEvent e) {
//                if (prefs.getBoolean("autorefresh",true)) {
//                    log.fine("mainpane activated, refreshing dirs");
//                    pane1.refresh();
//                    pane2.refresh();
//                }
//            }
//        });

        //restore size
        int h=prefs.getInt("frameheight",600);
        int w=prefs.getInt("framewidth",800);
        log.finer("restore framesize w="+w+" h="+h);
        if(w<10 || h<10){
            log.warning("unsensible framesizes, using defaults w="+w+" h="+h);
            w=800; h=600;
        }
        setSize(w,h);
        mainPan.setDividerLocation(0.5f);
        addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                final int h = getHeight();
                final int w = getWidth();
                log.finer("saving framesize w="+w+" h="+h);
                prefs.putInt("frameheight",h);
                prefs.putInt("framewidth",w);
            }
        });


        //setup glasspane for blocking input to this frame
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        JPanel jp1 = new JPanel(gridbag);
        final Color backColor = new Color(0,0,0,0);//255/2);
        final TextDisplay td = new TextDisplay(messageFont,Color.black,Color.white);
        jp1.setOpaque(false);
        messageArea=td.getMessageArea();
        messageArea.setWrapStyleWord(false);
        messageArea.setLineWrap(false);
        messageArea.setOpaque(true);
        messageArea.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        c.weightx=1;
        c.weighty=1;
        jp1.add(td,c);
        glassDialog = new GlassDialog(CommanderFrame.this,backColor,jp1);
    }
    private static void showPrefs(CommanderFrame cf){
        PrefsDialog pd = new PrefsDialog(cf);
        pd.setLocationRelativeTo(cf);
        pd.setVisible(true);
    }
    private Url getDefaultUrl(String paneName){
        Preferences p = Preferences.userNodeForPackage(Main.class);
        String prop = p.get(paneName+".vfile",null);
        if(prop!=null){
          //  try {
                return Url.getUrl(prop);
//            } catch (MalformedURLException e) {
//                log.warning("couldn't convert prop="+prop+" to URL");
//            }
        }
    //    try {
            return Url.getUrl(new File(System.getProperty("user.home")));
//        } catch (MalformedURLException e) {
//            log.log(Level.WARNING, "exception", e);
//            return null;
//        }
    }
    public void loadDefaultFiles(Url dir1, Url dir2) {
        try {
            pane1.openUrl(dir1!=null?dir1:getDefaultUrl("pane1"));
            pane2.openUrl(dir2!=null?dir2:getDefaultUrl("pane2"));
        } catch (CannotOpenUrlException e) {
            log.log(Level.WARNING, "exception", e);
        }
//        pane1.openFile(dir1!=null?new Vfile(dir1):getDefaultUrl("pane1"), null);
//        pane2.openFile(dir2!=null?new Vfile(dir2):getDefaultUrl("pane2"), null);
    }



    private Commandable getCommandable(){
        if(bookmarsBar.hasCommanderFocus()){
            return bookmarsBar;
        }
        else return getView();
    }
    private DirectoryView getView(){
        if(pane1.hasCommanderFocus()) return pane1;
        else if(pane2.hasCommanderFocus()) return pane2;
        else{
            DirectoryView dv=(DirectoryView) cf.getMostRecentFocusHolder(pane1, pane2);
            if(dv!=null) return dv;
            log.fine("neither pane has the focus, defaulting to pane1");
            return pane1;
        }
//        long time = pane1.getLastGainedFocusTime();
//        long time2 = pane2.getLastGainedFocusTime();
//        log.finer("time="+time+" time2="+time2);
//        if(time>time2) return pane1;
//        else if(time2>time) return pane2;
//        else if(time==time2){
//            log.warning("both panes had focus at same time: surely impossible?  defaulting to pane1");
//            return pane1;
//        }
//        else{
//            log.fine("neither pane has ever had the focus, defaulting to pane1");
//            return pane1;
//        }
//        if(pane1.hasFocus()) return current?pane1:pane2;
//        else if(pane2.hasFocus()) return current?pane2:pane1;
//        return pane1;
    }
    private void setupActions() {
        ((JComponent)getContentPane()).getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB,0),"tab");
        ((JComponent)getContentPane()).getActionMap().put("tab",moveFocusAction);

        final InputMap inputMap = mainPan.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        final ActionMap actionMap = mainPan.getActionMap();

        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_B,Main.ctrl_mask),"bookmarks");
        actionMap.put("bookmarks",new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                cf.giveFocus(bookmarsBar);
                //bookmarsBar.requestFocus();
            }
        });



        actions = new ArrayList<Action>();
//        actions.add(setupAction(null,new AbstractAction("dumpstats"){
//            public void actionPerformed(ActionEvent e) {
//                log.info("stats:");
//                log.info(Vfile.getStats());
//            }
//        },KeyStroke.getKeyStroke(KeyEvent.VK_F1,Main.ctrl_mask),"dumpstats"));
  //      actions.add(setupAction(filterAction,KeyStroke.getKeyStroke(KeyEvent.VK_Y,Main.ctrl_mask),"Filter.."));
        actions.add(setupAction("fileopen",launchAction,KeyStroke.getKeyStroke(KeyEvent.VK_F4,0),"Launch"));//+Main.getCtrlName()+"+P"));

        actions.add(setupAction("editcopy",copyAction,KeyStroke.getKeyStroke(KeyEvent.VK_F5,0),"Copy"));
        actions.add(setupAction("edit_add",duplicateAction,KeyStroke.getKeyStroke(KeyEvent.VK_F5,KeyEvent.SHIFT_MASK),"Duplicate"));
        actions.add(setupAction("filesaveas",moveAction,KeyStroke.getKeyStroke(KeyEvent.VK_F6,0),"Move"));
        actions.add(setupAction("filesaveas",renameAction,KeyStroke.getKeyStroke(KeyEvent.VK_F6,KeyEvent.SHIFT_MASK),"Rename"));
        actions.add(setupAction("folder_new",mkdirAction,KeyStroke.getKeyStroke(KeyEvent.VK_F7,0),"Make dir"));
        actions.add(setupAction("editdelete",deleteAction,KeyStroke.getKeyStroke(KeyEvent.VK_F8,0),"Delete"));

        actions.add(setupAction("window_new",newFrameAction,KeyStroke.getKeyStroke(KeyEvent.VK_N,Main.ctrl_mask),"New window"));



        actions.add(setupAction("reload",refreshAction,KeyStroke.getKeyStroke(KeyEvent.VK_R,Main.ctrl_mask),"Refresh"));
        // inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0),"delete");
        // inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE,0),"delete");

        actions.add(setupAction("back",recall,KeyStroke.getKeyStroke(KeyEvent.VK_P,Main.ctrl_mask),"Open previous directory"));//+Main.getCtrlName()+"+P"));

        actions.add(setupAction(syncAction,KeyStroke.getKeyStroke(KeyEvent.VK_E,Main.ctrl_mask),"Sync left->right"));//+Main.getCtrlName()+"+E"));
        actions.add(setupAction(swapAction,KeyStroke.getKeyStroke(KeyEvent.VK_U,Main.ctrl_mask),"Swap left/right"));//+Main.getCtrlName()+"+U"));


        //actions.add(setupAction(null,defaultColsAction,null,"Launch"));//+Main.getCtrlName()+"+P"));

        for(Action a:actions){
            inputMap.put((KeyStroke) a.getValue(Action.ACCELERATOR_KEY),a.getValue(Action.ACTION_COMMAND_KEY));
            actionMap.put(a.getValue(Action.ACTION_COMMAND_KEY),a);
        }
        //filterAction.putValue(Action.SHORT_DESCRIPTION,"Filter the current view");
        refreshAction.putValue(Action.SHORT_DESCRIPTION,"Refresh view from the file system");
        deleteAction.putValue(Action.SHORT_DESCRIPTION,"Delete selected file(s)");
        mkdirAction.putValue(Action.SHORT_DESCRIPTION,"Create a new directory");
        moveAction.putValue(Action.SHORT_DESCRIPTION,"Move selected file(s) to another directory");
        renameAction.putValue(Action.SHORT_DESCRIPTION,"Rename the selected file");
        duplicateAction.putValue(Action.SHORT_DESCRIPTION,"Duplicate the selected file");
        copyAction.putValue(Action.SHORT_DESCRIPTION,"Copy selected file(s) to another directory");
        launchAction.putValue(Action.SHORT_DESCRIPTION,"Launch selected file in OS default application");
        //defaultColsAction.putValue(Action.SHORT_DESCRIPTION,"Restore column widths to default values");

        newFrameAction.putValue(Action.SHORT_DESCRIPTION,"New window");
        recall.putValue(Action.SHORT_DESCRIPTION,"Open previous directory");
        swapAction.putValue(Action.SHORT_DESCRIPTION,"Swap left and right directories");
        syncAction.putValue(Action.SHORT_DESCRIPTION,"Set right directory to same as left directory");

        copyAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_C);
        duplicateAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_D);
        renameAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_E);
        moveAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_V);
        deleteAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_T);
        mkdirAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_M);
        refreshAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_R);
        launchAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_L);

        setupButtonPanel();

    }

    private void setupButtonPanel() {
        buttonPanel.add(createButton(launchAction));
        // buttonPanel.addSeparator();
        buttonPanel.add(createButton(copyAction));
        buttonPanel.add(createButton(duplicateAction));
        buttonPanel.add(createButton(renameAction));
        //  buttonPanel.addSeparator();
        buttonPanel.add(createButton(moveAction));
        buttonPanel.add(createButton(mkdirAction));
        // buttonPanel.addSeparator();
        buttonPanel.add(createButton(deleteAction));
        buttonPanel.addSeparator();

        // buttonPanel.addSeparator();
        buttonPanel.add(createButton(refreshAction));

    }
    private void createMenuBar(){
        JMenuBar jmb = new JMenuBar();
        fileMenu.setMnemonic('f');
        fileMenu.add(newFrameAction);
        fileMenu.addSeparator();
        fileMenu.add(launchAction);
        fileMenu.add(copyAction);
        fileMenu.add(duplicateAction);
        fileMenu.add(renameAction);
        fileMenu.add(moveAction);
        fileMenu.add(deleteAction);
        fileMenu.add(mkdirAction);


        jmb.add(fileMenu);
        JMenu view = new JMenu("View");
        view.setMnemonic('v');
        view.add(refreshAction);


        view.add(recall);
        view.add(syncAction);
        view.add(swapAction);
        view.addSeparator();
//        view.add(defaultColsAction);
//        view.add(saveColSizesAction);
        final String title = "Bookmark Current View";
        view.add(new AbstractAction(title) {
            public void actionPerformed(ActionEvent e) {
                Url vf1=pane1.getURL();
                Url vf2=pane2.getURL();
                if(vf1==null || vf2==null){
                    JOptionPane.showMessageDialog(CommanderFrame.this,"Both panes must have valid directories set",title,JOptionPane.ERROR_MESSAGE);
                    return;
                }
                log.finer("prompt user for create bookmark vf1="+vf1+" vf2="+vf2);
                String name = (String) JOptionPane.showInputDialog(CommanderFrame.this
                        ,"enter a name for the bookmark\n"
                        ,title
                        ,JOptionPane.QUESTION_MESSAGE,null,null
                        ,null
                ) ;
                if(name!=null){
                    //try {
                        bookmarsBar.addBookmark(vf1, vf2, name);
//                    } catch (MalformedURLException e1) {
//                        log.log(Level.WARNING,"couldn't convert file into url",e1);
//                        JOptionPane.showMessageDialog(CommanderFrame.this
//                                ,"can't convert file reference into URL ",title,JOptionPane.ERROR_MESSAGE);
//                    }
                }
            }
        });
     //   view.add(filterAction);

        jmb.add(view);

        if ("true".equals(System.getProperty("debug"))) {
            JMenu debug = new JMenu("debug");
            jmb.add(debug);
            debug.add(new AbstractAction("block input") {
                public void actionPerformed(ActionEvent e) {
                    blockInput("Input blocked\n\n(press ESC to unblock)"
                            ,new KeyAdapter() {
                                public void keyPressed(KeyEvent e) {
                                    log.fine("keyTyped="+e);
                                    if(e.getKeyCode()==KeyEvent.VK_ESCAPE){
                                        unblockInput();
                                    }
                                }
                            }
                    );
                }
            });
        }
        setJMenuBar(jmb);
    }
    private KeyListener kl;
    public void blockInput(String text){
        blockInput(text,null);
    }
    public void blockInput(String text, KeyListener kl){
        this.kl=kl;
        if(kl!=null) glassDialog.addKeyListener(kl);
        updateBlockedMessage(text);
        glassDialog.setVisible(true);
        final boolean b = glassDialog.requestFocusInWindow();
        log.finest("requestFocusInWindow="+b);
        if(!b) glassDialog.requestFocus();
    }

    protected void updateBlockedMessage(String text) {
        messageArea.setText(text);
    }

    public void unblockInput(){
        if(kl!=null) glassDialog.removeKeyListener(kl);
        glassDialog.setVisible(false);
    }




    private JButton createButton(Action a){
        JButton jb = new JButton(a);
        jb.setFocusable(false);
        KeyStroke accelerator = (KeyStroke) a.getValue(Action.ACCELERATOR_KEY);
        String acceleratorText;
        if (accelerator != null) {
            acceleratorText = getAcceleratorText(accelerator);
            if (acceleratorText.length()>0) {
                jb.setText("<html>"+jb.getText()+"<font color=\"gray\"><small> "+acceleratorText+"</small></font></html>");
            }
        }
        return jb;
    }

    public static String getAcceleratorText(KeyStroke keyStroke) {
        String acceleratorText="";
        int modifiers = keyStroke.getModifiers();
        if (modifiers > 0) {
            log.fine("creatbutton mods="+modifiers+" mac="+MacApp.isMac()+" crtlname="+Main.getCtrlName());
            if (MacApp.isMac() && (modifiers & Main.ctrl_mask)!=0) {
                acceleratorText=Main.getCtrlName();
            } else {
                acceleratorText = KeyEvent.getKeyModifiersText(modifiers);
            }
            acceleratorText += '+';
        }

        int keyCode = keyStroke.getKeyCode();
        if (keyCode != 0) {
            acceleratorText += KeyEvent.getKeyText(keyCode);
        } else {
            acceleratorText += keyStroke.getKeyChar();
        }
        return acceleratorText;
    }

    private Action setupAction(Action copyAction, KeyStroke keystroke, String name){
        return setupAction(null,copyAction,keystroke,name);
    }
    private Action setupAction(String iconname, Action copyAction, KeyStroke keystroke, String name){
        copyAction.putValue(Action.ACCELERATOR_KEY,keystroke);
        copyAction.putValue(Action.ACTION_COMMAND_KEY,name);
        copyAction.putValue(Action.NAME,name);
        if(iconname!=null) copyAction.putValue(Action.SMALL_ICON,
                ImageProvider.getInstance().getIcon("icons/16/"+iconname+".png"));
        return copyAction;
    }

//    public Action getFilterAction() {
//        return filterAction;
//    }

//    Action filterAction = new AbstractAction("Filter..") {
//        public void actionPerformed(ActionEvent e) {
//            final DirectoryView view = getView();
//            if (view!=null) {
//                final FilterDialog fd = getFilterDialog();
//                view.getName();
//                Filter f = fd.showDialog(view==pane1?"Filter left pane":"Filter right pane",view.getFilter());
//                if(f!=null) view.filter(f);
//                else if(fd.filterClear){
//                    view.filter(null);
//                }
//            }
//        }
//    };
    Action copyAction = new AbstractAction("Copy F5") {
        public void actionPerformed(ActionEvent e) {
            final Commandable view = getCommandable();
            if (view!=null) {
                view.copy();
            }
        }
    };
    Action duplicateAction = new AbstractAction("Duplicate SHFT+F5") {
        public void actionPerformed(ActionEvent e) {
            final Commandable view = getCommandable();
            if (view!=null) {
                view.duplicate();
            }
        }
    };
    Action renameAction = new AbstractAction("Rename SHFT+F6") {
        public void actionPerformed(ActionEvent e) {
            final Commandable view = getCommandable();
            if (view!=null) {
                view.rename();
            }
        }
    };
    Action moveAction = new AbstractAction("Move F6") {
        public void actionPerformed(ActionEvent e) {
            final Commandable view = getCommandable();
            if (view!=null) {
                view.move();
            }
        }
    };
    Action deleteAction = new AbstractAction("Delete F8") {
        public void actionPerformed(ActionEvent e) {
            final Commandable view = getCommandable();
            if (view!=null) {
                view.delete();
            }
        }
    };
    Action mkdirAction = new AbstractAction("Make Dir F7") {
        public void actionPerformed(ActionEvent e) {
            final Commandable view = getCommandable();
            if (view!=null) {
                view.mkdir();
            }
        }
    };
    Action refreshAction=new AbstractAction("Refresh "+Main.getCtrlName()+"+R") {
        public void actionPerformed(ActionEvent e) {
            final DirectoryView view = getView();
            if (view!=null) {
                view.refresh();
            }
        }
    };
    Action syncAction=new AbstractAction("Sync "+Main.getCtrlName()+"+E") {
        public void actionPerformed(ActionEvent e) {
            try {
                Url otherF = pane1.getURL();
                pane2.openUrl(otherF);
            } catch (CannotOpenUrlException e1) {
                log.log(Level.WARNING, "exception", e1);
            }
        }
    };
    Action swapAction=new AbstractAction("Swap") {
        public void actionPerformed(ActionEvent e) {
            final DirectoryView view = getView();
            if (view!=null) {
                try {
                    DirectoryView v2 = view==pane1?pane2:pane1;
                    Url f1 = view.getURL();
                    Url f2 = v2.getURL();
                    view.openUrl(f2);
                    v2.openUrl(f1);
                } catch (CannotOpenUrlException e1) {
                    log.log(Level.WARNING, "exception", e1);
                }
            }
        }
    };
    Action recall=new AbstractAction("open previous dir") {
        public void actionPerformed(ActionEvent e) {
            final DirectoryView view = getView();
            if (view!=null) {
                try {
                    view.showPreviousUrl();
                } catch (CannotOpenUrlException e1) {
                    log.log(Level.WARNING, "exception", e1);
                }
            }
        }
    };
    Action launchAction = new AbstractAction("launch") {
        public void actionPerformed(ActionEvent e) {
            final Commandable view = getCommandable();
            if (view!=null) {
                view.launch();
            }
        }
    };

    Action moveFocusAction=new AbstractAction("tab") {

        public void actionPerformed(ActionEvent e) {
            //moveFocus(DirectoryView.this);
            Commandable dv = getCommandable();
            if(dv instanceof BookmarksBar){
                pane1.getFocus();//todo: moving focus from bookmarksbar to a view should select the last selected view (not the lastselected+1)
            }
            else{
                DirectoryView other=(dv==pane1)?pane2:pane1;
                other.getFocus();
            }
        }
    };



    private static void showHelpDialog(CommanderFrame cf) {
        if(aboutDialog==null){
            aboutDialog = new AboutDialog(cf,"about CommanderBond");
        }
        aboutDialog.setLocationRelativeTo(cf);
        aboutDialog.setVisible(true);
    }


    Action newFrameAction = new AbstractAction("New Frame") {
        public void actionPerformed(ActionEvent e) {
            CommanderFrame cf = new CommanderFrame();
            cf.setSize(CommanderFrame.this.getSize());
            cf.setLocationByPlatform(true);
            cf.setVisible(true);
            cf.loadDefaultFiles(null, null);
        }
    };
//    Action defaultColsAction = new AbstractAction("Restore default col widths") {
//        public void actionPerformed(ActionEvent e) {
//            pane1.setDefaultColumnSizes();
//            pane2.setDefaultColumnSizes();
//        }
//    };
//    Action saveColSizesAction = new AbstractAction("Save col widths") {
//        public void actionPerformed(ActionEvent e) {
//            pane1.saveColumnSizes();
//            pane2.loadColumnSizes();
//        }
//    };


    public void setDividerLocation(float v) {
        mainPan.setDividerLocation(v);
    }
    private FilterDialog fd;
    public FilterDialog getFilterDialog() {
        if(fd==null) fd = new FilterDialog(this);
        return fd;
    }

    public Url getTargetUrl() {
        DirectoryView dv = getView();
        if(dv==pane1){
            return pane2.getURL();
        }
        return pane1.getURL();
    }
    public TreePlugin getTargetTreePlugin() {
        DirectoryView dv = getView();
        if(dv==pane1){
            return pane2.getTreePlugin();
        }
        return pane1.getTreePlugin();
    }

    public void setDefaultFocus() {
        cf.giveFocus(pane1);
      //  pane1.requestFocus();
    }
}
