package uk.co.hcsoftware.commander.gui;

import uk.co.hcsoftware.commander.treeplugin.file.Vfile;
import uk.co.hcsoftware.commander.Filter;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.ActionListener;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import java.util.List;
import java.util.ArrayList;

//editor isn't as flexible as would like: really filters should be sql-like strings e.g.:
//name like '.*' && (type=f || size>201b)
//then we could have a GUI editor for the query..
//maybe just a simplified syntax to start with..
public class FilterDialog extends JDialog {
    private final JRadioButton show;
    private final JRadioButton hide;
    private List<Condition> conditions=new ArrayList<Condition>();
    private final JPanel conditionPan;


    public FilterDialog(Frame cf) throws HeadlessException {
        super(cf,"Filter",true);
        setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);

        JPanel jp = new JPanel(new BorderLayout(5,5));
        getContentPane().add(jp);
        jp.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
        JPanel mainP = new JPanel(new BorderLayout());
        jp.add(mainP);

        Box showHidePan = new Box(BoxLayout.X_AXIS);
        show = new JRadioButton("show");
        hide = new JRadioButton("hide");
        JRadioButton[] jrbs = {show,hide};
        ButtonGroup bg = new ButtonGroup();
        for(JRadioButton jrb:jrbs){
            showHidePan.add(jrb);
            bg.add(jrb);
        }
        show.setSelected(true);

        mainP.add(showHidePan,BorderLayout.NORTH);

        JPanel queryPan = new JPanel(new BorderLayout());


        conditionPan = new JPanel();
        queryPan.add(conditionPan);
        Action addConditionAction=new AbstractAction("+") {
            public void actionPerformed(ActionEvent e) {
                 conditions.add(createDefaultCondition());
                 relay();
            }
        };
        Action remConditionAction=new AbstractAction("-") {
            public void actionPerformed(ActionEvent e) {
                if (conditions.size()>0) {
                    conditions.remove(conditions.size()-1);
                    relay();
                }
            }
        };
        JPanel addPan = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        addPan.add(new JButton(remConditionAction));
        addPan.add(new JButton(addConditionAction));
        queryPan.add(addPan,BorderLayout.SOUTH);

        mainP.add(queryPan);
        queryPan.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.black)
        ,BorderFactory.createEmptyBorder(5,5,5,5))
        );

        JPanel bPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        final Action okAction = new AbstractAction("OK") {
            public void actionPerformed(ActionEvent e) {
                try {
                    check();
                    isOK=true;
                    filterSet=true;
                    dispose();
                } catch (InvalidInputException e1) {
                    JOptionPane.showMessageDialog(FilterDialog.this,e1.getMessage()
                            ,"Filter",JOptionPane.ERROR_MESSAGE);
                }
            }
        };
        final Action canAction = new AbstractAction("Cancel") {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        };
        final Action clearAction = new AbstractAction("Clear filter") {
            public void actionPerformed(ActionEvent e) {
                filterClear=true;
                dispose();
            }
        };
        okAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_O);
        canAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_C);
        clearAction.putValue(Action.MNEMONIC_KEY,KeyEvent.VK_L);
        JButton ok = new JButton(okAction),cancel=new JButton(canAction),clear=new JButton(clearAction);
        for(JButton jb:new JButton[]{cancel,clear,ok}){
            bPanel.add(jb);
        }
        jp.add(bPanel,BorderLayout.SOUTH);
        getRootPane().setDefaultButton(ok);
        final InputMap inputMap = jp.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        final ActionMap actionMap = jp.getActionMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),"esc");
        actionMap.put("esc",canAction);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),"ok");
        actionMap.put("ok",okAction);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_PLUS,0),"add");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_EQUALS,KeyEvent.SHIFT_MASK),"add");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_EQUALS,0),"add");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ADD,0),"add");

        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_MINUS,0),"remove");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_SUBTRACT,0),"remove");

        actionMap.put("add",addConditionAction);
        actionMap.put("remove",remConditionAction);

    }
    static class InvalidInputException extends Exception{
        public InvalidInputException(String message) {
            super(message);
        }
    };
    static interface Condition{
        public abstract boolean accept(Vfile vf);
        public String getDescription();
        public void checkValid() throws InvalidInputException;
        Component getControl();

        ConditionTypes getType();
    }
    static class NameCondition extends JTextField implements Condition{
        private Pattern p;

        public ConditionTypes getType() {
            return ConditionTypes.Name;
        }

        public NameCondition() {
            this(".*");
        }

        public NameCondition(String text) {
            super(text);
        }

        public Component getControl() {
            return this;
        }

        public void checkValid() throws InvalidInputException {
            String regex = getText();
            if(regex.length()>0){
                try {
                    p = Pattern.compile(regex);
                } catch (PatternSyntaxException e) {
                    throw new InvalidInputException("Invalid regex: "+e.getDescription());
                }
            }
        }

        public boolean accept(Vfile vf) {
            String regex = getText();
            if(regex.length()>0){
                if (p==null) {
                    p = Pattern.compile(regex);
                }
                return p.matcher(vf.getName()).matches();
            }
            else return true;
        }

        public String getDescription() {
            String regex = getText();
            return (regex.length()>0)?"name: "+regex:"";
        }
    }

    static class TypeCondition extends JPanel implements Condition{
        JRadioButton file=new JRadioButton("file"),dir=new JRadioButton("directory")
               ;// ,both=new JRadioButton("all");
        public ConditionTypes getType() {
            return ConditionTypes.Type;
        }
        public TypeCondition() {
            super(new FlowLayout());
            JRadioButton[] jrbs = {/*both,*/file,dir};
            ButtonGroup bg = new ButtonGroup();
            for(JRadioButton j:jrbs){
                add(j);
                bg.add(j);
            }
            file.setSelected(true);
        }

        public Component getControl() {
            return this;
        }
        public void checkValid() throws InvalidInputException { }
        public boolean accept(Vfile vf) {
            final boolean result;
            /*if(both.isSelected()) result=true;
            else */if(file.isSelected()){
                result=!vf.isDirectory();
            }
            else result=vf.isDirectory();
            return result;
        }

        public String getDescription() {
           /* if(both.isSelected()) return "";
            else*/ if(file.isSelected()){
                return "files";
            }
            else return "dirs";
        }
    }


    public boolean filterSet=false;
    public boolean filterClear=false;

    private boolean isOK;
    // private static FilterDialog instance;
    public Filter showDialog(String title, Filter f){

        reset(f);
        setTitle(title);

        relay();
        setVisible(true);
        if(!isOK) return null;
        return new MyFilter(new ArrayList<Condition>(conditions),hide.isSelected());
    }
    private static class MyFilter implements Filter{
        private List<Condition> conditions;
        private boolean hide;

        public MyFilter(List<Condition> conditions, boolean hide) {
            this.conditions = conditions;
            this.hide = hide;
        }

        public boolean shouldDisplay(Vfile vf) {
            final boolean show =!hide;
            for(Condition c:conditions){
                if(show){ //should show only things taht pass all tests of all conditions
                    if(!c.accept(vf)) return false;
                }
                else{//hide: if it passes any of the tests return true
                    if(c.accept(vf)) return false;
                }
            }
            return true;
        }
        public String getDescription() {
            StringBuilder sb = new StringBuilder();
            sb.append(hide?"hide: ":"show: ");
            boolean prevDesc=false;
            for(Condition c:conditions){
                final String desc = c.getDescription();
                if (desc.length()>0 && prevDesc) {
                    sb.append(',');
                }
                sb.append(desc);
                if(desc.length()>0) prevDesc=true;
            }
            return sb.toString();
        }
    }

    private void relay() {
        conditionPan.removeAll();
        conditionPan.setLayout(new GridLayout(conditions.size(),2,3,3));
        for(Condition c:conditions){
            final JComboBox jcb = createConditionCombo();
            conditionPan.add(jcb);
            final Component control = c.getControl();
            conditionPan.add(control);
            final int index = conditionPan.getComponentCount()-1;
            final int condInd=index/2;
            jcb.setSelectedItem(c.getType());
            jcb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    conditions.set(condInd, ((ConditionTypes) jcb.getSelectedItem()).create()) ;
                    relay();
                }
            });
        }

        pack();
        setLocationRelativeTo(super.getOwner());
    }

    private static enum ConditionTypes{
        Name("Name like (regex):"){
            public Condition create() {
            return new NameCondition();
            }
        },
        Type("Is a:"){
            public Condition create() {
                return new TypeCondition();
            }
        };
        private String label;
        private ConditionTypes(String label) {
            this.label = label;
        }
        public abstract Condition create();
        public String getLabel() {
            return label;
        }
    }
    private JComboBox createConditionCombo() {
        JComboBox jcb = new JComboBox(ConditionTypes.values());
        jcb.setRenderer(new DefaultListCellRenderer(){
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                value=((ConditionTypes)value).getLabel();
                return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            }
        });
        jcb.setEditable(false);
        return jcb;
    }

    public static void main(String[] args) {
        FilterDialog fd = new FilterDialog(null);
        fd.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
       // fd.setVisible(true);
        fd.showDialog("test",null);
    }




    private void check() throws InvalidInputException {
        for(Condition c:conditions){
            c.checkValid();
        }
    }

    private void reset(Filter f){
        conditions.clear();
        isOK=false;
        filterSet=false;
        filterClear=false;
        if(f==null || !(f instanceof MyFilter)){
            conditions.add(createDefaultCondition());
        }
        else{
            conditions=new ArrayList(((MyFilter)f).conditions);
        }

    }

    private NameCondition createDefaultCondition() {
        return new NameCondition();   //default condition
    }

}
