package uk.co.hcsoftware.commander.gui;

import uk.co.hcsoftware.commander.Main;

import java.awt.*;
import java.util.logging.Logger;
import java.util.prefs.PreferenceChangeEvent;
import java.util.prefs.PreferenceChangeListener;
import java.util.prefs.Preferences;


/**
 * Title: ThemeListener
 * Copyright:    Copyright (c) 17-Nov-2005
 *
 * @author spurlinn
 */
public class ThemeListener implements PreferenceChangeListener {
    private static final Preferences prefs = Preferences.userNodeForPackage(Main.class);
    public static final Logger log = Logger.getLogger(ThemeListener.class.getName());
    private final Themeable t;

    public void preferenceChange(PreferenceChangeEvent evt) {
        String key=evt.getKey();
        if("forecolor".equals(key) || "backcolor".equals(key) || "selcolor".equals(key)){
            log.fine("pref change detected, setting colors");
            loadColors();
        }
    }
    public ThemeListener(Themeable t) {
        this.t = t;
        final WeakListener wl = new WeakListener(this, PreferenceChangeListener.class){
            public Object onNullObserver() {
                prefs.removePreferenceChangeListener((PreferenceChangeListener) getProxy());
                return null;
            }
        };
        prefs.addPreferenceChangeListener((PreferenceChangeListener) wl.getProxy());
        setupColors();

    }

    private void setupColors() {
        loadColors();
    }
    private void loadColors() {
        Color foreColor=new Color(prefs.getInt("forecolor",Color.green.getRGB()));
        Color backColor=new Color(prefs.getInt("backcolor",Color.black.getRGB()));
        Color selColor=new Color(prefs.getInt("selcolor",Color.green.getRGB()));
        t.setColors(backColor, foreColor, selColor);
    }

}
