package uk.co.hcsoftware.commander.io;

import uk.co.hcsoftware.commander.treeplugin.file.DeleteFailedException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;


/**
 * Title: FileUtils
 * Copyright:    Copyright (c) 01-Dec-2005
 *
 * @author spurlinn
 */
public class FileUtils {
    public static final Logger log = Logger.getLogger(FileUtils.class.getName());
    private static FileUtils instance;

    public static FileUtils getInstance(){
        if(instance==null) instance = new FileUtils();
        return instance;
    }
     /**
     * recuresively deletes all files/dirs under f, and f itself
     * @return list of deleted files and dirs
     * @throws uk.co.hcsoftware.commander.treeplugin.file.DeleteFailedException if cannot delete any vfile
     */
    public List<File> deleteRecursive(File f) throws DeleteFailedException{
        List<File> deleted = new ArrayList<File>();
        if(f.isDirectory()){
            log.fine("delete dir="+f+", recursing to children");
            File[] kids = f.listFiles();
            for(File k:kids){
                log.fine("delete child="+k);
                deleteRecursive(k);
            }
        }

        if(f.delete()) deleted.add(f);
        else throw new DeleteFailedException(deleted,f);
        return deleted;
    }
    public void copyTo(final File sourceFile, final File destFile) throws IOException {
        log.fine("copyfile src="+toString()+" dest="+destFile);
        FileChannel srcChannel = new FileInputStream(sourceFile).getChannel();
        FileChannel dstChannel = new FileOutputStream(destFile).getChannel();
        dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
        srcChannel.close();
        dstChannel.close();
    }

}
