package uk.co.hcsoftware.commander.prefs;

import uk.co.hcsoftware.commander.Main;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.logging.Logger;
import java.util.prefs.Preferences;


public class ColorPanel implements PrefsTab{
    private static final Logger log = Logger.getLogger(ColorPanel.class.getName());
    private static final Preferences directoryViewPrefs = Preferences.userNodeForPackage(Main.class);
    JPanel jp;
    private List<Scheme> schemes=new ArrayList<Scheme>();
    private final ButtonGroup bg = new ButtonGroup();
    public ColorPanel() {
        jp = new JPanel(new BorderLayout());
        addDefaultSchemes(schemes);

        jp.setBorder(BorderFactory.createTitledBorder("select a colour scheme:"));
        JPanel radioPan = new JPanel(new BorderLayout());
        jp.add(radioPan,BorderLayout.NORTH);
        radioPan.setLayout(new GridLayout(schemes.size()+1,1,0,2));
        radioPan.setBorder(BorderFactory.createEmptyBorder(3,20,0,0));

        final CustomScheme customScheme = new CustomScheme();
        final ActionListener jrbList = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Scheme s = (Scheme) ((JRadioButton)e.getSource()).getClientProperty("owner");
                customScheme.setButtonIcons(s.fore,s.back,s.sel);
            }
        };
        boolean oneActive=false;
        for(final Scheme s:schemes){
            final JRadioButton jrb = s.getRadio();
            if(s==schemes.get(0)){
                jrb.setText(s.getName()+" [default]");
            }
            bg.add(jrb);
            radioPan.add(jrb);
            jrb.setAlignmentX(0);

            jrb.addActionListener(jrbList);
            boolean active = s.isActive();
            jrb.setSelected(active);
            if(active) oneActive=true;
        }

        schemes.add(customScheme);
        final JRadioButton radio = customScheme.getRadio();
        if(!oneActive) radio.setSelected(true);
        bg.add(radio);
        JPanel customPan2 = new JPanel();
        customPan2.setLayout(new FlowLayout(FlowLayout.LEFT,2,0));//new BoxLayout(customPan2,BoxLayout.Y_AXIS));
        customPan2.add(radio);

        customPan2.add(customScheme.foreButton);
        customPan2.add(customScheme.selButton);
        customPan2.add(customScheme.backButton);
        radioPan.add(customPan2);
    }

    private void addDefaultSchemes(List<Scheme> schemes) {
        schemes.add(new Scheme("Invaders",Color.green,Color.black, Color.green
        ,"The classic theme, as envisioned by Ian Fleming"));
        schemes.add(new Scheme("Hi Viz",Color.orange,Color.black, Color.orange
        ,"Maximum contrast for precision operations"));
        schemes.add(new Scheme("Vorsprung durch Technik",Color.red,Color.black, Color.red
        ,"Helps maintain night vision"));
        schemes.add(new Scheme("Blackboard",Color.white,Color.black, Color.lightGray
        ,"Monochrome"));
        schemes.add(new Scheme("Snow",Color.black,Color.white, Color.lightGray
                ,"Minimalist"
        ));
        schemes.add(new Scheme("Bluebell",Color.black,new Color(230,230,230), new Color(121,118,227)
        ,"Undemanding theme, a favourite"));
        schemes.add(new Scheme("Hayley",Color.black,Color.white, new Color(247,99,233)
        ,"Pink on white. Remember to rest your eyes every few hours."));
        schemes.add(new Scheme("Hayley Plus",new Color(0,0,204),Color.white, new Color(204,0,204)
        ,"An instant classic"));
        schemes.add(new Scheme("Papyrus",Color.black,new Color(255,255,204), new Color(191,44,44)
        ,"Traditional heavyweight feel"));

    }

    public class CustomScheme extends Scheme{
        final JButton foreButton = new JButton("Fore");
        final JButton selButton = new JButton("Hi");
        final JButton backButton = new JButton("Back");

        public CustomScheme() {
            super("Custom"
                    , new Color(directoryViewPrefs.getInt("forecolor",Color.green.getRGB()))
                    , new Color(directoryViewPrefs.getInt("backcolor",Color.black.getRGB()))
                    , new Color(directoryViewPrefs.getInt("selcolor",Color.green.getRGB()))
            ,"Define your own custom scheme"
            );
            ActionListener al = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    final Object source = e.getSource();
                    boolean isFore=source==foreButton;
                    boolean isBack=source==backButton;
                    String name=isFore?"foreground":isBack?"background":"selection";
                    Color c = JColorChooser.showDialog(jp,name+" colour",isFore?fore:isBack?back:sel);
                    if(c!=null){
                        if(isFore){
                            fore=c;
                        }
                        else if(isBack) back=c;
                        else sel=c;
                        setButtonIcons();
                        jrb.setSelected(true);
                    }
                }
            };
            foreButton.addActionListener(al);
            selButton.addActionListener(al);
            backButton.addActionListener(al);
            foreButton.setToolTipText("foreground");
            backButton.setToolTipText("background");
            selButton.setToolTipText("highlight");
            setButtonIcons();
        }
        private void setButtonIcons(){
            setButtonIcons(fore,back,sel);
        }
        public void setButtonIcons(Color fore, Color back, Color sel){
            foreButton.setIcon(getIcon(fore,10,10));
            backButton.setIcon(getIcon(back,10,10));
            selButton.setIcon(getIcon(sel,10,10));
            this.fore=fore;
            this.back=back;
            this.sel=sel;
        }
        private Icon getIcon(final Color col,final int h,final int w){
            return new Icon() {
                public void paintIcon(Component c, Graphics g, int x, int y) {
                    g.setColor(col);
                    g.fillRect(x,y,w,h);
                    g.setColor(Color.black);
                    g.drawRect(x,y,w,h);
                }

                public int getIconWidth() {
                    return w;
                }

                public int getIconHeight() {
                    return h;
                }
            };
        }

    }
    private class Scheme{
        protected String name;
        protected Color fore;
        protected Color back;
        protected Color sel;
        protected JRadioButton jrb ;
        public Scheme(String name, Color fore, Color back, Color sel) {
            this(name,fore,back,sel,null);
        }
        public Scheme(String name, Color fore, Color back, Color sel,String toolTip) {
            this.name = name;
            this.fore = fore;
            this.back = back;
            this.sel=sel;
            jrb = new JRadioButton(name);
            jrb.putClientProperty("owner",this);
            if(toolTip!=null) jrb.setToolTipText(toolTip);
        }

        public String getName() {
            return name;
        }

        public Scheme(String name) {
            this.name = name;
        }

        public boolean isActive(){
            Color foreColor=new Color(directoryViewPrefs.getInt("forecolor",Color.green.getRGB()));
            Color backColor=new Color(directoryViewPrefs.getInt("backcolor",Color.black.getRGB()));
            Color selColor=new Color(directoryViewPrefs.getInt("selcolor",Color.green.getRGB()));
            return foreColor.equals(fore) && backColor.equals(back) && selColor.equals(sel);
        }
        public void apply() {
            directoryViewPrefs.putInt("forecolor",fore.getRGB());
            directoryViewPrefs.putInt("backcolor",back.getRGB());
            directoryViewPrefs.putInt("selcolor",sel.getRGB());
            log.fine("set preferred scheme to "+name);
        }

        public JRadioButton getRadio(){
            // jrb.setSelected(isActive());
            return jrb;
        }

    }

    public void apply() {

        for(Enumeration<AbstractButton> en = bg.getElements();en.hasMoreElements();){
            AbstractButton jrb = en.nextElement();
            if(jrb.isSelected()){
                Scheme s = (Scheme) (jrb).getClientProperty("owner");
                s.apply();
                return;
            }
        }
        log.fine("tried to apply but no button selected!");
    }


    public String getName() {
        return "colours";
    }

    public Component getComponent() {
        return jp;
    }
    public static void main(String[] args) {
        final JFrame jf = new JFrame(" test");
        jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        jf.getContentPane().add(new ColorPanel().getComponent());
        jf.setSize(400,600);
        jf.setVisible(true);
    }
}
