package uk.co.hcsoftware.commander.test;

import uk.co.hcsoftware.commander.tree.CtreeModel;
import uk.co.hcsoftware.commander.tree.CommanderTree;
import uk.co.hcsoftware.commander.tree.SortDirection;

import javax.swing.*;
import java.util.logging.Logger;
import java.util.List;
import java.util.Arrays;
import java.util.Date;
import java.io.File;
import java.awt.*;


/**
 * Title: CommanderTreeSimulation
 * Copyright:    Copyright (c) 08-Nov-2005
 *
 * @author spurlinn
 */
public class CommanderTreeSimulation {
    public static final Logger log = Logger.getLogger(CommanderTreeSimulation.class.getName());
    private static CtreeModel<File> model;

    public static void main(String[] args) {
        final List<String> cols = Arrays.asList(
                new String[] {
                    "name"
                    ,"type"
                    ,"size"
                    ,"lastmodified"
                } )  ;

        JFrame jf = new JFrame("test");

        model = new CtreeModel<File>() {
            public List<File> getChildren(File o) {
                return Arrays.asList(o.listFiles());
            }

            public boolean hasChildren(File file) {
                return file.isDirectory();
            }

            public int getAttributeCount() {
                return 3;
            }

            public String getAttributeName(int index) {
                return cols.get(index);
            }

            public File getParent(File s) {
                return s.getParentFile();
            }

            public Object getAttributeValue(File s, int attribute) {
                switch(attribute){
                    case 0: return s.getName();
                    case 1: return null;
                    case 2: return new Long(s.length());
                    default: return new Date(s.lastModified());
                }
            }

            public int compare(File s, File s1, int attribute) {
                switch(attribute){
                    case 0: return s.getName().compareTo(s1.getName());
                    case 2: return (int) -(s1.length()-s.length());
                }

                throw new IllegalArgumentException("cannot sort by attribute="+attribute);
            }

            public boolean isComparable(int attribute) {
                return attribute==0||attribute==2;
            }

            public SortDirection getDefaultSortDirection(int attribute) {
                switch(attribute){
                    case 0: return SortDirection.ascending;
                    case 2: return SortDirection.descending;
                }
                throw new IllegalArgumentException("cannot sort by attribute="+attribute);
            }

            public void addChangeListener(ChangeListener<File> changeListener) {

            }

            public void removeChangeListener(ChangeListener<File> changeListener) {

            }

        };
        final CommanderTree ct = new CommanderTree(model);
        ct.setFont( new Font("lucida sans",Font.PLAIN,12));
        jf.add(ct);
        ct.setNodeAsynchronous(new File(System.getProperty("user.home")), null, -1, false, null);
        jf.setSize(800,600);
        jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        jf.setVisible(true);
    }
    

}
