package uk.co.hcsoftware.commander.tree;

import ns.TaskTimer;
import ns.gui.SwingWorker3;
import uk.co.hcsoftware.commander.Main;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.event.*;
import java.util.*;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * commander-style tree view, similar to jtree but the actual GUI is a commander IF
 *
 * displays a Node object
 *
 * Title: CommanderTree
 * Copyright:    Copyright (c) 31-Oct-2005
 *
 * @author spurlinn
 *
 *
 */
public class CommanderTree extends JPanel{
  //  private static final Preferences prefs = Preferences.userNodeForPackage(DirectoryView.class);

    public static final Logger log = Logger.getLogger(CommanderTree.class.getName());
    private CtreeModel model;
    private JTable table = new JTable();
    private ColumnSpec columnSpec;
    //private Object node;
    private TableModelAdapter tableModel;
    private NodeComparator comparator;
    private JScrollPane tableScrollPane=new JScrollPane(table);


    public CommanderTree(final CtreeModel model) {
        this(new DefaultColumnSpec(model),model );
    }
    public CommanderTree(final ColumnSpec columns, final CtreeModel model) {
        super(new BorderLayout());

        table.setAutoCreateColumnsFromModel(false);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        table.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        table.setShowGrid(false);
        table.setIntercellSpacing(new Dimension(0,0));
        table.setRowHeight(15);
        //table.setc

        comparator= new NodeComparator(model);
        setModel(model);

        setupTableListeners();
        setColumnSpec(columns);
        add(tableScrollPane);
        setupActions();
    }

    public CtreeModel getModel() {
        return model;
    }


    public synchronized void setDropTarget(DropTarget dt) {
        table.setDropTarget(dt);
        tableScrollPane.setDropTarget(dt);
    }

    Action openAction = new AbstractAction("open") {
        public void actionPerformed(ActionEvent e) {
            openSelectedRow();
        }
    };
    Action openParentAction = new AbstractAction("open parent") {
        public void actionPerformed(ActionEvent e) {
            openParent();
        }
    };

    private Action firstRow = new AbstractAction("home") {
        public void actionPerformed(ActionEvent e) {
            selectFirstRow();
        }
    };
    /**
     *
     * @param nodes
     * @param scrollToVisibleIndex if -1 then ignore
     */
    public void selectNodes(List nodes, int scrollToVisibleIndex) {
        ListSelectionModel lsm = table.getSelectionModel();
        lsm.clearSelection();
        for(Object n:nodes){
            int ro = tableModel.getRow(n);
            if (ro>-1) {
                lsm.addSelectionInterval(ro,ro);
            }
        }
        if(scrollToVisibleIndex>-1 && nodes.size()>0){
            scrollToVisible(tableModel.getRow(nodes.get(scrollToVisibleIndex)));
        }
    }
    public void selectNode(Object node) {
        int ro = (tableModel).getRow(node);
        if(ro!=-1){
            selectRow(ro);
        }
        else log.info("ignoring selectNode as no such node in table");
    }
    public void selectRow(int ro) {
        if(table.getRowCount()>ro){
            log.finer("selectrow="+ro);
            table.getSelectionModel().setSelectionInterval(ro,ro);
            final Rectangle cellRect = table.getCellRect(ro,0,true);
            // cellRect.width=10;
            log.finer("scrolltovisible cellRect="+cellRect);
            table.scrollRectToVisible(cellRect);
        }
        else log.info("ingoring selectRow as ro="+ro+" rocount="+table.getRowCount());
    }
    public void selectFirstRow() {
        selectRow(0);
    }

    Action lastRow = new AbstractAction("end") {
        public void actionPerformed(ActionEvent e) {
            selectLastRow();
        }
    };

    protected void selectLastRow() {
        int lastrow = table.getRowCount();
        selectRow(lastrow-1);
    }

    /**
     * opens the parent of the currently displayed node (regardless of where selected from)
     */
    private void openParent() {
        log.fine("openParent");
        if (getNode()!=null) {
            Object node = getNode();
            Object parent=model.getParent(node);
            if (parent!=null) {
                openNode(parent,node,-1);
            }
            else log.fine("ignoring openParent as parent is null");
        }
        else log.fine("ignoring openParent as node is null");
    }
    private void setupTableListeners() {
        final JTableHeader th = table.getTableHeader();
        th.addMouseListener(new MouseAdapter(){
            public void mouseClicked(MouseEvent e) {
                int col = th.columnAtPoint(e.getPoint());
                log.fine("mouseclicked col="+col);
                headerClicked(col, (e.getModifiers() & Main.ctrl_mask)!= 0, e.isShiftDown());
            }
        });

        table.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if(e.getClickCount()==2){
                    openSelectedRow();
                }
            }
        });
    }

    /**
     *
     * @return the node, the children of which are currently visible (or null if no such node)
     */
    public Object getNode() {
        return tableModel.node;
    }

    private JTableHeader cachedTableHeader;
    public void setHeadersVisible(boolean b) {
        if(!b){
            final JTableHeader tableHeader = table.getTableHeader();
            if(tableHeader!=null){
                cachedTableHeader=tableHeader;
                table.setTableHeader(null);
            }
        }
        else{
            if(cachedTableHeader!=null) table.setTableHeader(cachedTableHeader);
        }
    }

    public void clearSelection() {
        table.clearSelection();
    }

    public boolean isRowSelected(int ro) {
        return table.isRowSelected(ro);
    }

    public void setSelected(int rowInt, int rowInt1) {
        table.getSelectionModel().addSelectionInterval(rowInt,rowInt);
    }
    public void setSelection(int rowInt, int rowInt1) {
        table.getSelectionModel().setSelectionInterval(rowInt,rowInt);
    }
//    public void selectRow(int ro) {
//        if(ct.getRowCount()>ro){
//            log.finer("selectrow="+ro);
//            ct.setSelection(ro,ro);
//            ct.scrollToVisible(ro);
//        }
//        else log.info("ingoring selectRow as ro="+ro+" rocount="+ct.getRowCount());
//    }

    public void scrollToVisible(int row) {
        final Rectangle cellRect = table.getCellRect(row,0,true);
        log.finer("scrolltovisible cellRect="+cellRect);
        table.scrollRectToVisible(cellRect);

    }

    public int getRowCount() {
        return table.getRowCount();
    }

    public int getFirstSelectedRow() {
        return table.getSelectedRow();
    }


    public int[] getSelectedRows() {
        return table.getSelectedRows();
    }

    public boolean isSelected(int ro) {
        return table.getSelectionModel().isSelectedIndex(ro);
    }

    public boolean isParent(int ro) {
        return tableModel.isParentRow(ro);
    }

    public boolean isCurrent(int ro) {
        return tableModel.isCurrentRow(ro);
    }

    public void createDefaultDragGestureRecognizer(int actionCopy, DragGestureListener dragGestureListener, DragSource ds) {
        ds.createDefaultDragGestureRecognizer(table,DnDConstants.ACTION_COPY,dragGestureListener);
    }

    public void addSelectionListener(ListSelectionListener lsl) {
        table.getSelectionModel().addListSelectionListener(lsl);
    }
    public void removeSelectionListener(ListSelectionListener lsl) {
        table.getSelectionModel().removeListSelectionListener(lsl);
    }

    public int getColumnWidth(int viewCol) {
        final TableColumnModel columnModel = table.getColumnModel();
        return columnModel.getColumn(viewCol).getWidth();
    }

//    public void setNode(Object node) {
//        this.node = node;
//    }

    public boolean isSource(Object keyEventSource) {
        return table.equals(keyEventSource) || this.equals(keyEventSource);
    }

    public static enum SelectionMode{
        single,single_interval,multiple_interval;
    }
    public void setSelectionMode(SelectionMode sm){
        final int i;
        if(sm==SelectionMode.single) i=ListSelectionModel.SINGLE_SELECTION;
        else if(sm==SelectionMode.single_interval) i=ListSelectionModel.SINGLE_INTERVAL_SELECTION;
        else if(sm==SelectionMode.multiple_interval) i=ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;
        else{
            log.warning("unrecognized selmode="+sm+" ignoring");
            return;
        }
        table.getSelectionModel().setSelectionMode(i);

    }

//     public void saveColumnSizes() {
//        log.fine("saving columnwidths");
//        final TableColumnModel columnModel = table.getColumnModel();
//         for(Enumeration<TableColumn> en=columnModel.getColumns();en.hasMoreElements();){
//             TableColumn tc = en.nextElement();
//             String id = (String) tc.getIdentifier();
//             prefs.putInt(id,columnModel.getColumn(0).getWidth());
//         }
//    }
//
//    private int getColWidth(String propertyName, int defaultWidth) {
//        final int anInt = prefs.getInt(propertyName,defaultWidth);
//        if(anInt<2){
//            log.warning("prefval for col="+propertyName+" was unsensbile="+anInt+" using default="+defaultWidth);
//            return defaultWidth;
//        }
//        return anInt;
//    }

//    public void setDefaultColumnSizes(){
//        final TableColumnModel columnModel = table.getColumnModel();
//        columnModel.getColumn(0).setPreferredWidth(5);
//        columnModel.getColumn(1).setPreferredWidth(200);
//        columnModel.getColumn(2).setPreferredWidth(50);
//        columnModel.getColumn(3).setPreferredWidth(120);
//    }
    private Color selectionForeground,selectionBackground;

    public Color getSelectionForeground() {
        return selectionForeground==null?UIManager.getColor("Table.selectionForeground"):selectionForeground;
    }

    public void setSelectionForeground(Color selectionForeground) {
        this.selectionForeground = selectionForeground;
    }

    public Color getSelectionBackground() {
        return selectionBackground==null?UIManager.getColor("Table.selectionBackground"):selectionBackground;
    }

    public void setSelectionBackground(Color selectionBackground) {
        this.selectionBackground = selectionBackground;
    }

    private class HeaderRenderer extends JLabel implements TableCellRenderer {
        private int attributeIndex;
        private String colName;
        public HeaderRenderer(int attributeIndex, String colName) {
            this.attributeIndex = attributeIndex;
            this.colName = colName;
            super.setHorizontalAlignment(JLabel.CENTER);
            setBorder(BorderFactory.createLineBorder(Color.black));
        }
        private String getHeaderText(){
            StringBuilder sb = new StringBuilder();
            sb.append("<html><body><font=lucida>");
            if (comparator!=null) {
                int index = comparator.getIndex(attributeIndex);
                if(index>-1){
                    sb.append("<gtOne>");
                    final boolean gtOne = comparator.gtOne();
                    if(gtOne){
                        sb.append("(");
                        sb.append(index+1);
                    }
                    sb.append(comparator.isForwardDirection(index)?"\u25b2":"\u25bc");//"\u25b4":"\u25be");//"\u25b2":"\u25bc");
                    sb.append("</gtOne>");
                    if(gtOne) sb.append(")");
                    sb.append(" ");
                }
            }
            sb.append(colName);
            sb.append("</body></html>");
            return sb.toString();
        }
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            setText(getHeaderText());
            return this;
        }
    }

    private boolean hideCurrentIfNoParent;

    public void setHideCurrentIfRoot(boolean hideCurrentIfNoParent) {
        this.hideCurrentIfNoParent = hideCurrentIfNoParent;
    }

    public Object getNode(int row){
        return tableModel.getNode(row);
    }
    public List getSelectedNodes(boolean includeParent, boolean includeCurrent){
        int[] sel = getSelectedRows();
        List l = new ArrayList(sel.length);
        for(int row:sel){
            if(!includeParent && tableModel.isParentRow(row)) continue;
            if(!includeCurrent && tableModel.isCurrentRow(row)) continue;
            l.add(getNode(row));
        }
        return l;
    }
    public int[] getSelectedRows(boolean includeParent, boolean includeCurrent){
        int[] sel = getSelectedRows();
        int[] tmp=new int[sel.length];
        int i=0;
        for(int row:sel){
            if(!includeParent && tableModel.isParentRow(row)) continue;
            if(!includeCurrent && tableModel.isCurrentRow(row)) continue;
            tmp[i++]=row;
        }
        int[] ret=new int[i];
        System.arraycopy(tmp,0,ret,0,i);
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSelectedFileIndices i="+i+" ret.length="+ret.length);
        }
        return ret;
    }

   private class TableModelAdapter extends AbstractTableModel implements /*NodeListener, */CtreeModel.ChangeListener {
       private final CtreeModel model;
       private List rows;
       private Object node;
       private Object parent;
       public TableModelAdapter(CtreeModel model) {
           this.model = model;
           //addNodeListener(this);
           model.addChangeListener(this);
       }
       /**
        * WARNING: doesn't find parent or curr nodes
        * @param node
        * @return
        */
       public int getRow(Object node){
           int ro=0;
           for (Iterator iterator = rows.iterator(); iterator.hasNext();ro++) {
               Object o = iterator.next();
               if(node.equals(o)) return ro+getExtraRowCount();
           }
           return -1;
       }
       protected boolean isCurrentRow(int row){
          if(hideCurrentIfNoParent && parent==null) return false;
          return row==0;
       }
       protected boolean isParentRow(int row){
           if(parent==null) return false;
           return row==1;
       }
       private int getExtraRowCount(){
           if(parent==null){
               if(hideCurrentIfNoParent) return 0;
               return 1;
           }
           else return 2;
       }
       public Object getNode(int row){
           if(rows==null) throw new ArrayIndexOutOfBoundsException("no such row as "+row+": node is null");
           else if(isCurrentRow(row)) return node;
           else if(isParentRow(row)) return parent;
           else {
               final int r = row-getExtraRowCount();
               if(r>=rows.size()){
                   log.warning("about to get aiiob r="+r+" row="+row+" extra="+getExtraRowCount()+" rows.size()="+rows.size()+" rowcount="+getRowCount());
               }
               return rows.get(r);
           }
       }
       public int getRowCount() {
        //   if(node==null) return 0;
           if(rows==null) return 0;
           final int i = rows.size()+getExtraRowCount();
           return i;
       }

       public String getColumnName(int column) {
           return model.getAttributeName(column);
       }

       public int getColumnCount() {
           final int attributeCount = model.getAttributeCount();
           return attributeCount;
       }
       public Object getValueAt(int rowIndex, int columnIndex) {
           if(rows==null) return null;
           final Object node = getNode(rowIndex);
           if (node!=null) {
              return model.getAttributeValue(node,columnIndex);
           }
           else throw new ArrayIndexOutOfBoundsException("no node at row="+rowIndex);
       }

       private boolean isDirColumn(int columnIndex) {
           return columnIndex==DirColumnIndex;
       }

       private boolean isNameColumn(int columnIndex) {
           return columnIndex==NameColumnIndex;
       }

       public void setNewParent(Object newParent, List children) {
           this.node=newParent;
           this.parent=model.getParent(newParent);
           this.rows=children==null?model.getChildren(newParent):children;
           if (comparator!=null) {
               sortNodes(comparator);
           }
           fireTableDataChanged();
       }
       public void sortNodes(Comparator c){
           Collections.sort(rows,c);
       }

       public void nodesChanged(ChangeType ct, List list) {
           log.fine("received chagne ct="+ct+" list.size()="+ list.size());
           if(list.size()<=0) return;
           //for now just refresh the whole list
           if(ct==ChangeType.changed){ //some node was changed so repaint these rows
               //work out the 1st and last rows that have changed
               List<Integer> indices = getRowIndices(list);
               if(indices.size()<=0) return;
               int min =getRowCount();
               int max = 0;
               for(Integer i:indices){
                   if(i.intValue()<min) min=i.intValue();
                   if(i.intValue()>max) max=i.intValue();
               }
               log.fine("fireTableRowsUpdated min="+min+" max="+max);
               fireTableRowsUpdated(min,max);
           }
           else{
               setNewParent(node, null);
           }
       }

       private List<Integer> getRowIndices(List nodes) {
           //NB: this is v. inefficient (nested search) - so record the times; if it proves a bottleneck replace
           //with something more efficient (like sort, lookup)
           TaskTimer tt = new TaskTimer();
           List<Integer> indices = new ArrayList<Integer>(nodes.size());
           for (Iterator iterator = nodes.iterator(); iterator.hasNext();) {
               Object node = iterator.next();
               int i = getRow(node);
               if(i>-1){
                  indices.add(i);
               }
           }
           log.fine("took "+tt.stopString()+" to getRowIndices for "+nodes.size()+" nodes out of "+getRowCount());
           return indices;
       }

       public void treeChanged() {
            setNodeAsynchronous(node, null, -1, false, null);
       }
   }
    private int NameColumnIndex=0;
    private int DirColumnIndex=1;

    /**
     * in teh commander style, the 1st row displays the 'current' node and the second the 'parent' node
     * On these rows, one of the columns typically displays the special '.' and '..' values instread of its usual value
     * How these special values appear is determined by teh CommanderCellRenderer
     * However, whcih columns are used to display the special values can be set here (default is column 0)
     */
    public void setNameColumnIndex(int i) {
         NameColumnIndex=i;
    }
    /**
     * in teh commander style, the 1st row displays the 'current' node and the second the 'parent' node
     * On these rows, one of the columns typically displays the special '.' and '..' values instread of its usual value
     * How these special values appear is determined by teh CommanderCellRenderer
     * However, whcih columns are used to display the special values can be set here (default is column 1)
     */
    public void setDirColumnIndex(int i) {
        DirColumnIndex=i;
    }

    public void setModel(final CtreeModel model, final ColumnSpec columns){
        setModel(model);
        setColumnSpec(columns);
    }
    public void setModel(final CtreeModel model) {
        this.model=model;
        tableModel = new TableModelAdapter(model);
        table.setModel(tableModel);//have to do this last, as triggers all kinds of repaints
    }
    public void setColumnSpec(final ColumnSpec columns) {
        this.columnSpec=columns;
        //remove existing cols
        while(table.getColumnCount()>0){
            table.removeColumn(table.getColumnModel().getColumn(0));
        }

        //add cols defined by spec
        for(int index=0;index<model.getAttributeCount();index++){
            final TableColumn tc = new TableColumn(index,columnSpec.getDefaultWidth(index),null,null);
            tc.setHeaderRenderer(new HeaderRenderer(index, columnSpec.getAttributeName(index)));
            table.addColumn(tc);
        }


        //add a renderer that uses spec
        table.setDefaultRenderer(Object.class,new DefaultTableCellRenderer(){
            public Component getTableCellRendererComponent(JTable table1, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                final Object node1 = tableModel.getNode(row);
                final CommanderCellRenderer cellRenderer = columnSpec.getCellRenderer(column);
                SpecialValues sv = null;
                if(tableModel.isNameColumn(column)){
                    if(tableModel.isCurrentRow(row)){
                        sv=SpecialValues.Currentnode;
                    }
                    else if(tableModel.isParentRow(row)) {
                        sv=SpecialValues.Parentnode ;
                    }
                }
                else if(tableModel.isDirColumn(column) && model.hasChildren(node1)){
                    sv=SpecialValues.Nonleaf;
                }
                if(sv!=null){
                    return cellRenderer.getSpecialRendererComponent(
                            CommanderTree.this
                            ,sv
                            ,node1
                            ,isSelected
                            ,hasFocus
                    );
                }
                else{
                    final Object attribueValue = model.getAttributeValue(node1,column);


                    return cellRenderer.getRendererComponent(
                            CommanderTree.this
                            ,attribueValue
                            ,node1
                            ,isSelected
                            ,hasFocus
                    );
                }
            }
        });
    }
    public static interface ColumnSizeListener{
        void columnSizesChanged();
    }
    public void addColumnSizeListener(final ColumnSizeListener csl){
         table.getColumnModel().addColumnModelListener(new TableColumnModelListener() {
            public void columnAdded(TableColumnModelEvent e) { }
            public void columnRemoved(TableColumnModelEvent e) { }
            public void columnMoved(TableColumnModelEvent e) {  }
            public void columnMarginChanged(ChangeEvent e) {
                //todo: is there some way of ignoring 'intermediate' events while the user is dragging the column around?
                csl.columnSizesChanged();
            }
            public void columnSelectionChanged(ListSelectionEvent e) { }
        });
    }

    public static interface OpenNodeListener{
        void openingNode(Object node);
        void finishedOpeningNode(Object node);
    }
    private List<OpenNodeListener> openNodeListeners=new ArrayList<OpenNodeListener>();
    public void addOpenNodeListener(OpenNodeListener onl){
        openNodeListeners.add(onl);
    }
     public void removeOpenNodeListener(OpenNodeListener onl){
        openNodeListeners.remove(onl);
    }
    private void fireOpenNode(boolean opening, Object openNode){
        for(OpenNodeListener onl:openNodeListeners){
            if(opening) onl.openingNode(openNode);
            else onl.finishedOpeningNode(openNode);
        }
    }
    /**
     * WARNING: this meth spawns a new thread to set the node and returns immediately
     * @param node
     * @param selectNode
     * @param selectRow
     * @param grabFocusWhenFinished
     * @param children
     */
    public void setNodeAsynchronous(final Object node, final Object selectNode, final int selectRow, final boolean grabFocusWhenFinished, final List children){
        log.finer("setnode to "+node);
        final Object prev = this.getNode();
        log.finer("setnode="+node+"..");
        //clear selection now.. changing the node will cause teh selection to change anyway, firing a sel-changed
      //  table.clearSelection();
        fireOpenNode(true,node);
        SwingWorker3 sw = new SwingWorker3() {
            public Object construct() {
                try {
                    tableModel.setNewParent(node, children);
                } catch (Exception e) {
                    log.log(Level.WARNING, "exception", e);
                }
                return null;
            }

            public void finished() {
                log.fine("setNodeAsynchronous.finished()");
                fireParentChanged(node, prev);
                fireOpenNode(false,node);
                if(selectNode!=null) selectNode(selectNode);
                else if(selectRow>-1) selectRow(selectRow);
                if(grabFocusWhenFinished){
                    requestFocus();
                }
               // repaint();
            }
        };
        sw.start();
    }
    public static interface NodeListener{
        public void parentNodeChanged(Object newParent, Object oldParent);
    }
    private List<NodeListener> nodeListeners = new ArrayList<NodeListener>(1);
    public void addNodeListener(NodeListener nl){
        nodeListeners.add(nl);
    }
    private void fireParentChanged(Object newParent, Object prevParent){
        for(NodeListener nl:new ArrayList<NodeListener>(nodeListeners)) nl.parentNodeChanged(newParent, prevParent);
    }

    private void openNode(Object node, Object selectNode, int selectRow){
        //Object node =  ((TableModelAdapter)table.getModel()).getNode(row);
        final boolean b = model.hasChildren(node);
        log.finer("opennode haschildren="+b);
        if(b){
            setNodeAsynchronous(node, selectNode, selectRow, false, null);
        }
        else log.finer("ignoring openNode as node has no children");
      //  return b;
    }
    /**
     *
     * @return the 1st node that is selected selected (or null if no nodes are selected)
     */
    public Object getSelectedNode(){
        int ro = table.getSelectedRow();
        if(ro==-1) return null;
        return (tableModel).getNode(ro);
    }
    private void openSelectedRow(){
        Object node = getSelectedNode();
        if(node!=null){
            openNode(node, null, 0);
        }
        else{
            log.finer("ignoring as selected node is null");
        }
    }

    public void setBackground(Color bg) {
        super.setBackground(bg);
        if (table!=null && tableScrollPane!=null) {
            table.setBackground(bg);
            tableScrollPane.getViewport().setBackground(bg);
        }
    }

//    Action clearselAction = new AbstractAction("clear selection") {
//        public void actionPerformed(ActionEvent e) {
//            clearSel();
//        }
//    };
//    public void addSelectionListener(ListSelectionListener l){
//        table.getSelectionModel().addListSelectionListener(l);
//    }
//    public void removeSelectionListener(ListSelectionListener l){
//        table.getSelectionModel().removeListSelectionListener(l);
//    }

//    protected void clearSel() {
////        int[] selected=getSelected(false, true, false);
////        if(selected.length>0){
////            for (int i = 0; i < selected.length; i++) {
////                children.get(selected[i]).setMarked(false);
////            }
////            fireSelectionMarkChanged();
////            fileTableModel.fireTableRowsUpdated(selected[0],selected[selected.length-1]);
////        }
//        table.clearSelection();
//    }


    private void setupActions() {


       // actions.add(setupAction(selectLocationBarAction,KeyStroke.getKeyStroke(KeyEvent.VK_L,Main.ctrl_mask),"lbar"));
//        actions.add(setupAction(new AbstractAction("delete") {
//            public void actionPerformed(ActionEvent e) {
//                delete();
//            }
//        }, KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0),"delete"));
//        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE,0),"delete");
//        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0),"delete");

        //remove bindings for tab key (TAB should do nothing)

        // setting keystroke to null, still has table doing its default thing.. why? NS: looking at ui code there seems
        //to be a UI inputmap that has replaced the table's inputmap on metal..
        final KeyStroke tabKs = KeyStroke.getKeyStroke(KeyEvent.VK_TAB,0);

        table.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(tabKs,null);
        table.getInputMap(JComponent.WHEN_FOCUSED).put(tabKs,null);
        table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(tabKs,null);
        final InputMap uiInputMap = SwingUtilities.getUIInputMap(table,JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        if (uiInputMap!=null) {
            uiInputMap.put(tabKs,null);
        }

        List<Action> actions=new ArrayList<Action>();
      // actions.add(setupAction(clearselAction,KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),"clearsel"));
        actions.add(setupAction(firstRow,KeyStroke.getKeyStroke(KeyEvent.VK_HOME,0),"fr"));
        actions.add(setupAction(lastRow,KeyStroke.getKeyStroke(KeyEvent.VK_END,0),"lr"));
        actions.add(setupAction(openAction,KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),"enterpressed"));
        table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,0),"enterpressed");
        actions.add(setupAction(openParentAction,KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,0),"openparent"));

        final InputMap inputMap = table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        final ActionMap actionMap = table.getActionMap();
        for(Action a:actions){
            inputMap.put((KeyStroke) a.getValue(Action.ACCELERATOR_KEY),a.getValue(Action.ACTION_COMMAND_KEY));
            actionMap.put(a.getValue(Action.ACTION_COMMAND_KEY),a);
        }
    }

//    public Action getAction(int inputMapCondition, KeyStroke ks){
//        String key= (String) table.getInputMap(inputMapCondition).get(ks);
//        return table.getActionMap().get(key);
//    }
    public void registerInput(int inputMapCondition, KeyStroke ks, String actionkey){
        table.getInputMap(inputMapCondition).put(ks,actionkey);
    }
    public void registerAction(String key, Action a){
        table.getActionMap().put(key,a);
    }

    private Action setupAction(Action copyAction, KeyStroke keystroke, String name){
        copyAction.putValue(Action.ACCELERATOR_KEY,keystroke);
        copyAction.putValue(Action.ACTION_COMMAND_KEY,name);
        copyAction.putValue(Action.NAME,name);
        return copyAction;
    }



    private void headerClicked(int col, boolean ctrlDown, boolean shiftDown) {
        int attributeIndex = ((HeaderRenderer) table.getColumnModel().getColumn(col).getHeaderRenderer()).attributeIndex;
        if(!model.isComparable(attributeIndex)){
            log.fine("ignoring header click as col="+attributeIndex+" is not comparable");
            return;
        }
        //if ctrldown && >1cols sorted
        try {
            final NodeComparator comparator;
            if(this.comparator!=null){
                comparator=(NodeComparator) this.comparator.clone();
            }
            else{
                comparator=new NodeComparator(model);
            }

            if(comparator.getIndex(attributeIndex)!=-1){//sorted by this column
                if(shiftDown){ //remove this col from sort
                    comparator.removeFromCompare(attributeIndex);
                }
                else if(comparator.gtOne() && !ctrlDown){//if multisort and ctrl not down, clear multisort and sort by this cols default order
                    comparator.setCompare(attributeIndex);
                }
                else{//else reverse direction
                    comparator.toggleDirection(attributeIndex);
                }
            }
            else if(ctrlDown){//add this col to sorted
                comparator.addCompare(attributeIndex);
            }
            else{ //else clear sort and sort by this col
                comparator.setCompare(attributeIndex);
            }

            sort(comparator);
        } catch (CloneNotSupportedException e) {
            log.log(Level.WARNING, "exception", e);
        }
    }
    private void sort(NodeComparator nodeComparator){
        log.fine("sort table comp="+nodeComparator);
        tableModel.sortNodes(nodeComparator);

        this.comparator=nodeComparator;
        fireSortChanged(nodeComparator);
        final JTableHeader th = table.getTableHeader();
        //th.invalidate();
        tableModel.fireTableDataChanged();
        th.repaint();
//        th.invalidate();
//        th.getParent().validate();
    }


    //listeners
    public static interface SortListener{
        public void sortChanged(NodeComparator newComparator);
    }
    private List<SortListener> sortListeners=new ArrayList<SortListener>(1);
    public void addSortListener(SortListener sl){
        sortListeners.add(sl);
    }
    private void fireSortChanged(NodeComparator newComparator){
        for(SortListener sl:sortListeners) sl.sortChanged(newComparator);
    }

    public synchronized void addFocusListener(FocusListener l) {
        table.addFocusListener(l);
    }

    public boolean requestFocusInWindow() {
        return table.requestFocusInWindow();
    }

    public void requestFocus() {
        table.requestFocus();
    }
    public synchronized MouseListener[] getMouseListeners() {
        return table.getMouseListeners();
    }
    public synchronized void removeMouseListener(MouseListener l) {
        table.removeMouseListener(l);
    }
    public synchronized void addMouseListener(MouseListener l) {
        table.addMouseListener(l);
    }
    public synchronized KeyListener[] getKeyListeners() {
        return table.getKeyListeners();
    }
    public synchronized void removeKeyListener(KeyListener l) {
        table.removeKeyListener(l);
    }
    public synchronized void addKeyListener(KeyListener l) {
        table.addKeyListener(l);
    }
    public void setDefaultComparison(int attribute, SortDirection direction){
        if(comparator!=null){
            comparator.setDefaultCompare(attribute,direction);
        }
    }


    private class TableAction extends AbstractAction{
        private Action tableAction;
        public TableAction(String name, Action tableAction) {
            super(name);
            this.tableAction = tableAction;
        }
        public void actionPerformed(ActionEvent e) {
            table.requestFocusInWindow();
            e.setSource(table);
            tableAction.actionPerformed(e);
        }
    }
     /**
     * find the relevant action in our inputmap and return an action
     * that 1) requests focus for the table 2) executes that action to the table
     * @param ks
      */
     public Action getForwardAction(int inputMapcondition, final KeyStroke ks) {
         try {
             ActionMap tableMap = table.getActionMap();
             InputMap table_im = table.getInputMap(inputMapcondition);
             String key = (String) table_im.get(ks);
             log.finer("tableky="+key);
             Action tableUp=tableMap.get(key);
             log.finer("tableup="+tableUp);
             return new TableAction(key,tableUp) ;
         } catch (Exception e) {
             log.log(Level.WARNING
                     ,"couldn't map key " +
                     "ks="+ks
                     ,e);
         }
         return null;
    }

    public boolean hasFocus() {
        return super.hasFocus() || table.hasFocus();
    }

}
