package uk.co.hcsoftware.commander.tree;

import java.util.logging.Logger;
import java.util.List;


/**
 * Title: CtreeModel
 * Copyright:    Copyright (c) 31-Oct-2005
 *
 * @author ns
 */
public interface CtreeModel<N> {
    public static final Logger log = Logger.getLogger(CtreeModel.class.getName());
    /**
     *
     * @param node
     * @return all children of this parent
     */
    List<N> getChildren(N node);
    /**
     *
     * @param node
     * @return true if this node is not a leaf
     */
    boolean hasChildren(N node);

    /**
     *
     * @param node
     * @return parentnode of this node
     */
    N getParent(N node);

    /**
     *
     * @return number of attributes each node has
     */
    int getAttributeCount();

    /**
     *
     * @param index
     * @return name of the attribute which has this index
     */
    String getAttributeName(int index);

    /**
     *
     * @param node
     * @param attribute
     * @return    value of the attribue at this index, belonging to this node
     */
    Object getAttributeValue(N node, int attribute);


    /**
     *
     * @param attribute
     * @return true if compare is implemented for this attribute
     */
    boolean isComparable(int attribute);
    /**
     *
     * @param node1
     * @param node2
     * @param attribute
     * @return 0,1,-1 based on comparison between these nodes using this attribute
     */
    int compare(N node1, N node2, int attribute);
    /**
     *
     * @param attribute
     * @return wether by default nodes should be sorted in asc or desc order when sorted on this attribute
     */
    SortDirection getDefaultSortDirection(int attribute);

    
    public static interface ChangeListener<N>{
        public static enum ChangeType{added,deleted,changed};
        void nodesChanged(ChangeType ct, List<N> nodes);
        void treeChanged();
    }
    public void addChangeListener(ChangeListener<N> listener);
    public void removeChangeListener(ChangeListener<N> listener);
}
