package uk.co.hcsoftware.commander.tree;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.io.Serializable;
import java.util.logging.Logger;


/**
 * Title: DefaultCommanderCellRenderer
 * Copyright:    Copyright (c) 14-Nov-2005
 *
 * @author spurlinn
 */
public class DefaultCommanderCellRenderer  extends JLabel implements CommanderCellRenderer, Serializable
    {
    public static final Logger log = Logger.getLogger(DefaultCommanderCellRenderer.class.getName());

    protected static Border noFocusBorder = new EmptyBorder(1, 1, 1, 1);

    public DefaultCommanderCellRenderer() {
        super();
        setOpaque(true);
        setBorder(noFocusBorder);
    }

    public Component getSpecialRendererComponent(CommanderTree ct, SpecialValues value, Object node, boolean isSelected, boolean hasFocus) {
        final Object val;
        if(value==SpecialValues.Currentnode) val= ".";
        else if(value==SpecialValues.Parentnode) val= "..";
        else if(value==SpecialValues.Nonleaf) val= "<DIR>";
        else val="";
        final Component comp = getComp(isSelected, ct, hasFocus, val);
        setHorizontalAlignment(JLabel.HORIZONTAL);
        return comp;
    }

    public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
        setHorizontalAlignment(JLabel.LEFT);
        return getComp(isSelected, ct, hasFocus, attributeValue);

    }

    private Component getComp(boolean isSelected, CommanderTree ct, boolean hasFocus, Object attributeValue) {
        if (isSelected) {
            super.setForeground(ct.getSelectionForeground());
            super.setBackground(ct.getSelectionBackground());
        }
        else {
            super.setForeground(ct.getForeground());
            super.setBackground(ct.getBackground());
        }

        setFont(ct.getFont());

        if (hasFocus) {
            Border border = null;
            if (isSelected) {
                border = UIManager.getBorder("Table.focusSelectedCellHighlightBorder");
            }
            if (border == null) {
                border = UIManager.getBorder("Table.focusCellHighlightBorder");
            }
            setBorder(border);
        } else {
            setBorder(noFocusBorder);
        }

        setText((attributeValue == null) ? "" : attributeValue.toString());

        return this;
    }
}
