package uk.co.hcsoftware.commander.tree;

import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;


public class NodeComparator implements Comparator, Cloneable{
    Logger log = Logger.getLogger(NodeComparator.class.getName());

    private final CtreeModel model;

    public NodeComparator(CtreeModel model) {
        this.model = model;
        defaultCompare=new Compare(DEFAULT_COMPARETYPE,model.getDefaultSortDirection(DEFAULT_COMPARETYPE));
     //   addCompare(DEFAULT_COMPARETYPE, )
    }



    protected Object clone() throws CloneNotSupportedException {
        return super.clone();
    }

    public int getIndex(int comparisonColumn) {
        int i=0;
        for(Compare c:compares){
            if(c.ct ==comparisonColumn) return i;
            i++;
        }
        return -1;
    }
    public boolean isForwardDirection(int index){
        return compares.get(index).direction==SortDirection.ascending;
    }

    public boolean gtOne() {
        return compares.size()>1;
    }

    public void removeFromCompare(int ct) {
        for (Iterator<Compare> iterator = compares.iterator(); iterator.hasNext();) {
            Compare compare = iterator.next();
            if(compare.ct ==ct) iterator.remove();
        }
    }


    private class Compare{
        private int ct;
        private SortDirection direction;
        public Compare(int ct, SortDirection direction) {
            if(!model.isComparable(ct)){
                throw new IllegalArgumentException("model says attribute="+ct+" is not comparable! and yet we are trying to build a comparator for it!");
            }
            this.ct = ct;
            this.direction = direction;
        }

        public int getCt() {
            return ct;
        }

        public SortDirection isDirection() {
            return direction;
        }

        public String toString() {
            return "ct="+ct+" "+(direction==SortDirection.ascending?"asc":"desc");
        }
    }

    public List<Compare> getCompares() {
        return compares;
    }

    private final static int DEFAULT_COMPARETYPE = 0;
    LinkedList<Compare> compares=new LinkedList<Compare>();
    public void addCompare(int ct){
       addCompare(ct,model.getDefaultSortDirection(ct));
    }

    public String toString() {
        return "compares="+compares;
    }

    public void setCompare(int ct){
       compares.clear();
       addCompare(ct);
    }
    public void addCompare(int ct, SortDirection direction){
       compares.add(new Compare(ct,direction));

    }
    public void reset(){
        compares.clear();
    }
    public void setDirection(int ct, SortDirection newDirection){
        for(Compare c:compares){
            if(c.ct==ct) c.direction=newDirection;
        }
    }
    public void toggleDirection(int ct) {
        for(Compare c:compares){
            if(c.ct==ct){
                if(c.direction==SortDirection.ascending){
                    c.direction=SortDirection.descending;
                }
                else if(c.direction==SortDirection.descending){
                    c.direction=SortDirection.ascending;
                }
                else{
                    throw new IllegalArgumentException("cannot reverse sort direction as do not recognise dir="+c.direction);
                }
            }
        }
    }

    private int compareLikeTypes(Object f1, Object f2){
        for(Compare c:compares){
            int i = compare(c, f1, f2);
            if(i!=0){
                return c.direction==SortDirection.ascending?i:-i;
            }
        }
        //try default if still 0
        return compare(defaultCompare,f1,f2);
    }
    Compare defaultCompare;//=new Compare(DEFAULT_COMPARETYPE,model.getDefaultSortDirection(DEFAULT_COMPARETYPE));

    public void setDefaultCompare(int attribute, SortDirection direction) {
        this.defaultCompare = new Compare(attribute,direction);
    }

    private int compare(Compare c, Object f1, Object f2) {
        return model.compare(f1,f2,c.ct);
    }
    private boolean directoriesFirst=true;

    public void setDirectoriesFirst(boolean directoriesFirst) {
        this.directoriesFirst = directoriesFirst;
    }

    public int compare(Object node1, Object fr2) {
        int i;
        if(directoriesFirst && model.hasChildren(node1)){
            if(model.hasChildren(fr2)){
                i=compareLikeTypes(node1,fr2);
            }
            else return -1;
        }
        else if(model.hasChildren(fr2)) return 1;
        else{
            i=compareLikeTypes(node1,fr2);
        }
        if(i==0 && !node1.equals(fr2)){
            i= node1.hashCode()-fr2.hashCode();
            log.warning("identically ordered nodes for which equals returns false! f1="+node1+" f2="+fr2+" splitting on hashcode ret="+i);
        }
        return i;
    }
}
