package uk.co.hcsoftware.commander.tree;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;


/**
 * wraps a treeModel in a CtreeModel
 * Title: TreeModelAdapter
 * Copyright:    Copyright (c) 15-Nov-2005
 *
 * @author spurlinn
 */
public abstract class TreeModelAdapter implements CtreeModel<TreeNode> {
    public static final Logger log = Logger.getLogger(TreeModelAdapter.class.getName());

  //  private TreeModel tm;

    public TreeModelAdapter(TreeModel tm) {
     //   this.tm = tm;
        tm.addTreeModelListener(new TreeModelListener() {
            private List<TreeNode> getNodes(Object[] children){
                List<TreeNode> l = new ArrayList<TreeNode>(children.length);
                for(Object o:children){
                    l.add((TreeNode) o);
                }
                return l;
            }
            public void treeNodesChanged(TreeModelEvent e) {
                 fireNodeChange(ChangeListener.ChangeType.changed,getNodes(e.getChildren()));
            }

            public void treeNodesInserted(TreeModelEvent e) {
                fireNodeChange(ChangeListener.ChangeType.added,getNodes(e.getChildren()));
            }

            public void treeNodesRemoved(TreeModelEvent e) {
                fireNodeChange(ChangeListener.ChangeType.deleted,getNodes(e.getChildren()));
            }

            public void treeStructureChanged(TreeModelEvent e) {
                log.fine("structure changed: ignoring");
                fireTreeChanged();
            }
        });
    }


    /**   
     * @return all children of this parent
     */
    public List<TreeNode> getChildren(TreeNode treeNode) {
        List<TreeNode> tn = new ArrayList<TreeNode>(treeNode.getChildCount());
        for(int i=0;i<treeNode.getChildCount();i++){
            tn.add(treeNode.getChildAt(i));
        }
        return tn;
    }

    /**
     * @return true if this node is not a leaf
     */
    public boolean hasChildren(TreeNode treeNode) {
        return !treeNode.isLeaf();
    }

    /**
     * @return parentnode of this node
     */
    public TreeNode getParent(TreeNode treeNode) {
        return treeNode.getParent();
    }

    private void fireNodeChange(ChangeListener.ChangeType ct, List<TreeNode> nodes){
        log.fine("nodeschange ct="+ct+" nodecount="+nodes.size()+" firing to "+listeners.size()+" listeners");
         for(ChangeListener cl:new ArrayList<ChangeListener>(listeners)){
             cl.nodesChanged(ct,nodes);
         }
    }
    private void fireTreeChanged(){
         for(ChangeListener cl:new ArrayList<ChangeListener>(listeners)){
             cl.treeChanged();
         }
    }

    private List<ChangeListener<TreeNode>> listeners = new ArrayList<ChangeListener<TreeNode>>();
    public void addChangeListener(ChangeListener<TreeNode> changeListener) {
         listeners.add(changeListener);
    }

    public void removeChangeListener(ChangeListener<TreeNode> changeListener) {
         listeners.remove(changeListener);
    }

}
