package uk.co.hcsoftware.commander.treeplugin;

import uk.co.hcsoftware.commander.tree.CommanderTree;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.logging.Logger;
import java.util.logging.Level;


/**
 * add itself to a  commandertree, and provides the functionality to jump to some row as letters are typed
 *
 * Title: AutocompleteJumper
 * Copyright:    Copyright (c) 30-Nov-2005
 *
 * @author spurlinn
 */
public class AutocompleteJumper extends KeyAdapter {
    public static final Logger log = Logger.getLogger(AutocompleteJumper.class.getName());
    private final CommanderTree ct;
    private final RowIdentifier ri;
    private static int id=0;
    private final int myId;
    private long lastTimeCharTyped=-1;
    private final long maxInterval_ms=2000;

    public static interface RowIdentifier{
        public String getRowId(Object node);
    }

    public AutocompleteJumper(CommanderTree ct, RowIdentifier ri) {
        this.ct = ct;
        this.ri = ri;
        myId=id++;
        ct.addKeyListener(this);
    }
    private String isearch="";   //todo: reset isearch after some amount of time
    public void keyTyped(KeyEvent e) {   //jump to letter if typing
        char c = e.getKeyChar();
        //if it's more than 2s since last key was typed then reset isearch
        long time=System.currentTimeMillis();
        if(lastTimeCharTyped>0 && time-lastTimeCharTyped > maxInterval_ms){
            reset();
        }
        lastTimeCharTyped=time;

        log.log(Level.FINER,"chartyped="+c+" jumper="+toString()+" ke="+e/*,new Throwable()*/);
        if(c==KeyEvent.VK_BACK_SPACE){
            if(isearch.length()==1)
                reset();
            else if(isearch.length()>1) isearch= isearch.substring(0,isearch.length()-1);
        }
        else if(c==KeyEvent.VK_ESCAPE || c==KeyEvent.VK_DOWN || c==KeyEvent.VK_ENTER || c==KeyEvent.VK_UP){
            reset();
            return;
        }
        else if(e.getModifiers()==0 && e.getModifiersEx()==0 /*&& (Character.isLetter(c) || Character.isDigit(c))*/){
            isearch+=Character.toLowerCase(c);  //don't ignore non-alphanumeric as there might be punctuation in name
        }
        else{
            log.fine("ignorning char="+c);
            return;
        }


        // if(e.getModifiers()==0 && e.getModifiersEx()==0 && (Character.isLetter(c) || Character.isDigit(c))){
        //  char upper = Character.toUpperCase(c);
        //select the first vfile beginning with this letter
        int row=ct.getFirstSelectedRow()+1;// start from selrow+1 and visit every row in sequence (i.e. wrap at end)
        if(isearch.length()>1 && row>0){ //if there is an isearch ongoing then start searching from the current row
            //not the nextrow (in practise this seems more intuitive)
            row--;
        }
        // (if selrow==-1 does the right thing)
        final int rowCount = ct.getRowCount();
        for(int i=0;i<rowCount;i++,row++){
            if(row>=rowCount) row=0;
            Object node = ct.getNode(row);
            if (!ct.isParent(row) && !ct.isCurrent(row)) {
                String name = ri.getRowId(node);//vf.getName();
                if(name==null) continue;
                name=name.toLowerCase();
                if (name.startsWith(isearch)) {
                    // char c1 = name.charAt(0);
                    //if(c==c1 || upper==c1){
                    log.finer("found matching row for char="+c+" index="+row+" fname="+name);
                    ct.selectRow(row);
                    return;
                    //  }
                }
            }
        }
        log.finer("no vfile having firstletter matching c="+c);
        //  }
        // else log.finer("ignorning char="+c);
    }

    private void reset() {
        isearch="";
    }

    public String toString() {
        return "autocomplete jumper id="+myId;
    }
}
