package uk.co.hcsoftware.commander.treeplugin;

import uk.co.hcsoftware.commander.tree.CommanderTree;
import uk.co.hcsoftware.commander.gui.ThemeListener;
import uk.co.hcsoftware.commander.gui.Themeable;

import javax.swing.*;
import javax.swing.border.Border;
import java.util.logging.Logger;
import java.awt.*;


/**
 * convenience class for managing theme colours and customizing controls based on the chosen theme
 *
 * Title: ColorHandler
 * Copyright:    Copyright (c) 28-Nov-2005
 *
 * @author spurlinn
 */
public class ColorHandler implements Themeable{
    public static final Logger log = Logger.getLogger(ColorHandler.class.getName());

    private final CommanderTree ct;

    public ColorHandler(CommanderTree ct) {
        this.ct = ct;
        new ThemeListener(this);
    }

      //Themeable interface
    private Color backColor,foreColor,selColor;
    public void setColors(Color backColor, Color foreColor, Color selColor) {
        ct.setBackground(backColor);
        ct.setForeground(foreColor);
        this.backColor=backColor;
        this.foreColor=foreColor;
        this.selColor=selColor;
        rowSelBorder=BorderFactory.createLineBorder(backColor,1);
        markSelBorder=BorderFactory.createLineBorder(selColor,1);//important that mark be differentiable from sel
        noSelBorder=BorderFactory.createLineBorder(backColor,1);;
        ct.repaint();
    }
    private Border rowSelBorder,markSelBorder,noSelBorder;
    public void setColors(JLabel jl, boolean isSelected, boolean marked){
        if(backColor!=null&&foreColor!=null&&selColor!=null){
            if(isSelected){
                jl.setBackground(selColor);
                jl.setForeground(backColor);
                 jl.setBorder(rowSelBorder);
            }
            else if (marked){
                jl.setBackground(selColor);
                jl.setForeground(backColor);
                jl.setBorder(markSelBorder);
            }
            else{
                jl.setBackground(backColor);
                jl.setForeground(foreColor);
                jl.setBorder(noSelBorder);
            }
        }
    }
}
