package uk.co.hcsoftware.commander.treeplugin;

import uk.co.hcsoftware.commander.Main;
import uk.co.hcsoftware.commander.tree.CommanderTree;
import uk.co.hcsoftware.commander.tree.CtreeModel;

import java.util.logging.Logger;
import java.util.prefs.Preferences;


/**
 * utility class, helps in saving columnsizes to prefs
 * Title: ColumnSizeSaver
 * Copyright:    Copyright (c) 28-Nov-2005
 *
 * @author spurlinn
 */
public class ColumnSizeSaver {
    public static final Logger log = Logger.getLogger(ColumnSizeSaver.class.getName());
    private final CommanderTree ct;
    private String[] keyNames;

    private ColumnSizeSaver(CommanderTree ct){
        this.ct = ct;
        ct.addColumnSizeListener(new CommanderTree.ColumnSizeListener() {
            public void columnSizesChanged() {
                 saveColumnSizes();
            }
        });
    }
    /**
     * save columnsizes against these prefkeys - match with ct-column by order
     * @param prefKeys
     */
    public ColumnSizeSaver(CommanderTree ct, String... prefKeys){
        this(ct);
        this.keyNames=prefKeys;
    }
    /**
     * save columnsizes with this prefix+model-name value
     * @param uniquePrefix
     */
    public ColumnSizeSaver(CommanderTree ct, String uniquePrefix){
        this(ct);
        CtreeModel model = ct.getModel();
        keyNames = new String[model.getAttributeCount()];
        for(int i=0;i<model.getAttributeCount();i++){
            keyNames[i]=uniquePrefix+"."+model.getAttributeName(i);
        }
    }

    public void saveColumnSizes() {
        log.fine("saving columnwidths");
        int i=0;
        for(String keyName:keyNames){
            columnSizePrefs.putInt(keyName,ct.getColumnWidth(i++));
        }
//        columnSizePrefs.putInt("mark_column_w",ct.getColumnWidth(0));
//        columnSizePrefs.putInt("name_column_w",ct.getColumnWidth(1));
//        columnSizePrefs.putInt("size_column_w",ct.getColumnWidth(2));
//        columnSizePrefs.putInt("date_column_w",ct.getColumnWidth(3));
    }

    private static final Preferences columnSizePrefs = Preferences.userNodeForPackage(Main.class);
    public int getColWidth(int attributeIndex, int defaultWidth) {
        final String propertyName = keyNames[attributeIndex];
        final int anInt = columnSizePrefs.getInt(propertyName,defaultWidth);
        if(anInt<2){
            log.warning("prefval for col="+propertyName+" was unsensbile="+anInt+" using default="+defaultWidth);
            return defaultWidth;
        }
        return anInt;
    }
}
