package uk.co.hcsoftware.commander.treeplugin;

import uk.co.hcsoftware.commander.tree.CommanderTree;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;
import java.awt.event.ActionEvent;


/**
 * extends commandertree selection, allowing 'marked' files as well call mark() to mark all currently selected nodes
 *
 * Title: SelectionMarkHandler
 * Copyright:    Copyright (c) 28-Nov-2005
 *
 * @author spurlinn
 */
public class SelectionMarkHandler<E> {
    public static final Logger log = Logger.getLogger(SelectionMarkHandler.class.getName());
    private CommanderTree ct;

    public SelectionMarkHandler(CommanderTree ct) {
        this.ct = ct;
        ct.addSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                fireSelectionMarkChanged();
            }
        });
    }

    public void clearMarkedAndSelected() {
        ct.clearSelection();
        clearMarked();
    }



//marking
    public interface SelectionMarkListener{
        public void selectionMarkChanged(int[] selectedFiles);
    }
    private List<SelectionMarkListener> slms=new ArrayList<SelectionMarkListener>();
    public void addSelectionMarkListener(SelectionMarkListener slm){
        slms.add(slm);
    }
    private void fireSelectionMarkChanged(){
        int[] selectedFiles=getSelectedOrMarked(true, true,false);
        for(SelectionMarkListener sml:new ArrayList<SelectionMarkListener>(slms)){
            sml.selectionMarkChanged(selectedFiles);
        }
    }

    public void mark() {
        int[] rows = ct.getSelectedRows(false,false);
        if (rows.length>0) {
            List<E> nodes = new ArrayList<E>(rows.length);
            int row;
            for (int i = 0; i < rows.length; i++) {
                row= rows[i];
                E vf = (E) ct.getNode(row);
                nodes.add(vf);
                if(!marked.remove(vf)){
                    marked.add(vf);
                }
            }
            //tell the tree to repaint these parts
            fireSelectionMarkChanged();
        }
    }
    public int[] getSelectedOrMarked(boolean includeSelected, boolean includeMarkedFiles, boolean includeParents){
        int count = ct.getRowCount();
        int[] tmp = new int[count];
        int i=0;
        for(int ro=0;ro<count;ro++){
            if(includeSelected && ct.isSelected(ro)
                    || includeMarkedFiles&& isMarked((E) ct.getNode(ro)) ){
                if(!includeParents && (ct.isParent(ro) || ct.isCurrent(ro))) continue;
                tmp[i++]=ro;
            }
        }

        int[] ret=new int[i];
        System.arraycopy(tmp,0,ret,0,i);
        return ret;
    }

    private Set<E> marked = new HashSet<E>();
    public void clearMarked(){
       // List<E> nodes = new ArrayList<E>(marked);
        marked.clear();
        fireSelectionMarkChanged();
    }

    public boolean isMarked(E vf){
        return marked.contains(vf);
    }


    private Action MarkAction = new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
           mark();
        }
    };
    private Action ClearAction = new AbstractAction() {
        public void actionPerformed(ActionEvent e) {
            clearMarkedAndSelected();
        }
    };

    public Action getMarkAction() {
        return MarkAction;
    }

    public Action getClearAction() {
        return ClearAction;
    }
}
