package uk.co.hcsoftware.commander.treeplugin;

import uk.co.hcsoftware.commander.gui.HostFrame;
import uk.co.hcsoftware.commander.tree.CommanderTree;

import java.awt.*;
import java.util.logging.Logger;


/**
 * Title: StatusUpdater
 * Copyright:    Copyright (c) 28-Nov-2005
 *
 * @author spurlinn
 */
public class StatusUpdater {
    public static final Logger log = Logger.getLogger(StatusUpdater.class.getName());

    private CommanderTree ct;
    private HostFrame hf;
    private SelectionMarkHandler sml;

    public StatusUpdater(final CommanderTree ct, final HostFrame hf, SelectionMarkHandler smh) {
        this.ct = ct;
        this.hf = hf;
        this.sml = smh;
        sml.addSelectionMarkListener(new SelectionMarkHandler.SelectionMarkListener() {
            public void selectionMarkChanged(int[] selectedFiles) {
                // update status label
                updateStatus(getStatusText(selectedFiles.length));
            }
        });

        ct.addOpenNodeListener(new CommanderTree.OpenNodeListener() {
            String status,s;
            public void openingNode(Object node) {
                s =hf.getStatus();
                status = "Reading...";
                ct.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                updateStatus(status);
                ignoreStatusUpdates=true;
                //todo: disable ct while this is going on
                log.fine("opening url="+node+"..");
            }

            public void finishedOpeningNode(Object node) {
                ct.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                ignoreStatusUpdates=false;
                updateStatus(getStatusText(-1));
            }
        });
    }
     private void updateStatus(String statusText) {
        if(ignoreStatusUpdates) return;
        hf.setStatus(statusText);
    }
    private boolean ignoreStatusUpdates=false;

    /**
     *
     * @param selectedcount can be -1 if don't know
     * @return
     */
    public String getStatusText(int selectedcount) {
        StringBuffer sb = new StringBuffer();
        sb.append(ct.getRowCount()==1?"1 file":ct.getRowCount()+" files");
        final int len;
        if(selectedcount<0){
            int[] selectedFiles = sml.getSelectedOrMarked(true,true,false);
            len = selectedFiles.length;
        }
        else len = selectedcount;
        if(len>0) sb.append(", "+len+" selected/marked");

//        if(filter!=null){
//            sb.append(", filter: "+filter.getDescription());
//        }
        //todo: add sort text?
        return sb.toString();
    }
}
