package uk.co.hcsoftware.commander.treeplugin;

import java.util.logging.Logger;
import java.util.List;
import java.io.InputStream;
import java.awt.datatransfer.DataFlavor;


/**
 *
 * a hierarical structure, each node of which may have an associated inputstream
 * intended for use with Transferables; e.g. you want to transfer files out of a zip or some logical filestore which
 * doesn't actually use files. Each file is an inputstream node and the nodes are connected together in the usual way
 * It is up to the target to make of this structure what it will (e.g. preserver the sturcture, convert to a flat list..)
 *
 * Title: TreeStream
 * Copyright:    Copyright (c) 02-Dec-2005
 *
 * @author spurlinn
 */
public class TreeStream {
    public static final Logger log = Logger.getLogger(TreeStream.class.getName());
    public static final DataFlavor DATAFLAVOR=new DataFlavor(TreeStream.class,"Tree of Streams");
    private List<Node> roots;

    public TreeStream(List<Node> roots) {
        this.roots = roots;
    }

    /**
     *
     * @return list of root nodes (which may have descendants)
     */
    public List<Node> getRoots() {
        return roots;
    }
    /**
     * call this to indicate you do not need to call getInputStream() on any more nodes, so the underlying resources can be disposed
     */
    public void finishedReading(){}

    public static interface Node{
        /**
         * name of this particular node, should be unique wrt its siblings
         * @return
         */
        String getName();
        /**
         *
         * @return immediate children of this node (if is a leaf return an empty list)
         */
        List<Node> getChildren();
        /**
         *
         * @return may be null; inputstream for this node (if it is a leaf) otherwise null; in which case it is a directory
         * and has 0 or more children. There is no provision for directories that have inputstreams or leaves which don't have inputstreams
         */
        InputStream getInputStream();
    }

}
