package uk.co.hcsoftware.commander.treeplugin.file;

import net.sourceforge.clearcase.simple.ClearcaseException;
import net.sourceforge.clearcase.simple.ClearcaseFactory;
import net.sourceforge.clearcase.simple.IClearcase;
import net.sourceforge.clearcase.simple.ClearcaseUtil;
import uk.co.hcsoftware.commander.NameComparator;
import uk.co.hcsoftware.commander.tree.SortDirection;

import java.util.*;
import java.util.logging.Level;
import java.io.File;


/**
 * Title: FileModel
 * Copyright:    Copyright (c) 07-Dec-2005
 *
 * @author spurlinn
 */
public class ClearcaseModel extends FileModel{
    protected static final int MARK=0;
    protected static final int NAME=1;
    protected static final int SIZE=2;
    protected static final int CO=3;
    protected static final int CC=4;
    protected static final int DATE=5;
    protected static final int EXT=6;
    protected static final int SYM=9,DIF=7,HI=8;

    private IClearcase cc;
    public static int columncount=10;

    public ClearcaseModel() {
        try {
            cc = ClearcaseFactory.getInstance().createInstance(ClearcaseFactory.JNI);
          //  System.out.println("vobs="+ClearcaseUtil.findVOBs(cc));
         //   cc.getViewRoot()
           // System.out.println("ishijacked="+cc.isHijacked(new File(),));
        }  catch (ClearcaseException e) {
            log.log(Level.WARNING, "exception", e);
        }
    }

    public ClearcaseModel(final FileTreePlugin fileTreePlugin) {
        super(fileTreePlugin);
    }
    public CcVfile getCcVfile(Vfile vf){
        return new CcVfile(vf);
    }

    public List<CcVfile> findCheckouts(CcVfile node) throws ClearcaseException {
        final String absolutePath = node.getAbsolutePath();
        log.fine("find checkouts from path="+absolutePath);
        List files = ClearcaseUtil.findCheckouts(cc,absolutePath);
        List<CcVfile> res = new ArrayList<CcVfile>(files.size());
        log.fine("found "+files.size()+" checkouts");
        for (Iterator iterator = files.iterator(); iterator.hasNext();) {
            String s=(String) iterator.next();
            res.add(new CcVfile(new File(s)));
        }
        return res;
    }
    public List<CcVfile> findViewPrivates(CcVfile node) throws ClearcaseException {
//        final String absolutePath = node.getAbsolutePath();
//        log.fine("find checkouts from path="+absolutePath);
//        List files = ClearcaseUtil.findCheckouts(cc,absolutePath);
//        List<CcVfile> res = new ArrayList<CcVfile>(files.size());
//        log.fine("found "+files.size()+" checkouts");
//        for (Iterator iterator = files.iterator(); iterator.hasNext();) {
//            String s=(String) iterator.next();
//            res.add(new CcVfile(new File(s)));
//        }
//        return res;
        //something like cleartool ls -recurse -view_only
       //@see findcheckouts cc.cleartool("ls -recurse -view_only")
        return Collections.emptyList();//todo
    }


    public class CcVfile extends Vfile{
        public CcVfile(File f) {
            super(f);
        }

        public CcVfile(Vfile f) {
            super(f==null?null:f.getFile());
            if(f==null) throw new IllegalArgumentException("f is null!");
        }

        BitSet cached = new BitSet(5), attribs = new BitSet(5);
        public boolean isHijacked() {
            if(!cached.get(0)){
               attribs.set(0,cc.isHijacked(getAbsolutePath(),false));
               cached.set(0,true);
            }
            return attribs.get(0);
        }
        public boolean isCheckedOut() {
           // return cc.isCheckedOut(getAbsolutePath(),false);
            if(!cached.get(1)){
               attribs.set(1,cc.isCheckedOut(getAbsolutePath(),false));
               cached.set(1,true);
            }
            return attribs.get(1);
        }
        public boolean isSymbolicLink() {
          //  return cc.isSymbolicLink(getAbsolutePath());
            if(!cached.get(2)){
               attribs.set(2,cc.isSymbolicLink(getAbsolutePath()));
               cached.set(2,true);
            }
            return attribs.get(2);
        }

        public boolean isElement() {
           // return cc.isElement(getAbsolutePath());
            if(!cached.get(3)){
               attribs.set(3,cc.isElement(getAbsolutePath()));
               cached.set(3,true);
            }
            return attribs.get(3);
        }

        public boolean isDifferent() {
         //   return cc.isDifferent(getAbsolutePath());
            if(!cached.get(4)){
               attribs.set(4,cc.isDifferent(getAbsolutePath()));
               cached.set(4,true);
            }
            return attribs.get(4);
        }
    }

    public Vfile getParent(Vfile vfile) {
        return new CcVfile( super.getParent(vfile));
    }

    public List<Vfile> getChildren(Vfile vfile) {
        final List<Vfile> children = super.getChildren(vfile);
        List<Vfile> children2=new ArrayList<Vfile>(children.size());
        for(Vfile v:children) children2.add(new CcVfile(v));
        return children2;
    }

    public int getAttributeCount() {
        return columncount;
    }
    public String getAttributeName(int index) {
        switch(index){
            case MARK: return "";
            case NAME: return "Name";
            case SIZE: return "Size";
            case DATE: return "Date";
            case CO: return "CO";
            case CC: return "CC";
            case SYM: return "SYM"; case DIF: return "DIF"; case HI: return "HI";
            default: return "Ext";
        }
    }
    public Object getAttributeValue(Vfile vfile, int attribute) {
        if(vfile==null) return null;
        CcVfile cv = (CcVfile) vfile;
        switch(attribute){
            case MARK:
                return super.getAttributeValue(vfile,attribute);
            case NAME: return vfile.getName();
            case SIZE: return vfile.length();
            case DATE: return vfile.lastModified();
            case CO: return cv.isCheckedOut();
            case CC: return cv.isElement();
            case SYM: return cv.isSymbolicLink();
            case DIF: return cv.isDifferent();
            case HI: return cv.isHijacked();
            default: return vfile.getExtension();
        }
    }


    private NameComparator nc=new NameComparator();
    private int compare(long l1, long l2){
        return l1<l2?-1:l1==l2?0:1;
    }
    private int compare(boolean b1, boolean b2){
        if(b1){
            if(b2) return 0;
            else return 1;
        }
        else if(b2) return -1;
        else return 0;
    }
    private int compare(String b1, String b2){
        if(b1==null){
            if(b2!=null) return -1;//null comes first
            else return 1;
        }
        else if(b2==null) return 1;//null comes first
        else return b1.compareTo(b2);
    }
    public int compare(Vfile vfile, Vfile vfile1, int attribute) {
        CcVfile cv = (CcVfile) vfile;
        final int i;
        switch(attribute){
            case MARK:{
                i=compare(fileTreePlugin.smh.isMarked(vfile),fileTreePlugin.smh.isMarked(vfile1));
                break;
            }
            case NAME:
                i= nc.compare(vfile.getName(),vfile1.getName());
                break;
            case SIZE:
                i=compare(vfile.length(),vfile1.length());
                break;
            case DATE:
                i=compare(vfile.lastModified(),vfile1.lastModified());
                break;
            case CO: return compare(cv.isCheckedOut(),cv.isCheckedOut());
            case CC: return compare(cv.isElement(),cv.isElement());
            case SYM: return compare(cv.isSymbolicLink(),cv.isSymbolicLink());
            case DIF: return compare(cv.isDifferent(),cv.isDifferent());
            case HI: return compare(cv.isHijacked(),cv.isHijacked());
            default:
                i=compare(vfile.getExtension(),vfile1.getExtension());
        }
        return i;
    }

    public boolean isComparable(int attribute) {
        return true;
    }

    public SortDirection getDefaultSortDirection(int attribute) {
        switch(attribute){
            case MARK: return SortDirection.descending;
            case NAME: return SortDirection.ascending;
            case SIZE: return SortDirection.descending;
            case DATE: return SortDirection.descending;
            case CO: return SortDirection.descending;
            case CC: return SortDirection.descending;
            case SYM: return SortDirection.descending;
            case DIF: return SortDirection.descending;
            case HI: return SortDirection.descending;
            default: return SortDirection.ascending;
        }
    }

    public void destroy() {
        if(cc!=null) cc.destroy();
    }
}
