package uk.co.hcsoftware.commander.treeplugin.file;

import net.sourceforge.clearcase.simple.ClearcaseException;
import uk.co.hcsoftware.commander.NumberFormatter;
import uk.co.hcsoftware.commander.Url;
import uk.co.hcsoftware.commander.gui.HostFrame;
import uk.co.hcsoftware.commander.tree.*;
import uk.co.hcsoftware.commander.treeplugin.ColorHandler;
import uk.co.hcsoftware.commander.treeplugin.ColumnSizeSaver;
import uk.co.hcsoftware.commander.treeplugin.SelectionMarkHandler;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * todo: eliminate unecessary classes
 * todo: shortcut for cc actions
 * todo: dynasearch to collect the results of findcheckouts
 * Title: ClearcaseTreePlugin
 * Copyright:    Copyright (c) 07-Dec-2005
 *
 * @author spurlinn
 */
public class ClearcaseTreePlugin extends FileTreePlugin{
    public static final Logger log = Logger.getLogger(ClearcaseTreePlugin.class.getName());
    private ColumnSizeSaver css;//= new ColumnSizeSaver(ct,"mark_column_w","cc_name_column_w","size_column_w","date_column_w","ext_column_w") ;;
    private ClearcaseModel ccmodel;

    public ClearcaseTreePlugin() {
        //super(new ClearcaseModel());
        ccmodel = new ClearcaseModel();
        init(ccmodel);
        addPopupCommands();
    }

    private void addPopupCommands() {
        popup.addSeparator();
        popup.add(new AbstractAction("Find checkouts"){
            public void actionPerformed(ActionEvent e) {
                final Object node = ct.getNode();
                try {
                    if (node!=null) {
                        List<ClearcaseModel.CcVfile> viewPrivates = ccmodel.findCheckouts((ClearcaseModel.CcVfile) node);
                        ct.setNodeAsynchronous(node,null,-1,true,viewPrivates);
                    }
                } catch (ClearcaseException e1) {
                    //todo: errmess
                    log.log(Level.WARNING, "exception", e1);
                }
            }
        });
        popup.add(new AbstractAction("Find view-privates"){
            public void actionPerformed(ActionEvent e) {
                final Object node = ct.getNode();
                try {
                    if (node!=null) {
                        List<ClearcaseModel.CcVfile> viewPrivates = ccmodel.findViewPrivates((ClearcaseModel.CcVfile) node);
                        ct.setNodeAsynchronous(node,null,-1,true,viewPrivates);
                    }
                } catch (ClearcaseException e1) {
                    //todo: errmess
                    log.log(Level.WARNING, "exception", e1);
                }
            }
        });
    }

    public void uninstall(HostFrame hf) {
        super.uninstall(hf);
        ccmodel.destroy();
    }

    public Url getUrl(Object node) {
        if(node==null) return null;
        String s = "clearcase:"+((Vfile)node).getAbsolutePath();
        return Url.getUrl(s);
    }

    protected void setFile(final Vfile dir,final Vfile selectFile) {
        ClearcaseModel.CcVfile f1 = ccmodel.getCcVfile(dir);
        ClearcaseModel.CcVfile f2 = selectFile==null?null:ccmodel.getCcVfile(selectFile);
        ct.setNodeAsynchronous(f1, f2, -1, false, null);
    }

    protected void initCommanderTree() {
        smh=new SelectionMarkHandler<Vfile>(ct) ;
        final String[] names = new String[ClearcaseModel.columncount];

        names[ClearcaseModel.MARK]="mark_column_w";
        names[ClearcaseModel.NAME]="cc_name_column_w";
        names[ClearcaseModel.SIZE]="size_column_w";
        names[ClearcaseModel.DATE]="date_column_w";
        names[ClearcaseModel.EXT]="ext_column_w";
        names[ClearcaseModel.CO]="cc_co_column_w";
       // protected static final int CC=6,SYM=7,DIF=8,HI=9;
        names[ClearcaseModel.CC]="cc_cc_column_w";
        names[ClearcaseModel.SYM]="cc_sym_column_w";
        names[ClearcaseModel.DIF]="cc_dif_column_w";
        names[ClearcaseModel.HI]="cc_hi_column_w";

        css = new ColumnSizeSaver(ct,names) ;
        ch=new ColorHandler(ct);
        ct.setColumnSpec(createColumnSpec());
        ct.setDefaultComparison(1,SortDirection.ascending);
        ct.setNameColumnIndex(1);
        ct.setDirColumnIndex(2);
    }

//    protected void initCommanderTree() {
//        smh=new SelectionMarkHandler<Vfile>(ct) ;
//        css = new ColumnSizeSaver(ct,"mark_column_w","cc_name_column_w","size_column_w","date_column_w","ext_column_w") ;;
//        ch=new ColorHandler(ct);
//        ct.setColumnSpec(createColumnSpec());
//        ct.setDefaultComparison(1,SortDirection.ascending);
//        ct.setNameColumnIndex(1);
//        ct.setDirColumnIndex(2);
//    }


    protected DefaultColumnSpec createColumnSpec() {
        DefaultColumnSpec cs = new DefaultColumnSpec(model);

        cs.setWidth(ClearcaseModel.MARK,css.getColWidth(ClearcaseModel.MARK,5));
        cs.setWidth(ClearcaseModel.NAME,css.getColWidth(ClearcaseModel.NAME,200));
        cs.setWidth(ClearcaseModel.SIZE,css.getColWidth(ClearcaseModel.SIZE,50));
        cs.setWidth(ClearcaseModel.DATE,css.getColWidth(ClearcaseModel.DATE,120));
        cs.setWidth(ClearcaseModel.EXT,css.getColWidth(ClearcaseModel.EXT,50));
        cs.setWidth(ClearcaseModel.CC,css.getColWidth(ClearcaseModel.CC,50));
        cs.setWidth(ClearcaseModel.SYM,css.getColWidth(ClearcaseModel.SYM,50));
        cs.setWidth(ClearcaseModel.DIF,css.getColWidth(ClearcaseModel.DIF,50));
        cs.setWidth(ClearcaseModel.HI,css.getColWidth(ClearcaseModel.HI,50));
//        names[ClearcaseModel.CC]="cc_cc_column_w";
//        names[ClearcaseModel.SYM]="cc_sym_column_w";
//        names[ClearcaseModel.DIF]="cc_dif_column_w";
//        names[ClearcaseModel.HI]="cc_hi_column_w";

        cs.setWidth(ClearcaseModel.CO,css.getColWidth(ClearcaseModel.CO,50));

        cs.setRenderer(ClearcaseModel.MARK,new DefaultCommanderCellRenderer() { //mark column
            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct
                        , attributeValue==null?"":((Boolean) attributeValue).booleanValue()?"*":""
                        , node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        });
        cs.setRenderer(ClearcaseModel.NAME,new DefaultCommanderCellRenderer() {  //name column
            public Component getSpecialRendererComponent(CommanderTree ct, SpecialValues value, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel) super.getSpecialRendererComponent(ct, value, node, isSelected, hasFocus);
                jl.setFont(boldFont);
                ch.setColors(jl,isSelected, smh.isMarked((Vfile) node));
                return jl;
            }

            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct, attributeValue, node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        })  ;
        cs.setRenderer(ClearcaseModel.SIZE,new DefaultCommanderCellRenderer() { //size column
            NumberFormatter nf = new NumberFormatter();
             public Component getSpecialRendererComponent(CommanderTree ct, SpecialValues value, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel) super.getSpecialRendererComponent(ct, value, node, isSelected, hasFocus);
                jl.setFont(boldFont);
                 ch.setColors(jl,isSelected, smh.isMarked((Vfile) node));
                return jl;
            }
            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct
                        ,attributeValue==null?"": nf.formatBytes(((Long) attributeValue).longValue())
                        , node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        });
        cs.setRenderer(ClearcaseModel.DATE,new DefaultCommanderCellRenderer() { //date column
            private final DateFormat df = new SimpleDateFormat("dd-MMM-yy HH:mm");
            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct
                        ,attributeValue==null?"": df.format(new Date((Long)attributeValue))
                        , node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        });
         cs.setRenderer(ClearcaseModel.EXT,new DefaultCommanderCellRenderer() { //date column
            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct
                        ,attributeValue
                        , node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        });

        cs.setRenderer(ClearcaseModel.CO,new BoolRenderer());
        cs.setRenderer(ClearcaseModel.CC,new BoolRenderer());
        cs.setRenderer(ClearcaseModel.SYM,new BoolRenderer());
        cs.setRenderer(ClearcaseModel.DIF,new BoolRenderer());
        cs.setRenderer(ClearcaseModel.HI,new BoolRenderer());
       // protected static final int CC=6,SYM=7,DIF=8,HI=9;
        return cs;
    }
    private class BoolRenderer extends Renderer{
        public Object getValue(Object attributeValue) {
                if(attributeValue!=null){
                   Boolean b = (Boolean) attributeValue;
                   return b.booleanValue()?"X":"";
               }
               else return null;
            }
            public void customizeLabel(JLabel jl) {
                jl.setHorizontalAlignment(JLabel.HORIZONTAL);
            }
    }
    private abstract class Renderer extends DefaultCommanderCellRenderer{
        public Component getSpecialRendererComponent(CommanderTree ct, SpecialValues value, Object node, boolean isSelected, boolean hasFocus) {
            JLabel jl =  (JLabel) super.getSpecialRendererComponent(ct, value, node, isSelected, hasFocus);
            jl.setFont(boldFont);
            ch.setColors(jl,isSelected, smh.isMarked((Vfile) node));
            return jl;
        }
        public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
            attributeValue=getValue(attributeValue);
            JLabel jl =  (JLabel)  super.getRendererComponent(ct
                    ,attributeValue
                    , node, isSelected, hasFocus);
            setupLabel(node, jl, isSelected);
            customizeLabel(jl);
            return jl;
        }
        public abstract Object getValue(Object attributeValue);
        public abstract void customizeLabel(JLabel jl) ;
//        {
//            jl.setHorizontalAlignment(JLabel.HORIZONTAL);
//        }
    }

}
