package uk.co.hcsoftware.commander.treeplugin.file;

import uk.co.hcsoftware.commander.CancelledCallback;
import uk.co.hcsoftware.commander.io.FileUtils;

import javax.swing.*;
import java.io.IOException;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class CopyOperation extends CopyMoveOperation{
    private static final Logger log = Logger.getLogger(CopyOperation.class.getName());
    protected List<CopyDescriptor> created,overwritten;

    public CopyOperation(File destination, List<File> files, JComponent jc) {
        this.destination = destination;
        this.files = files;
        this.jc = jc;
    }

    public String toString() {
        return "dest="+destination+" filecount="+files.size();
    }

    public void copy(final CopyOperation.CopyMode copyMode, final CancelledCallback cc){
        //   final TaskTimer tt = new  TaskTimer();
        log.fine("copy="+toString()+" mode="+copyMode);
        created=new ArrayList<CopyDescriptor>();
        overwritten=new ArrayList<CopyDescriptor>();
        failures=new ArrayList<CopyFailure>();
        final List<CopyDescriptor> copySchedule = getSchedule();
        log.fine("schedule="+copySchedule);
        int toCopy=copySchedule.size();
        for(CopyDescriptor cd:copySchedule){
            if(cc.wasCancelled()){
                setStatusText("cancelled..");
                return;
            }
            final File from = cd.getFrom();
            final File to = cd.getTo();
            CopyMode cm = getCopyMode(from,to,copyMode);
            log.finer("cm for from="+from+"="+cm);
            if(cm==CopyMode.skip) continue;
            else if(cm==CopyMode.abort) return;
            //else  //cm==CopyMode.overwrite

            try {
                final boolean isOverwrite=to.exists();
                final File parentFile = to.getParentFile();
                if (parentFile!=null ) {
                    if (!parentFile.exists() && !parentFile.mkdirs()) {
                        log.warning("parentfile="+parentFile+" mkdirs() failed! attempting copy anwyay");
                    }
                }
                else{
                    log.info("parentfile of to="+to+" was null, ignoring mkdirs attempting copy anwyay");
                }
                if(from.isDirectory()){
                    if(to.exists() && !to.isDirectory()){
                       log.warning("to="+to+" exists and is a file! from"+from);
                       failures.add(new CopyFailure(cd,"destination was a file, not a dir"));
                       continue;
                    }
                    else if (!to.exists() && !to.mkdirs()) {
                        log.warning("couldn't mkdirs from"+from);
                        failures.add(new CopyFailure(cd,"create directory failed"));
                        continue;
                    }
                } else{
                   FileUtils.getInstance().copyTo(from,to);// from.copyTo(to);
                }
                if(!isOverwrite) created.add(cd);
                else overwritten.add(cd);
            } catch (IOException e1) {
                log.log(Level.WARNING,"couldn't copy "+cd,e1);
                failures.add(new CopyFailure(cd,e1));
            }
            --toCopy;
            setStatusText(getText(toCopy));
        }
    }

    private String getText(int toCopy) {
        if(toCopy==1) return "Copy 1 file";
        else if(toCopy==0) return "";
        return "Copy "+(toCopy)+" files";
    }

    protected List<CopyDescriptor> buildSchedule() {
        //this recursively visits all files that will be copied.
        //since we are actually using cp(vfile,vfile) and mkdirs() we need to build a complete list of
        //every dir that would be made and every vfile that would be copied, in the order we will do them
        List<CopyDescriptor>  cd=new ArrayList<CopyDescriptor>();
        for (File f:files) {
            addToList(f,destination,cd) ;
        }
        return cd;
    }
    private void addToList(File f, final File destFile, List<CopyDescriptor> l) {
        if(f.isDirectory()){
            File newParent=new File(destFile,f.getName());
            l.add(new CopyDescriptor(f,newParent));
            for(File kid:f.listFiles()){
                addToList(kid,newParent,l);
            }
        }
        else{
            l.add(new CopyDescriptor(f,new File(destFile,f.getName())));
        }
    }


    public boolean wereFailures() {
        return failures.size()>0;
    }

    public List<File> getCreatedFiles() {
        List<File> copied = new ArrayList<File>(created.size());
        for(CopyDescriptor cd:created){
            copied.add(cd.getTo()); //create new vfile in case the new file's attributes have changed (e.g. the lastModified())
        }
        return copied;
    }
    public List<File> getOverwrittenFiles() {
        List<File> copied = new ArrayList<File>(overwritten.size());
        for(CopyDescriptor cd:overwritten){
            copied.add(cd.getTo()); //create new vfile in case the new file's attributes have changed (e.g. the lastModified())
        }
        return copied;
    }

    public List<CopyDescriptor> getCreated() {
        return created;
    }
}
