package uk.co.hcsoftware.commander.treeplugin.file;

import uk.co.hcsoftware.commander.treeplugin.file.Vfile;

import java.util.logging.Logger;


/**
 * collects data by recursing down the tree
 */
public class DirScanner {
    public static final Logger log = Logger.getLogger(DirScanner.class.getName());
    private int filecount;
    private long totalBytes;
    private long maxBytes,minBytes;

    private int maxRecurse=1000;
    private boolean maxRecurseBreached;

    public void scan(Vfile vf){
        //recurse down the tree, harvesting data
        harvestData(vf,0);
    }
    private void harvestData(Vfile vf, int recursions){
        if(recursions>maxRecurse){
            maxRecurseBreached=true;
            return;
        }
        //recurse down the tree, harvesting data
        if (vf.isDirectory()) {
            Vfile[] children = vf.listFiles();
            if (children!=null) {
                for(Vfile kid:children) harvestData(kid,recursions+1);
            }
            else log.warning("children of "+vf+" was null, though isDirectory returned true");
        }
        filecount++;
        final long len = vf.length();
        totalBytes+=len;
        if(len>maxBytes) maxBytes=len;
        if(len<minBytes) minBytes=len;
    }
    public long getAvBytes(){
        return filecount==0?0:totalBytes/filecount;
    }
    public int getFilecount() {
        return filecount;
    }

    public long getTotalBytes() {
        return totalBytes;
    }

    public long getMaxBytes() {
        return maxBytes;
    }

    public long getMinBytes() {
        return minBytes;
    }

    public int getMaxRecurse() {
        return maxRecurse;
    }

    public boolean isMaxRecurseBreached() {
        return maxRecurseBreached;
    }
}
