package uk.co.hcsoftware.commander.treeplugin.file;

import uk.co.hcsoftware.commander.tree.CtreeModel;
import uk.co.hcsoftware.commander.tree.SortDirection;
import uk.co.hcsoftware.commander.gui.WeakListener;
import uk.co.hcsoftware.commander.NameComparator;

import java.util.*;
import java.io.File;


/**
 * Title: FileModel
 * Copyright:    Copyright (c) 07-Dec-2005
 *
 * @author spurlinn
 */
public class FileModel implements CtreeModel<Vfile>, FileEventManager.FileEventListener{
    private List<Vfile> children;
    private Vfile parent;
    protected FileTreePlugin fileTreePlugin;
    protected static final int MARK=0;
    protected static final int NAME=1;
    protected static final int SIZE=2;
    protected static final int DATE=3;
    protected static final int EXT=4;

    public FileModel() {
    }

    public FileModel(final FileTreePlugin fileTreePlugin) {
        setFileTreePlugin(fileTreePlugin);
    }

    public void setFileTreePlugin(final FileTreePlugin fileTreePlugin) {
        this.fileTreePlugin = fileTreePlugin;
        WeakListener wl = new WeakListener(this,FileEventManager.FileEventListener.class){
            public Object onNullObserver() {
                fileTreePlugin.fem.removeListener((FileEventManager.FileEventListener) getProxy());
                return null;
            }
        };
        fileTreePlugin.fem.addListener((FileEventManager.FileEventListener) wl.getProxy());
    }


    public void filesChanged(FilesChangedEvent fe) {
        log.finer("filesChanged fe");
        final Vfile directory = parent;
        if(directory==null){
            log.fine("ignoring event as directory==null");
            return;
        }
        File directoire = directory.getFile();
        List<File> del = fe.getDeleted(directoire);
        List<File> added = fe.getAdded(directoire);
        List<File> changed = fe.getChanged(directoire);
        int changeCount = del.size()+added.size()+changed.size();
        log.fine("dir="+directory+" changeCount="+changeCount);
        List<Vfile> vfd = Vfile.getVfiles(del),vfa= Vfile.getVfiles(added),vfc= Vfile.getVfiles(changed);
        if(changeCount>0){
            if (children!=null) {
                Set<Vfile> curr = new HashSet<Vfile>(children);

                // now apply changes
                curr.removeAll(vfd);
                curr.addAll(vfa);

                //replace the changed ones as the currs might have stale caches
                curr.removeAll(vfc);
                curr.addAll(vfc);
                log.fine("children prevlen="+children.size()+" afterlen="+curr.size());
                children = new ArrayList<Vfile>(curr);
            }
            //forward changes to model listeners
            if(del.size()>0){
                fireNodesChanged(ChangeListener.ChangeType.deleted,vfd);
            }
            if(added.size()>0){
                log.fine("firing nodesadded message addedsize="+added.size());
                fireNodesChanged(ChangeListener.ChangeType.added,vfa);
            }
            if(changed.size()>0){
                fireNodesChanged(ChangeListener.ChangeType.changed,vfc);
            }
            fileTreePlugin.smh.clearMarkedAndSelected();
        }

    }


    public List<Vfile> getChildren(Vfile vfile) {
        parent=vfile;
        if(vfile==null) children= Collections.emptyList();
        else{
            final Vfile[] vfiles = vfile.listFiles();
            this.children = new ArrayList<Vfile>(vfiles.length);
            for(Vfile vf:vfiles) this.children.add(vf);
        }
        return this.children;
    }
    public boolean hasChildren(Vfile vfile) {
        return vfile.isDirectory();
    }
    public Vfile getParent(Vfile vfile) {
        return vfile.getParentFile();
    }
    public int getAttributeCount() {
        return 5;
    }
    public String getAttributeName(int index) {
        switch(index){
            case MARK: return "";
            case NAME: return "Name";
            case SIZE: return "Size";
            case DATE: return "Date";
            default: return "Ext";
        }
    }
    public Object getAttributeValue(Vfile vfile, int attribute) {
        switch(attribute){
            case MARK:
                return fileTreePlugin.smh.isMarked(vfile);
            case NAME: return vfile.getName();
            case SIZE: return vfile.length();
            case DATE: return vfile.lastModified();
            default: return vfile.getExtension();
        }
    }
    public boolean isComparable(int attribute) {
        return true;
    }
    private NameComparator nc=new NameComparator();
    private int compare(long l1, long l2){
        return l1<l2?-1:l1==l2?0:1;
    }
    private int compare(boolean b1, boolean b2){
        if(b1){
            if(b2) return 0;
            else return 1;
        }
        else if(b2) return -1;
        else return 0;
    }
    private int compare(String b1, String b2){
        if(b1==null){
            if(b2!=null) return -1;//null comes first
            else return 1;
        }
        else if(b2==null) return 1;//null comes first
        else return b1.compareTo(b2);
    }
    public int compare(Vfile vfile, Vfile vfile1, int attribute) {
        final int i;
        switch(attribute){
            case MARK:{
                i=compare(fileTreePlugin.smh.isMarked(vfile),fileTreePlugin.smh.isMarked(vfile1));
                break;
            }
            case NAME:
                i= nc.compare(vfile.getName(),vfile1.getName());
                break;
            case SIZE:
                i=compare(vfile.length(),vfile1.length());
                break;
            case DATE:
                i=compare(vfile.lastModified(),vfile1.lastModified());
                break;
            default:
                i=compare(vfile.getExtension(),vfile1.getExtension());
        }
        return i;
    }
   // yName(true,"ByName"),BySize(false,"BySize"),ByDate(false,"ByDate"),ByMarkedness(false,"ByMarkedness");
    public SortDirection getDefaultSortDirection(int attribute) {
        switch(attribute){
            case MARK: return SortDirection.descending;
            case NAME: return SortDirection.ascending;
            case SIZE: return SortDirection.descending;
            case DATE: return SortDirection.descending;
            default: return SortDirection.ascending;
        }
    }

    public void addChangeListener(ChangeListener<Vfile> changeListener) {
         changeListeners.add(changeListener);
    }

    public void removeChangeListener(ChangeListener<Vfile> changeListener) {
         changeListeners.remove(changeListener);
    }

    private List<CtreeModel.ChangeListener<Vfile>> changeListeners=new ArrayList<CtreeModel.ChangeListener<Vfile>>();


    protected void fireNodesChanged(CtreeModel.ChangeListener.ChangeType ct, List<Vfile> nodes){
        for(CtreeModel.ChangeListener<Vfile> cl: changeListeners){
            cl.nodesChanged(ct,nodes);
        }
    }

}
