package uk.co.hcsoftware.commander.treeplugin.file;

import ns.gui.SwingWorker3;
import uk.co.hcsoftware.commander.*;
import uk.co.hcsoftware.commander.gui.ActionHelper;
import uk.co.hcsoftware.commander.gui.HostFrame;
import uk.co.hcsoftware.commander.gui.HtmlDialog;
import uk.co.hcsoftware.commander.io.FileUtils;
import uk.co.hcsoftware.commander.tree.*;
import uk.co.hcsoftware.commander.treeplugin.*;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.*;
import java.net.MalformedURLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * todo: add copyTo(InputStream is) throws notsupported, cannotcopy etc.. to TreePlugin (plugins can optionally implement it)
 * so can copy from anything to anything
 *
 * Title: FileTreePlugin
 * Copyright:    Copyright (c) 22-Nov-2005
 *
 * @author spurlinn
 */
public class FileTreePlugin implements TreePlugin{
    public static final Logger log = Logger.getLogger(FileTreePlugin.class.getName());
    protected CommanderTree ct;

    protected final FileEventManager fem = FileEventManager.getInstance();
    private HostFrame hf;
    private Vfile previousFile=null;
    protected JPopupMenu popup;

    private ColumnSizeSaver css;
    protected ColorHandler ch;
    private static final DataFlavor[] RECEIVE_FLAVORS={TreeStream.DATAFLAVOR, DataFlavor.javaFileListFlavor};
    protected FileModel model;

//    protected FileTreePlugin(FileModel model){
//        init(model);
//    }
    FileTreePlugin() {

    }

    protected void init(FileModel model) {
        model.setFileTreePlugin(this);
        FileTreePlugin.this.model=model;
        ct = new CommanderTree(model);
        initCommanderTree();

        setupActions();
        setupDnD();
        setupPopup();

        smh.addSelectionMarkListener(new SelectionMarkHandler.SelectionMarkListener() {
            public void selectionMarkChanged(int[] selectedFiles) {
                List<Vfile> nodes = new ArrayList<Vfile>(selectedFiles.length);
                for(int i:selectedFiles){
                    nodes.add((Vfile) ct.getNode(i));
                }
                FileTreePlugin.this.model.fireNodesChanged(CtreeModel.ChangeListener.ChangeType.changed, nodes);
            }
        });
    }

    protected void initCommanderTree() {
        smh=new SelectionMarkHandler<Vfile>(ct) ;
        css = new ColumnSizeSaver(ct,"mark_column_w","name_column_w","size_column_w","date_column_w","ext_column_w") ;
        ch=new ColorHandler(ct);
        ct.setColumnSpec(createColumnSpec());
        ct.setDefaultComparison(1,SortDirection.ascending);
        ct.setNameColumnIndex(1);
        ct.setDirColumnIndex(2);
    }


    final static Font plainFont = new Font("lucida sans",Font.PLAIN,12);
    final static Font italicPlainFont = new Font("lucida sans",Font.ITALIC /*| Font.BOLD*/,12);
    final static Font boldFont = new Font("lucida sans",Font.BOLD,12);
    protected DefaultColumnSpec createColumnSpec() {
        DefaultColumnSpec cs = new DefaultColumnSpec(model);

        cs.setWidth(FileModel.MARK,css.getColWidth(FileModel.MARK,5));
        cs.setWidth(FileModel.NAME,css.getColWidth(FileModel.NAME,200));
        cs.setWidth(FileModel.SIZE,css.getColWidth(FileModel.SIZE,50));
        cs.setWidth(FileModel.DATE,css.getColWidth(FileModel.DATE,120));
        cs.setWidth(FileModel.EXT,css.getColWidth(FileModel.EXT,50));

        cs.setRenderer(FileModel.MARK,new DefaultCommanderCellRenderer() { //mark column
            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct
                        , attributeValue==null?"":((Boolean) attributeValue).booleanValue()?"*":""
                        , node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        });
        cs.setRenderer(FileModel.NAME,new DefaultCommanderCellRenderer() {  //name column
            public Component getSpecialRendererComponent(CommanderTree ct, SpecialValues value, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel) super.getSpecialRendererComponent(ct, value, node, isSelected, hasFocus);
                jl.setFont(boldFont);
                ch.setColors(jl,isSelected, smh.isMarked((Vfile) node));
                return jl;
            }

            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct, attributeValue, node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        })  ;
        cs.setRenderer(FileModel.SIZE,new DefaultCommanderCellRenderer() { //size column
            NumberFormatter nf = new NumberFormatter();
             public Component getSpecialRendererComponent(CommanderTree ct, SpecialValues value, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel) super.getSpecialRendererComponent(ct, value, node, isSelected, hasFocus);
                jl.setFont(boldFont);
                 ch.setColors(jl,isSelected, smh.isMarked((Vfile) node));
                return jl;
            }
            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct
                        ,attributeValue==null?"": nf.formatBytes(((Long) attributeValue).longValue())
                        , node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        });
        cs.setRenderer(FileModel.DATE,new DefaultCommanderCellRenderer() { //date column
            private final DateFormat df = new SimpleDateFormat("dd-MMM-yy HH:mm");
            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct
                        ,attributeValue==null?"": df.format(new Date((Long)attributeValue))
                        , node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        });
         cs.setRenderer(FileModel.EXT,new DefaultCommanderCellRenderer() { //date column

            public Component getRendererComponent(CommanderTree ct, Object attributeValue, Object node, boolean isSelected, boolean hasFocus) {
                JLabel jl =  (JLabel)  super.getRendererComponent(ct
                        ,attributeValue
                        , node, isSelected, hasFocus);
                setupLabel(node, jl, isSelected);
                return jl;
            }
        });
        return cs;
    }



    protected void setupLabel(Object node, JLabel jl, boolean isSelected) {
        Vfile file = (Vfile) node;
        boolean marked = false;
        if (file!=null) {
            if(file.isDirectory()){
                jl.setFont(boldFont);
            }
            else{
                jl.setFont(plainFont);
            }
            marked=smh.isMarked(file);
        }
        ch.setColors(jl,isSelected, marked);
    }

    public CommanderTree createTree() {
        return ct;
    }

    public void install(HostFrame o) {
        this.hf=o;
        new StatusUpdater(ct,hf,smh);
    }


    //ACTIONS

    private void setupActions() {
        //copy the selected filenames to the clipboard
        final Action copyNamesAction = ActionHelper.setupAction(new AbstractAction("copynames") {
            public void actionPerformed(ActionEvent e) {
                List<Vfile> l = getSelectedFiles(true, false, false);
                if (l.size()>0) {
                    StringBuffer sb = new StringBuffer();
                    for(Vfile vf:l){
                        if(sb.length()>0) sb.append("\n");
                        sb.append(vf.getName());
                    }
                    try {
                        Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
                        cb.setContents(
                                new StringSelection(sb.toString())
                                , new ClipboardOwner() { public void lostOwnership(Clipboard clipboard, Transferable contents) { }}
                        );
                    } catch (Exception e1) {
                        log.log(Level.WARNING, "couldn't copy "+sb.toString()+" to clipboard", e1);
                    }
                }

            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_C,Main.ctrl_mask),"copy");

        ct.registerAction("copynames",copyNamesAction);
        ct.registerInput(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_C,Main.ctrl_mask),"copynames");
        ct.registerInput(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_COPY,0),"copynames");


        new AutocompleteJumper(ct,new AutocompleteJumper.RowIdentifier() {
            public String getRowId(Object node) {
                return ((Vfile)node).getName();
            }
        });

        ct.registerAction("mark",smh.getMarkAction());
        ct.registerInput(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0),"mark");
        ct.registerAction("clear",smh.getClearAction());
        ct.registerInput(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),"clear");
        ct.registerAction("launch",EnterAction);
        ct.registerInput(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),"launch");

        ct.addMouseListener(new MouseAdapter(){
            private void mouse(MouseEvent e) {
                if(e.isPopupTrigger()){
                    popup.show(ct,e.getX(),e.getY());
                }
                else if (e.getClickCount()==2){
                    EnterAction.actionPerformed(null);
                }
            }
            public void mousePressed(MouseEvent e) { mouse(e); }
            public void mouseReleased(MouseEvent e) { mouse(e); }
        });
    }



    private Action EnterAction = new AbstractAction() {
        private Url getArchiveUrl(String urlType, Vfile node) throws MalformedURLException {
           // Url url = Url.getUrl(urlType+":"+node.toURL().toExternalForm()+"!/");
             Url url = Url.getUrl(node.toURL().toExternalForm());
            return url;
        }
        public void actionPerformed(ActionEvent e) {
            List l = ct.getSelectedNodes(true,true);
            Vfile node ;
            if(l.size()==1 && !(node=(Vfile) l.get(0)).isDirectory()){  //ignore directories as enter causes them to open
                // if it's a jar/zip file then open it inline
                try {
                    if(node.getName().toLowerCase().endsWith(".zip")){
                        Url url = getArchiveUrl("zip",node);
                        hf.openUrl(url);
                    }
                    else if(node.getName().toLowerCase().endsWith(".jar")){
                        Url url = getArchiveUrl("jar",node);
                        hf.openUrl(url);
                    }
                    else launchFiles(Collections.singletonList(node));
                } catch (MalformedURLException e1) {
                    JOptionPane.showMessageDialog(ct,"Couldn't convert filename into URL\n" +
                            e1.getMessage(),"Open archive",JOptionPane.ERROR_MESSAGE);
                } catch (CannotOpenUrlException e1) {
                    log.log(Level.WARNING,"coudln't open archive",e1);
                    JOptionPane.showMessageDialog(ct,"Couldn't open archive\n" +
                            e1.getMessage(),"Open archive",JOptionPane.ERROR_MESSAGE);
                }
                hf.getFocus();
            }
        }


    };


    private void setupPopup() {
        popup = new JPopupMenu();
        popup.add(new AbstractAction("harvest data") {
            public void actionPerformed(ActionEvent e) {
                final List<Vfile> l = getSelectedFiles(true, true, true);
                if (l.size()>0) {
                    final CancelledCallback cc = hf.getCancelledCallback();
                    hf.blockInput("scanning "+l.size()+" dirs",cc);
                    new SwingWorker3(){
                        NumberFormatter nf = new NumberFormatter();
                        public Object construct() {
                            try {
                                StringBuffer sb = new StringBuffer();
                                sb.append("<html><table>");
                                sb.append("<tr>");
                                sb.append("<th>name</th>");
                                sb.append("<th>descendant files</th>");
                                sb.append("<th>total content size</th>");
                                sb.append("<th>av file size</th>");
                                sb.append("</tr>");
                                int i=l.size();
                                for(Vfile vf:l){
                                    if(vf.isDirectory()){
                                        DirScanner ds = new DirScanner();
                                        ds.scan(vf);
                                        sb.append("<tr>");
                                        sb.append("<td>"+vf.getName()+"</td>");
                                        sb.append("<td>"+ds.getFilecount()+"</td>");
                                        sb.append("<td>"+nf.formatBytes(ds.getTotalBytes())+"</td>");
                                        sb.append("<td>"+nf.formatBytes(ds.getAvBytes())+"</td>");
                                        sb.append("</tr>");
                                    }
                                    hf.updateStatus("scanning "+(--i)+" dirs");
                                    if(cc.wasCancelled()){
                                        hf.updateStatus("cancelling...");
                                        return null;
                                    }
                                }
                                sb.append("</html></table>");
                                return sb.toString();
                            } catch (Exception e1) {
                                log.log(Level.WARNING,"e1",e1);
                                return e1;
                            }
                        }

                        public void finished() {
                            hf.blockInput(false,null);
                            if(cc.wasCancelled()) return;
                            if(get()instanceof Exception){
                                JOptionPane.showMessageDialog(ct,
                                        "couldn't gather data:\n"+ ((Exception) get()).getMessage(),"harvest data"
                                        ,JOptionPane.ERROR_MESSAGE);
                                return;
                            }
                            Window f = SwingUtilities.getWindowAncestor(ct);
                            HtmlDialog td = new HtmlDialog((Frame) f, "Directory details", false);
                            td.loadHtml((String) get());
                            td.setSize(500,300);
                            td.setLocationRelativeTo(ct);
                            td.setVisible(true);
                        }
                    }.start();
                }
            }
        })   ;
    }

    private void setupDnD() {
        //todo: work out if the source is this control and ignore (can't drag to self)
        final DropTargetListener dtl = new DropTargetListener(){
            public void dragEnter(DropTargetDragEvent evt) {
                for(DataFlavor df: RECEIVE_FLAVORS){
                    if(evt.isDataFlavorSupported(df)){
                        evt.acceptDrag(DnDConstants.ACTION_COPY);
                        return;
                    }
                }
                evt.rejectDrag();
            }
            public void dragOver(DropTargetDragEvent evt) {
                }
            public void dragExit(DropTargetEvent evt) {;}
            public void dropActionChanged(DropTargetDragEvent evt) {;}
            public void drop(DropTargetDropEvent evt) {
                onDrop(evt);
            }
        };
        ct.setDropTarget(new DropTarget(ct,dtl));
      //  tableScrollPane.setDropTarget(new DropTarget(tableScrollPane,dtl));
        final DragSource ds = new DragSource();
        ct.createDefaultDragGestureRecognizer(DnDConstants.ACTION_COPY,new DragGestureListener() {
            public void dragGestureRecognized(DragGestureEvent dge) {
                onDrag(dge, ds);
            }
        },ds);
    }
    private Transferable getTransferable(){
        final int[] sel = getSelectedOrMarked(true, true,true);
        log.fine(sel.length+" files selected");
        if(sel.length==0){
            log.fine("ignore drag as sel.length==0");
            return null;
        }

        log.finer("start drag sels="+printInt(sel));
        Transferable t = new Transferable() {
            public DataFlavor[] getTransferDataFlavors() {
                return new DataFlavor[]{
                    DataFlavor.javaFileListFlavor
                };
            }
            public boolean isDataFlavorSupported(DataFlavor flavor) {
                return DataFlavor.javaFileListFlavor.equals(flavor);
            }
            List<File> dragFilesList ;
            public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException {
                final boolean dataFlavorSupported = isDataFlavorSupported(flavor);
                log.finer("getTransferData dataFlavorSupported="+dataFlavorSupported);
                if(!dataFlavorSupported) throw new UnsupportedFlavorException(flavor);
                if (dragFilesList==null) {
                    dragFilesList = new ArrayList<File>(sel.length);
                    for (int i = 0; i < sel.length; i++) {
                        final Vfile vfile = (Vfile) ct.getNode(sel[i]);//children.get(sel[i]);//getFile(sel[i]);
                        log.finer("i="+i+" directory="+vfile);
                        dragFilesList.add(vfile.getFile());
                    }
                }
                return dragFilesList;
            }
        };
        return t;
    }
    private void onDrag(DragGestureEvent dge, final DragSource ds) {
        Transferable t=getTransferable();
        if (t!=null) {
            log.finer("starting drag now");
            //NS 20051024: have noticed on Mac OS X 10.2.4/java1.5.0 sometimes drop doesn't succeed e.g. drop onto external window
            //and cursor changes back to normal. After this trags don't get initiated, e.g. call startDrag(..) and nothing happens
            //restarting app cures it..
            ds.startDrag (dge, DragSource.DefaultCopyDrop, t, new DragSourceListener() {
                public void dragEnter(DragSourceDragEvent dsde) {log.finest("dragenter");}
                public void dragOver(DragSourceDragEvent dsde) {log.finest("dragOver");}
                public void dropActionChanged(DragSourceDragEvent dsde) {log.finest("dropActionChanged"); }
                public void dragExit(DragSourceEvent dse) {log.finest("dragExit");}
                public void dragDropEnd(DragSourceDropEvent dsde) {log.finest("dragDropEnd");}
            });
        } else {
            log.fine("nothing to transfer");
        }
    }
    private Vfile getDirectory(){
        return (Vfile) ct.getNode();
    }
    private void onDrop(DropTargetDropEvent evt) {
        log.fine("src="+evt.getSource());
        for(DataFlavor df: RECEIVE_FLAVORS){
            if(evt.isDataFlavorSupported(df)){
                evt.acceptDrop(evt.getDropAction());
                try {
                    save(evt.getTransferable());
                } catch (NotSupportedException e) {
                    log.log(Level.WARNING, "exception", e);
                } catch (CannotSaveException e) {
                    log.log(Level.WARNING, "exception", e);
                }
                return;
            }
        }
//        final boolean dataFlavorSupported = evt.isDataFlavorSupported(DataFlavor.javaFileListFlavor);
//        log.finer("drop dataFlavorSupported="+dataFlavorSupported);
//        if(dataFlavorSupported){
//            evt.acceptDrop(evt.getDropAction());
//            log.fine("accept drop javafilelist");
//            try {
//                Object data=evt.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);
//                final List<File> vfiles = (List<File>)data;
//                log.finer("copy(drop) "+vfiles.size()+" files");
////                final List<Vfile> vfiles = new ArrayList<Vfile>(files.size());
////                for(File f:files) vfiles.add(new Vfile(f));
//                // weed out invalid files (including ancestors)
//                final Vfile currDir = getDirectory();
//                if (currDir!=null) {
//                    for (Iterator<File> iterator = vfiles.iterator(); iterator.hasNext();) {
//                        File f = iterator.next();
//                        if(currDir.getFile().equals(f.getParentFile())){
//                            log.finer("weeding f="+f+" as parent is currdir");
//                            iterator.remove();
//                        }
//                        else if(currDir.getFile().equals(f)){
//                            log.finer("weeding f="+f+" as equals currdir");
//                            iterator.remove();
//                        }
//                    }
//                    if(vfiles.size()<1){
//                        log.finer("ignoring drop as 0 files (after weeding)");
//                        hf.getFocus();
//                        return;
//                    }
//                }
//                //think there might be a swing bug circa 1.4 (seen on Mac OS 10.3)
//                //createBookmark springs a JDialog, which hangs the prog..
//                SwingUtilities.invokeLater(new Runnable() {
//                    public void run() {
//                        int i=JOptionPane.showConfirmDialog(ct,"Copy "+vfiles.size()+"" +
//                                " file(s) to this directory?"
//                                ,"Copy (drop)",JOptionPane.YES_NO_OPTION);
//                        if(i!=JOptionPane.YES_OPTION){
//                            log.finer("user rejected confirm, aborting");
//                            hf.getFocus();
//                            return;
//                        }
//                        final CopyOperation co = new CopyOperation(getDirectory().getFile(),vfiles,ct);
//                        final CopyOperation.CopyMode cm = co.getCopyModeWithPrompt("Copy (drop)");
//                        if(cm==null){
//                            log.fine("abandoning drop-copy as cm==null");
//                            return;
//                        }
//                        final CancelledCallback cc = hf.getCancelledCallback();
//                        hf.blockInput("Copy (drop) "+co.getSchedule().size()+" file(s)",cc);
//                        co.setProgressUpdater(hf);
//                        new SwingWorker3(){
//                            public Object construct() {
//                                try {
//                                    co.copy(cm,cc);
//                                } catch (Exception e) {
//                                    log.log(Level.WARNING,"exception ",e);
//                                }
//                                return null;
//                            }
//                            public void finished() {
//                                hf.blockInput(false, null);
//                                fem.fireEvent(new FilesChangedEvent(null,co.getCreatedFiles(),null));
//                                if(co.wereFailures()){
//                                    List<CopyOperation.CopyFailure> failures = co.getFailures();
//                                    String lastMess=failures.get(failures.size()-1).getReason();
//                                    final String message="couldn't copy "+
//                                            (failures.size()>1
//                                            ?(failures.size()+" files")
//                                            :failures.get(0).getCopyDescriptor().getFrom().getName()+"\n" +
//                                            "["+lastMess+"]");
//                                    JOptionPane.showMessageDialog(ct,message,
//                                            "copy (drop)",
//                                            JOptionPane.WARNING_MESSAGE
//                                    );
//                                }
//                                else{
//                                    //select copied files
//                                    setSelectedFilesList(vfiles);
//                                }
//                                hf.getFocus();
//                            }
//                        }.start();
//                    }
//                });
//            } catch (UnsupportedFlavorException e) {
//                log.log(Level.WARNING,"UnsupportedFlavorException",e);
//            } catch (IOException e) {
//                log.log(Level.WARNING,"IOException",e);
//            }
//            hf.getFocus();
//        }
    }

    private String printInt(int[] sel) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < sel.length; i++) {
            sb.append(String.valueOf(sel[i]));
            if(i+1<sel.length) sb.append(',');
        }
        return sb.toString();
    }

    public static FileTreePlugin createFileTreePlugin() {
        final FileTreePlugin ftp = new FileTreePlugin();
        ftp.init(new FileModel(ftp));
        return ftp;
    }

    public static class CouldntCopyException extends Exception{
        private List<Vfile> failures;
        private List<Exception> reasons;

        public CouldntCopyException(List<Vfile> failures, List<Exception> reasons) {
            this.failures = failures;
            this.reasons = reasons;
        }

        public List<Vfile> getFailures() {
            return failures;
        }

        public List<Exception> getReasons() {
            return reasons;
        }

        public String getLastReason() {
            return reasons==null||reasons.size()<1?"":reasons.get(0).getMessage();
        }
    }
    public CopyOperation.CopyMode promptForCopyMode(int overwrites, Vfile destDir,String title){
        final CopyOperation.CopyMode[] options = CopyOperation.CopyMode.values();
        int i=JOptionPane.showOptionDialog(ct,//overwrites+
                //"vfile(s) already exist in the destination directory:" +
                //"\n" +destDir.getName()+"\n"
                "What do you want to do when a file already exists?"
                ,title
                ,JOptionPane.OK_CANCEL_OPTION
                ,JOptionPane.QUESTION_MESSAGE
                ,null,options,CopyOperation.CopyMode.defaultCopyMode());
        if(i>-1){
            return options[i];
        }
        else return null;
    }

    /**
     * WARNING returned vfile may not be a valid fileref
     * @return
     */
     private Vfile getTargetDir() {
        final TreePlugin tp = hf.getTargetTreePlugin();
        if(tp==null) return null;
        else if(tp instanceof FileTreePlugin){
            FileTreePlugin ftp = (FileTreePlugin) tp;
            return ftp.getDirectory();
        }
//        Url url=tp.getUrl(tp.get);
//        if(url!=null){
//            try {
//                return url.getVfile();//new Vfile(url);
//            } catch (CannotConvertUrlToFileException e) {
//                log.info("couldn't get vfile from opposite URL="+url+" returning this dir");
//            }
//        }
        return getDirectory();
    }


    private void performMove(final MoveOperation mo, final CopyOperation.CopyMode cm) {
        hf.blockInput(true, "Move "+mo.getSchedule().size()+" file(s)");
        mo.setProgressUpdater(hf);
        //use swingworker as multiple invocations of Spin.off can cause deadlocks
        new SwingWorker3(){
            public Object construct() {
                mo.move(cm);//mo.move(copySchedule, copyMode);
                return null;  //To change body of implemented methods use File | Settings | File Templates.
            }
            public void finished() {
                hf.blockInput(false, null);
                fem.fireEvent(mo.getDeletedFiles(),mo.getCreatedFiles(),mo.getOverwrittenFiles());

                if(mo.wereFailures()){
                    List<CopyOperation.CopyFailure> failures = mo.getFailures();
                    String lastMess=failures.get(failures.size()-1).getReason();
                    final String message="couldn't move "+
                            (failures.size()>1
                            ?(failures.size()+" files")
                            :failures.get(0).getCopyDescriptor().getFrom().getName()+"\n" +
                            "["+lastMess+"]");
                    JOptionPane.showMessageDialog(ct,message,
                            "Move",
                            JOptionPane.WARNING_MESSAGE
                    );
                    hf.getFocus();
                    List<File> l= new ArrayList<File>(failures.size());
                    for(CopyOperation.CopyFailure f:failures){
                        l.add(f.getCopyDescriptor().getFrom());
                    }
                    setSelectedFilesList(l);
                }
                else hf.getFocus();
            }
        }.start();
    }


    protected void setFile(final Vfile dir,final Vfile selectFile) {
        ct.setNodeAsynchronous(dir, selectFile, -1, false, null);
    }


    public void showPreviousFile(){
        if(previousFile!=null) openFile(previousFile, null);
    }

    protected SelectionMarkHandler<Vfile> smh;

    public int[] getSelectedOrMarked(boolean includeSelected, boolean includeMarkedFiles, boolean includeParents){
        return smh.getSelectedOrMarked(includeSelected,includeMarkedFiles,includeParents);
    }
    public List<Vfile> getSelectedFiles(boolean includeSelected, boolean includeMarkedFiles, boolean includeParents){
        int[] rows = getSelectedOrMarked(includeSelected, includeMarkedFiles,includeParents);
        List<Vfile> l = new ArrayList<Vfile>(rows.length);
        for(int ro:rows){
            l.add((Vfile) ct.getNode(ro));
        }
        return l;
    }

    private String getString(List<Vfile> l) {
        return l.size()!=1?l.size()+" files":l.get(0).getName();
    }

    private String getText(int toCopy) {
        if(toCopy==1) return "Delete 1 file";
        else if(toCopy==0) return "Delete";
        return "Delete "+(toCopy)+" files";
    }
    private void setSelectedFilesList(List<File> d) {
        List<Vfile> l = new ArrayList<Vfile>(d.size());
        for(File f:d) l.add(new Vfile(f));
        log.finer("select vfile="+l);
        ct.selectNodes(l, 0);
    }
    private void setSelectedFiles(List<Vfile> d) {
        log.finer("select vfile="+d);
        ct.selectNodes(d, 0);
    }
    private void selectFile(Vfile d) {
        ct.selectNode(d);
    }

    public void copy() {
        final List<Vfile> l=getSelectedFiles(true, true, false);
        if(l.size()>0){
            TreePlugin other = hf.getTargetTreePlugin();
            if(other instanceof FileTreePlugin){
                final Vfile targetDir=((FileTreePlugin)other).getDirectory();
                String sugTarg=targetDir==null?"":targetDir.getAbsolutePath();

                String s = l.size()!=1?l.size()+" files":l.get(0).getName();//getString(l);
                String dest=(String)JOptionPane.showInputDialog(ct,
                        "Copy "+s+" to:"
                        ,"Copy",JOptionPane.QUESTION_MESSAGE,null
                        ,null,sugTarg);
                if(dest!=null){
                    final Vfile d=new Vfile(dest);
                    if(!d.exists()){
                        JOptionPane.showMessageDialog(ct
                                ,"destination directory "+d+" doesn't exist!"
                                ,"Copy"
                                ,JOptionPane.ERROR_MESSAGE);
                    }
                    else if(!d.isDirectory()){
                        JOptionPane.showMessageDialog(ct
                                ,"destination "+d+" isn't a directory!\n" +
                                "please type a directory name"
                                ,"Copy"
                                ,JOptionPane.ERROR_MESSAGE);
                    }
                    else{
                        List<File> f = Vfile.getFiles(l);
                        final CopyOperation co = new CopyOperation(d.getFile(),f,ct);
                        final CopyOperation.CopyMode cm = co.getCopyModeWithPrompt("Copy");
                        if(cm==null){
                            log.fine("abandoning copy as cm==null");
                            return;
                        }
                        final CancelledCallback cc = hf.getCancelledCallback();
                        hf.blockInput("Copy "+co.getSchedule().size()+" file(s)",cc);
                        co.setProgressUpdater(hf);
                        new SwingWorker3(){
                            public Object construct() {
                                try {
                                    co.copy(cm, cc);
                                } catch (Exception e) {
                                    log.log(Level.WARNING,"exception ",e);
                                }
                                return null;  //To change body of implemented methods use File | Settings | File Templates.
                            }
                            public void finished() {
                                hf.blockInput(false, null);
                                fem.fireEvent(null,co.getCreatedFiles(),co.getOverwrittenFiles());
                                if(co.wereFailures()){
                                    List<CopyOperation.CopyFailure> failures = co.getFailures();
                                    String lastMess=failures.get(failures.size()-1).getReason();
                                    final String message="couldn't copy "+
                                            (failures.size()>1
                                            ?(failures.size()+" files")
                                            :failures.get(0).getCopyDescriptor().getFrom().getName()+"\n" +
                                            "["+lastMess+"]");
                                    JOptionPane.showMessageDialog(ct,message,
                                            "Copy",
                                            JOptionPane.WARNING_MESSAGE
                                    );
                                    hf.getFocus();
                                    List<Vfile> l= new ArrayList<Vfile>(failures.size());
                                    for(CopyOperation.CopyFailure f:failures){
                                        l.add(new Vfile(f.getCopyDescriptor().getFrom()));
                                    }
                                    setSelectedFiles(l);
                                }
                                else{
                                    log.finer("no failures, destinationDir.queals(directory)="+d.equals(getDirectory()));
                                    //select copied files
                                    hf.getFocus();
                                    setSelectedFiles(l);
                                }

                            }
                        }.start();
                    }
                }
            }
        } else{
            try {
                Transferable t = getTransferable();
                if(t==null){
                    log.fine("ignoring copy as no transferable files");
                    return;
                }
                TreePlugin tp = hf.getTargetTreePlugin();
                tp.save(t);
            } catch (NotSupportedException e) {
                JOptionPane.showMessageDialog(ct,"The opposite pane doesn't understand my data!","Copy",JOptionPane.ERROR_MESSAGE);
            } catch (CannotSaveException e) {
                log.log(Level.WARNING, "exception", e);
                JOptionPane.showMessageDialog(ct,"Couldn't save those data\n"+e.getMessage(),"Copy",JOptionPane.ERROR_MESSAGE);
            }
//            JOptionPane.showMessageDialog(ct
//                    ,"Can only copy to directories"
//                    ,"Copy"
//                    ,JOptionPane.ERROR_MESSAGE);
        }
    }
    public void delete() {
        final int[] selected=getSelectedOrMarked(true, true, false);
        if(selected.length>0){
            final List<Vfile> failures=new ArrayList<Vfile>();
            final List<File> successes = new ArrayList<File>();
            String s= selected.length!=1?selected.length+" files": ((Vfile) ct.getNode(selected[0])).getName();
            if(JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog(ct,"Delete "+s+" ?"
                    ,"delete",JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE)){
                final CancelledCallback cc = hf.getCancelledCallback();
                hf.blockInput(getText(selected.length),cc);
                new SwingWorker3(){
                    public Object construct() {
                        for (int i = selected.length - 1; i >= 0; i--) { //go backwards so removing doesn't affect list
                            if(cc.wasCancelled()){
                                hf.updateStatus("cancelled..");
                                return null;
                            }
                            hf.updateStatus(getText(i));
                            int index = selected[i];
                            Vfile f = (Vfile) ct.getNode(index);//children.get(index);
                            File ff = f.getFile();
                            try {
                                FileUtils.getInstance().deleteRecursive(ff);
                                //f.deleteRecursive();
                                successes.add(ff);
                            } catch (DeleteFailedException e) {
                                log.warning("failed to delete f="+f+" fail="+e.getFailed());
                                failures.add(f);
                            }
                        }
                        return null;
                    }
                    public void finished() {
                        hf.blockInput(false,null);
                        log.finer("successfully delted="+successes);
                        fem.fireEvent(successes,null,null);
                        if(failures.size()>0){
                            final String message="couldn't delete "+
                                    (failures.size()>1
                                    ?(failures.size()+" files")
                                    :failures.get(0).getName());
                            JOptionPane.showMessageDialog(ct,message,
                                    "Delete",
                                    JOptionPane.WARNING_MESSAGE
                            );
                        }
                        hf.getFocus();
                        if(failures.size()>0){ //select failures
                            setSelectedFiles(failures);//selectFiles(failures);
                        }
                    }
                }.start();
            }
        }
        else{
            JOptionPane.showMessageDialog(ct,"zero files selected\nplease select/mark some files before pressing delete"
                    ,"Delete",JOptionPane.WARNING_MESSAGE);
        }
        hf.getFocus();
    }
    public void duplicate() {
        List<Vfile> l=getSelectedFiles(true, true, false);
        if(l.size()>0){
            if(l.size()==1){
                final Vfile sourceFile = l.get(0);
                String s = sourceFile.getName();
                String dest=(String)JOptionPane.showInputDialog(ct,
                        "Duplicate "+s+" to:"
                        ,"Duplicate",JOptionPane.QUESTION_MESSAGE,null
                        ,null,s+".1");
                if(dest!=null){
                    Vfile d=new Vfile(getDirectory(),dest);
                    if(!d.exists()){
                        try {
                            sourceFile.copyTo(d);
                            //fileOperations.copyFile(sourceFile,d);
                            fem.fireEvent(null,Collections.singletonList(d.getFile()),null);
                            selectFile(d);
                        } catch (IOException e1) {
                            log.log(Level.WARNING,"couldn't duplicate",e1);
                            JOptionPane.showMessageDialog(ct
                                    ,"couldn't duplicate:\n" +
                                    e1.getMessage()
                                    ,"Duplicate"
                                    ,JOptionPane.ERROR_MESSAGE);
                        }
                    }
                    else if(d.isDirectory()){
                        JOptionPane.showMessageDialog(ct
                                ,"destination "+d+" is a directory!\n" +
                                "please type a file name"
                                ,"Duplicate"
                                ,JOptionPane.ERROR_MESSAGE);
                    }
                    else{
                        JOptionPane.showMessageDialog(ct
                                ,"couldn't duplicate:\n" +
                                d+" already exists!"
                                ,"Duplicate"
                                ,JOptionPane.ERROR_MESSAGE);
                    }
                }
            } else{
                JOptionPane.showMessageDialog(ct
                        ,"you can only duplicate 1 file at a time\n" +
                        "please select a single file"
                        ,"Duplicate"
                        ,JOptionPane.ERROR_MESSAGE);
            }
            hf.getFocus();
        }
    }
    public void launch() {
        List<Vfile> l=getSelectedFiles(true, true, true);
        if(l.size()>0){
            launchFiles(l);
        }
        hf.getFocus();
    }
    FileLauncher fl = new FileLauncher();
    private void launchFiles(List<Vfile> l) {
        String s = l.size()==1?l.get(0).getName():l.size()+" files";
        String[] options = new String[]{"launch","cancel"};
        if ( 0== JOptionPane.showOptionDialog(ct
                ,"launch "+s+"?","Launch"
                ,JOptionPane.YES_NO_CANCEL_OPTION
                ,JOptionPane.QUESTION_MESSAGE,null
                ,options,options[0])) {
            try {
                List<File> lf = new ArrayList<File>(l.size());
                for(Vfile vf:l) lf.add(vf.getFile());
                fl.launchFiles(lf);
            } catch (IOException e) {
                log.log(Level.WARNING,"ioe",e);
                JOptionPane.showMessageDialog(ct,"Error executing external process\n" +
                        "["+e.getMessage()+"]","Launch",JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    public void mkdir() {
        // Vfile parent=dragFilesList;
        String dest=(String)JOptionPane.showInputDialog(ct,
                "Create Directory:"
                ,"Create Directory",JOptionPane.QUESTION_MESSAGE,null
                ,null,"");
        if(dest!=null && dest.trim().length()>0){
            Vfile d=new Vfile(getDirectory(),dest);
            if(!d.exists() ){
                if (d.mkdir()) {
                    fem.fireEvent(new FilesChangedEvent(null,Collections.singletonList(d.getFile()),null));
                    //select the dir we just created
                    selectFile(d);
                } else {
                    JOptionPane.showMessageDialog(ct
                            ,"couldn't create directory"
                            ,"Create Directory",JOptionPane.ERROR_MESSAGE);
                }
            } else{
                JOptionPane.showMessageDialog(ct
                        ,d+" already exists"
                        ,"Create Directory"
                        ,JOptionPane.WARNING_MESSAGE);
                //select the existing dir
                selectFile(d);
            }
            hf.getFocus();
        }
    }
    public void move() {
        List<Vfile> l=getSelectedFiles(true, true, false);
        if(l.size()>0){
            Vfile destDir = getTargetDir();
            String s = getString(l);
            String dest=(String)JOptionPane.showInputDialog(ct,
                    "move "+s+" to:"
                    ,"Move",JOptionPane.QUESTION_MESSAGE,null
                    ,null,destDir.getAbsolutePath());
            if(dest!=null){
                Vfile d=new Vfile(dest);
                if(d.isDirectory()){
                    if(d.exists()){
                        final MoveOperation co = new MoveOperation(d.getFile(),Vfile.getFiles(l),ct);
                        final CopyOperation.CopyMode cm = co.getCopyModeWithPrompt("Move");
                        if(cm==null){
                            log.fine("abandoning move as cm==null");
                            return;
                        }
                        performMove(co, cm);
                    }
                    else{
                        //target dir deosn't exist
                        JOptionPane.showMessageDialog(ct
                                ,"destination directory "+d+" doesn't exist!"
                                ,"Move"
                                ,JOptionPane.ERROR_MESSAGE);
                    }
                } else{//input is a vfile -that's OK if only 1 sel, otherwise not
                    if(l.size()==1){
                        //check the vfile doesn't exist
                        if(!d.exists()){
                            final Vfile vfile = l.get(0);
                            if (vfile.renameTo(d)) {
                                fem.fireEvent(new FilesChangedEvent(Collections.singletonList(vfile.getFile()),Collections.singletonList(d.getFile()),null));
                                hf.getFocus();
                                if (d.getParentFile().equals(getDirectory())) {
                                    selectFile(d);
                                }
                                return;
                            }
                            else{
                                JOptionPane.showMessageDialog(ct
                                        ,"Couldn't move "+vfile
                                        ,"Move"
                                        ,JOptionPane.ERROR_MESSAGE);
                            }
                        } else  JOptionPane.showMessageDialog(ct
                                ,"destination file "+d+" already exists!"
                                ,"Move"
                                ,JOptionPane.ERROR_MESSAGE);
                    } else  JOptionPane.showMessageDialog(ct
                            ,"cannot rename more than 1 file to a new file name\n" +
                            "either type a new directory name, or make sure only 1 file is selected"
                            ,"Move"
                            ,JOptionPane.ERROR_MESSAGE);
                }

            }
            hf.getFocus();
        }
    }
    public void rename() {
        List<Vfile> l=getSelectedFiles(true, false, false);
        if(l.size()==1){
            Vfile sourceFile=l.get(0);
            String dest=(String)JOptionPane.showInputDialog(ct,
                    "rename "+sourceFile.getName()+" to:"
                    ,"Rename",JOptionPane.QUESTION_MESSAGE,null
                    ,null,sourceFile.getName());
            if(dest!=null){
                Vfile destinationFile=new Vfile(sourceFile.getParentFile(),dest);

                //check the vfile doesn't exist
                if(destinationFile.exists()){
                    JOptionPane.showMessageDialog(ct
                            ,"destination file "+destinationFile+" already exists!"
                            ,"Rename"
                            ,JOptionPane.ERROR_MESSAGE);
                }
                else if(sourceFile.equals(destinationFile)){
                    JOptionPane.showMessageDialog(ct
                            ,"Couldn't rename "+sourceFile+"\n Sourcefile and destination file are the same!"
                            ,"Rename"
                            ,JOptionPane.ERROR_MESSAGE);
                }
                else{
                    if(sourceFile.renameTo(destinationFile)) {
                        fem.fireEvent(
                                Collections.singletonList(sourceFile.getFile())
                                ,Collections.singletonList(destinationFile.getFile())
                                //bugfix, where previously we called destinationFile.isDirectory.
                                //if desitionFile doesn't exist java.io.File returns false. This got
                                //cached in vfile, so when we sent out the message listeners
                                //treated teh added file as a File where in fact it was now a dir
                                //we don't actually call isDirectory now, but might as well invalidate the cache anyway
                                ,null
                        );
                        selectFile(destinationFile);
                    } else{
                        JOptionPane.showMessageDialog(ct
                                ,"Couldn't rename "+sourceFile
                                ,"Rename"
                                ,JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        } else  JOptionPane.showMessageDialog(ct
                ,"Cannot rename: no file selected"
                ,"Rename"
                ,JOptionPane.ERROR_MESSAGE);
        hf.getFocus();
    }



//    public void filesChanged(FilesChangedEvent fe) {
//        log.finer("filesChanged fe");
//        final Vfile directory = getDirectory();
//        if(directory==null){
//            log.fine("ignoring event as directory==null");
//            return;
//        }
//        List<Vfile> del = fe.getDeleted(directory);
//        List<Vfile> added = fe.getAdded(directory);
//        List<Vfile> changed = fe.getChanged(directory);
//        int changeCount = del.size()+added.size()+changed.size();
//        log.fine("dir="+directory+" changeCount="+changeCount);
//        if(changeCount>0){
//            //forward changes to model listeners
//            if(del.size()>0){
//                fireNodesChanged(CtreeModel.ChangeListener.ChangeType.deleted,del);
//            }
//            if(added.size()>0){
//                fireNodesChanged(CtreeModel.ChangeListener.ChangeType.added,added);
//            }
//            if(changed.size()>0){
//                fireNodesChanged(CtreeModel.ChangeListener.ChangeType.changed,changed);
//            }
//            smh.clearMarkedAndSelected();
//        }
//    }


    private void openFile(Vfile f, Vfile select) {
        if(f==null){
            log.warning("openfile f=null, ignoring");
            return;
        }
        if(f.exists() && f.isDirectory()){
            setFile(f,select);
        }
        else if(!f.exists()){
            log.log(Level.FINE,"can't open f",new Throwable());
            Object[] options = {"open first extant ancestor","cancel"};
            int i=JOptionPane.showOptionDialog(ct,
                    "can't open "+f+" as it doesn't exist"
                    ,"Open",
                    JOptionPane.OK_CANCEL_OPTION
                    ,JOptionPane.WARNING_MESSAGE
                    ,null,options,options[0]);


            if(i==0){
                log.fine("opening first extant ancestor");
                while(true){
                    f=f.getParentFile();
                    if(f==null) return;
                    else if(f.exists() && f.isDirectory()){
                        openFile(f, null);
                        return;
                    }
                }
            }
        }
        else{
            log.fine("ignoring open on file="+f);
            JOptionPane.showMessageDialog(ct,"Cannot open \n"+f+" as it is a file, not a directory","Open",JOptionPane.ERROR_MESSAGE);

        }
    }

    public void openUrl(Url s) throws CannotOpenUrlException {
        try {
            File f = s.getFile();
            if(f==null) throw new CannotOpenUrlException(s, "cannot convert into file");
            openFile(new Vfile(f),null);
        } catch (CannotConvertUrlToFileException e) {
            log.log(Level.INFO,"can't open url",e);
            throw new CannotOpenUrlException(s, "unrecognised file: "+s.getFriendlySpec());
        }
    }

    public Url getUrl(Object node) {
        if(node==null) return null;
        return Url.getUrl((Vfile) node);
    }

    public void save(Transferable t) throws NotSupportedException, CannotSaveException {
        if(t.isDataFlavorSupported(TreeStream.DATAFLAVOR)){
            log.fine("TreeStream.DATAFLAVOR supported");
            try {
                TreeStream ts = (TreeStream) t.getTransferData(TreeStream.DATAFLAVOR);
                List<TreeStream.Node> roots = ts.getRoots();

                Vfile currDir=getDirectory();
                if(currDir==null){
                   throw new CannotSaveException("No current directory");
                }
                List<File> created=new ArrayList<File>(),overwritten = new ArrayList<File>();
                for(TreeStream.Node n:roots) saveNode(currDir.getFile(),n,created,overwritten);
                ts.finishedReading();
                final List<File> empty = Collections.emptyList();
                fem.fireEvent(empty,created,overwritten);
            } catch (UnsupportedFlavorException e) {
                log.log(Level.WARNING, "exception", e);
            } catch (IOException e) {
                log.log(Level.WARNING, "exception", e);
                throw new CannotSaveException(e.getMessage(),e);
            }
        }
        else if(t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)){
            log.fine("javaFileListFlavor supported");
            try {
                Object data=t.getTransferData(DataFlavor.javaFileListFlavor);
                final List<File> vfiles = (List<File>)data;
                log.finer("save "+vfiles.size()+" files");
                // weed out invalid files (including ancestors)
                final Vfile currDir = getDirectory();
                if (currDir!=null) {
                    for (Iterator<File> iterator = vfiles.iterator(); iterator.hasNext();) {
                        File f = iterator.next();
                        if(currDir.getFile().equals(f.getParentFile())){
                            log.finer("weeding f="+f+" as parent is currdir");
                            iterator.remove();
                        }
                        else if(currDir.getFile().equals(f)){
                            log.finer("weeding f="+f+" as equals currdir");
                            iterator.remove();
                        }
                    }
                    if(vfiles.size()<1){
                        log.finer("ignoring drop as 0 files (after weeding)");
                        hf.getFocus();
                        return;
                    }
                }
                final CopyOperation co = new CopyOperation(getDirectory().getFile(),vfiles,ct);
                final CopyOperation.CopyMode cm = co.getCopyModeWithPrompt("Copy (drop)");
                if(cm==null){
                    log.fine("abandoning drop-copy as cm==null");
                    return;
                }
                final CancelledCallback cc = hf.getCancelledCallback();
                hf.blockInput("Copy (drop) "+co.getSchedule().size()+" file(s)",cc);
                co.setProgressUpdater(hf);
                new SwingWorker3(){
                    public Object construct() {
                        try {
                            co.copy(cm,cc);
                        } catch (Exception e) {
                            log.log(Level.WARNING,"exception ",e);
                        }
                        return null;
                    }
                    public void finished() {
                        hf.blockInput(false, null);
                        fem.fireEvent(new FilesChangedEvent(null,co.getCreatedFiles(),null));
                        if(co.wereFailures()){
                            List<CopyOperation.CopyFailure> failures = co.getFailures();
                            String lastMess=failures.get(failures.size()-1).getReason();
                            final String message="couldn't copy "+
                                    (failures.size()>1
                                    ?(failures.size()+" files")
                                    :failures.get(0).getCopyDescriptor().getFrom().getName()+"\n" +
                                    "["+lastMess+"]");
                            JOptionPane.showMessageDialog(ct,message,
                                    "copy (drop)",
                                    JOptionPane.WARNING_MESSAGE
                            );
                        }
                        else{
                            //select copied files
                            setSelectedFilesList(vfiles);
                        }
                        hf.getFocus();
                    }
                }.start();

            } catch (UnsupportedFlavorException e) {
                log.log(Level.WARNING,"UnsupportedFlavorException",e);
            } catch (IOException e) {
                log.log(Level.WARNING,"IOException",e);
            }
        }
        else throw new NotSupportedException();
    }

    public void uninstall(HostFrame hf) {


    }

    private void saveNode(File currDir, TreeStream.Node n, List<File> created, List<File> overwritten) throws IOException {
        InputStream is=n.getInputStream();
        if(is!=null){
            File f = new File(currDir,n.getName());
            boolean existed=f.exists();
            OutputStream os = new BufferedOutputStream(new FileOutputStream(f));
            byte[] buf = new byte[256];
            int i;
            while((i=is.read(buf))!=-1) os.write(buf,0,i);
            os.close();
            is.close();
            if(existed) overwritten.add(f);
            else created.add(f);
        } else{//treat it like a directory
            File parent = new File(currDir,n.getName());
            if(!parent.exists()){
                if(!parent.mkdirs()) throw new IOException("couldn't create directory="+parent);
                else created.add(parent);
            }
            else if(!parent.isDirectory()) throw new IOException("file="+parent+" exists, but is not a directory");
            List<TreeStream.Node> kids=n.getChildren();
            for(TreeStream.Node kid:kids){
                 saveNode(parent,kid,created,overwritten);
            }
        }

    }
}
