package uk.co.hcsoftware.commander.treeplugin.file;

import uk.co.hcsoftware.commander.io.FileUtils;

import javax.swing.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

public class MoveOperation extends CopyMoveOperation {
    private static final Logger log = Logger.getLogger(MoveOperation.class.getName());

    private List<File> overwritten,created,deleted;

    public MoveOperation(File destination, List<File> files, JComponent jc) {
        this.destination = destination;
        this.files = files;
        this.jc = jc;
    }

    public String toString() {
        return "dest="+destination+" filecount="+files.size();
    }


    public void move(final CopyMode copyMode){
      //  TaskTimer tt = new  TaskTimer();
        log.fine("copy="+toString()+" mode="+copyMode);
        overwritten=new ArrayList<File>();
        created=new ArrayList<File>();
        deleted=new ArrayList<File>();
        failures=new ArrayList<CopyFailure>();
        overwritten = new ArrayList<File>();
        final List<CopyDescriptor> copySchedule = getSchedule();
        log.fine("schedule="+copySchedule);
        int toCopy = copySchedule.size();
        for(CopyDescriptor cd:copySchedule){
            final File from = cd.getFrom();
            final File to = cd.getTo();
            CopyMode cm = getCopyMode(from,to,copyMode);
            log.finer("cm for from="+from+"="+cm);
            if(cm==CopyMode.skip) continue;
            else if(cm==CopyMode.abort) return;
            //else  //cm==CopyMode.overwrite

            final File parentFile = to.getParentFile();
            if (parentFile!=null) {
                if (!parentFile.exists() && !parentFile.mkdirs()) {
                    log.warning("parentfile="+parentFile+" mkdirs() failed! attempting copy anwyay");
                }
            }
            else{
                log.info("parentfile of to="+to+" was null, ignoring mkdirs attempting copy anwyay");
            }
            if(to.exists()){ //files exists so have to simulate a move with a delete-rename
                log.fine("deleting vfile="+to+" as is in the way");
                try {
                    FileUtils.getInstance().deleteRecursive(to);
                    if(!from.renameTo(to)){ //we deleted the target but couldn't move the source
                        failures.add(new CopyFailure(cd,"move failed"));
                        deleted.add(to);
                    }
                    else{ //we succeeded in renaming
                        overwritten.add(to);
                        deleted.add(from);
                    }
                } catch (DeleteFailedException e) {
                    final File failed = e.getFailed();
                    log.warning("failed to delete="+to+" [couldn't delete "+failed+"]");
                    failures.add(new CopyFailure(cd,"delete existing vfile/dir="+e.getFailed()+" failed"));
                }
            } else{ //to doesn't exist
                if(!from.renameTo(to)){
                    log.warning("from="+from+" renameTo="+to+" failed, adding failure");
                    failures.add(new CopyFailure(cd,"move failed"));
                }
                else{
                    deleted.add(from);
                    created.add(to);
                }
            }

            --toCopy;
            setStatusText(getText(toCopy));
        }

    }
    private String getText(int toCopy) {
        if(toCopy==1) return "Move 1 file";
        else if(toCopy==0) return "";
        return "Move "+(toCopy)+" files";
    }

    protected List<CopyDescriptor> buildSchedule() {
         //just need a list of all the files/dirs that will be renamed.
        //we are using vfile.renameTo so no need to recurse into dirs: jsut rename the dir
        List<CopyDescriptor>  cd=new ArrayList<CopyDescriptor>();
        for (File f:files) {
            cd.add(new CopyDescriptor(f,new File(destination,f.getName())));
        }
        return cd;
    }

    public boolean wereFailures() {
        return failures.size()>0;
    }

    public List<File> getOverwrittenFiles(){
        return overwritten;
    }
    public List<File> getCreatedFiles(){
        return created;
    }
    public List<File> getDeletedFiles(){
        return deleted;
    }
}
