package uk.co.hcsoftware.commander.treeplugin.file;

import uk.co.hcsoftware.commander.Filter;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * virtual file, for now just wraps java.io.File and adds caching + more sane methods
 *
 * may be the basis of a VFS..  though probably need to change teh methods somewhat
 *
 *
 *  browsing around some NFS shares, the lion's share of time is spent in isDirectory (surprisingly)
 * but also in length() lastModified()
 * So we cache these properties
 *
 */
public class Vfile {
    public static final Logger log = Logger.getLogger(Vfile.class.getName());
    private final File file;

  //  private static long getlength,getname,getlastmodified,isdir;

    public Vfile(String pathname) {
        file = new File(pathname);
    }
    public Vfile(URI uri) {
        file = new File(uri);
    }
    public Vfile(URL url) {
        file = new File(url.getFile());
    }

    public Vfile(Vfile f){
       this(f.file);
    }
    public static List<Vfile> getVfiles(List<File> l){
        List<uk.co.hcsoftware.commander.treeplugin.file.Vfile> ll = new ArrayList<uk.co.hcsoftware.commander.treeplugin.file.Vfile>(l.size());
        for(File f:l) ll.add(new uk.co.hcsoftware.commander.treeplugin.file.Vfile(f));
        return ll;
    }


    public Vfile(File f){
        this.file=f;
        if(file==null) log.log(Level.WARNING,"f=null",new Throwable());
    }
    public Vfile(Vfile parent, String child) {
        file = new File(parent.file, child);
    }


    public Vfile getParentFile() {
        final File parentFile = file.getParentFile();
        return parentFile==null?null:new Vfile(parentFile);
    }

    public Vfile[] listFiles() {
        return listFiles(null);
    }
    public Vfile[] listFiles(Filter filter) {
        final File[] ss = file.listFiles();
        if (ss == null) return null;
        ArrayList v = new ArrayList(ss.length);
        for (int i = 0 ; i < ss.length ; i++) {
            Vfile f =  new Vfile(ss[i]);
            if ((filter == null) || filter.shouldDisplay(f)) {
                v.add(f);
            }
        }
        return (Vfile[])(v.toArray(new Vfile[v.size()]));
    }

    public boolean mkdir() {
        return file.mkdir();
    }

    public boolean mkdirs() {
        return file.mkdirs();
    }

    public boolean renameTo(Vfile dest) {
        return file.renameTo(dest.file);
    }
    public void copyTo(final Vfile destFile) throws IOException {
        log.fine("copyfile src="+toString()+" dest="+destFile);
        FileChannel srcChannel = new FileInputStream(file).getChannel();
        FileChannel dstChannel = new FileOutputStream(destFile.file).getChannel();
        dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
        srcChannel.close();
        dstChannel.close();
    }

    public boolean equals(Object obj) {
        if ((obj != null) && (obj instanceof Vfile)) {
            return file.equals(((Vfile)obj).file);
        }
        return false;
        // return file.equals(obj);
    }


//    public static String getStats(){
//        TaskTimer tt = new TaskTimer();
//        StringBuffer sb = new StringBuffer();
//        sb.append(tt.formatMillis(getlength)+" in length() \n");
//        sb.append(tt.formatMillis(getname)+" in getName() \n");
//        sb.append(tt.formatMillis(getlastmodified)+" in lastModified() \n");
//        sb.append(tt.formatMillis(isdir)+" in isDirectory() \n");
//        return sb.toString();
//    }




    //query properties
    public URL toURL() throws MalformedURLException {
        return file.toURL();
    }
    public String getName() {
      //  long l1 = System.currentTimeMillis();
        final String name = file.getName();
       // getname+=(System.currentTimeMillis()-l1);
        return name;
    }
    public int hashCode() {
        return file.hashCode();
    }

    public String toString() {
        return file.toString();
    }

    public String getAbsolutePath() {
        return file.getAbsolutePath();
    }
    public boolean exists() {
        return file.exists();
    }
    private boolean isDir,dirInited;
    public boolean isDirectory() {
        if(!dirInited){
            isDir=file.isDirectory();
            dirInited=true;
        }
        return isDir;
//        long l1 = System.currentTimeMillis();
//        final boolean directory = file.isDirectory();
//        isdir+=(System.currentTimeMillis()-l1);
//        return directory;
    }

    private long lm;
    private boolean lminit;
    public long lastModified() {
        if(!lminit){
            lm=file.lastModified();
            lminit=true;
        }
        return lm;
//        long l = System.currentTimeMillis();
//        final long lm = file.lastModified();
//        getlastmodified+=(System.currentTimeMillis()-l);
//        return lm;
    }

     private long len;
    private boolean leninit;
    public long length() {
         if(!leninit){
            len=file.length();
            leninit=true;
        }
        return len;
//        long l1 = System.currentTimeMillis();
//        final long l = file.length();
//        getlength+=(System.currentTimeMillis()-l1);
//        return l;
    }

    public File getFile() {
        return file;
    }

    public static List<File> getFiles(List<Vfile> d) {
        List<File> l = new ArrayList<File>(d.size());
        for(Vfile v:d) l.add(v.getFile());
        return l;
    }

    public String getExtension() {
        if(isDirectory()) return "";
        String n = getName();
        int i = n.lastIndexOf('.');
        if(i==-1 || i==n.length()-1) return null;
        return n.substring(i+1);
    }
}
